@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2007, 2008
+ * Copyright (C) 2007, 2008, 2009
  *       pancake <youterm.com>
  *       nibble <.ds@gmail.com>
  *
@@ -32,20 +32,29 @@
 #define MAX_FUN_SIZE 1023
 
 /* code analyzer */
-int (*arch_aop)(u64 addr, const u8 *bytes, struct aop_t *aop);
+int (*arch_aop)(ut64 addr, const u8 *bytes, struct aop_t *aop);
+
+int section_is_x (ut64 addr)
+{
+	struct section_t *sec = section_get (addr);
+	// XXX: rabin -rS is necesary here
+	// We need to load the section information!!1
+//	return 1;
+	return (sec && sec->rwx & SECTION_X);
+}
 
 /* code lines */
 struct reflines_t *code_lines_init(int linescall)
 {
 	struct reflines_t *list = (struct reflines_t*)malloc(sizeof(struct reflines_t));
 	struct reflines_t *list2;
-	int bar = (int)config_get("asm.linesout");
+	int bar = PTRCAST (config_get("asm.linesout"));
 	unsigned char *ptr = config.block;
 	unsigned char *end = config.block + config.block_size;
 	struct aop_t aop;
 	int dt, sz, bsz = 0;
 	int index = 0;
-	u64 seek = 0;
+	ut64 seek = 0;
 	int lines = -1;
 
 	if (config.visual)
@@ -64,7 +73,7 @@ struct reflines_t *code_lines_init(int linescall)
 			break;
 		dt = data_type(config.seek+bsz);
 		if (dt != DATA_FUN && dt != DATA_CODE) {
-			u64 sz = data_size(config.seek+bsz);
+			ut64 sz = data_size(config.seek+bsz);
 			if (sz > 0) {
 				ptr= ptr +sz;
 				bsz=bsz+sz;
@@ -124,7 +133,7 @@ void code_lines_free(struct list_head *list)
 	free(list);
 }
 
-void code_lines_print(struct reflines_t *list, u64 addr, int expand)
+void code_lines_print(struct reflines_t *list, ut64 addr, int expand)
 {
 	struct list_head *pos;
 	int foo = config_get_i("asm.linestyle");
@@ -158,18 +167,15 @@ void code_lines_print(struct reflines_t *list, u64 addr, int expand)
 			if (!expand) {
 				if (ref->from > ref->to)
 					cons_strcat(".");
-				else
-					cons_strcat("`");
+				else cons_strcat("`");
 				ch = '-';
-			} else
-				ch = '.';
+			} else ch = '.';
 		} else
 		if (addr == ref->from) {
 			if (!expand) {
 				if (ref->from > ref->to)
 					cons_strcat("`");
-				else
-					cons_strcat(".");
+				else cons_strcat(".");
 				ch = '=';
 			}
 		} else {
@@ -179,8 +185,7 @@ void code_lines_print(struct reflines_t *list, u64 addr, int expand)
 				if (addr > ref->from && addr < ref->to) {
 					if (ch=='-'||ch=='=')
 						cons_printf("%c",ch);
-					else
-						cons_strcat("|");
+					else cons_strcat("|");
 				} else
 				if (!expand) {
 					C {
@@ -199,9 +204,7 @@ void code_lines_print(struct reflines_t *list, u64 addr, int expand)
 						cons_printf("%c", ch);
 					else // ^
 						cons_strcat("|");
-				} else {
-					cons_printf("%c",ch);
-				}
+				} else cons_printf("%c",ch);
 			}
 		}
 		if (config_get("asm.lineswide")) { // TODO: convert to integer ?
@@ -234,18 +237,18 @@ void code_lines_print(struct reflines_t *list, u64 addr, int expand)
 }
 
 /* XXX not working properly */
-int code_analyze_r_split(struct program_t *prg, u64 seek, int depth)
+int code_analyze_r_split(struct program_t *prg, ut64 seek, int depth)
 {
 	struct aop_t aop;
-	u64 oseek = seek;
-	u64 tmp = config.seek;
+	ut64 oseek = seek;
+	ut64 tmp = config.seek;
 	unsigned int sz = 0, ret;
 	int bsz = 0;// block size
 	char buf[4096]; // bytes of the code block
 	unsigned char *ptr = (unsigned char *)&buf;
 	int callblocks =(int) config_get_i("graph.callblocks");
 	//int jmpblocks = (int) config_get_i("graph.jmpblocks");
-    int refblocks = (int) config_get_i("graph.refblocks");
+    	int refblocks = (int) config_get_i("graph.refblocks");
 	struct block_t *blf = NULL;
 	
 	if (arch_aop == NULL)
@@ -264,7 +267,7 @@ int code_analyze_r_split(struct program_t *prg, u64 seek, int depth)
 
 	blf = program_block_split_new (prg, config.seek);
 	if ( blf != NULL ) {
-		eprintf("Block splitted at address 0x%08llx\n",config.seek+bsz);
+		eprintf("Block split at address 0x%08llx\n",config.seek+bsz);
 		aop.jump = blf->tnext;
 		aop.fail = blf->fnext;
 		aop.eob = 1;
@@ -280,12 +283,18 @@ int code_analyze_r_split(struct program_t *prg, u64 seek, int depth)
 			break;
 		}
 
+		if ((aop.type == AOP_TYPE_JMP)
+		|| (aop.type == AOP_TYPE_TRAP)
+		|| (aop.type == AOP_TYPE_RET)) {
+			//eprintf("GO DIE at 0x%08llx\n", seek);
+			aop.eob = 1;
+		} else
 		if (aop.type == AOP_TYPE_CALL) {
 			if (callblocks)
 				aop.eob = 1;
 			else aop.eob = 0;
 		} else
-			if (aop.type == AOP_TYPE_PUSH && aop.ref !=0) {
+			if (aop.type == AOP_TYPE_PUSH && aop.ref != 0) {
 				/* TODO : add references */
 				if (refblocks) {
 					program_block_add_call(prg, oseek, aop.ref);
@@ -307,175 +316,21 @@ int code_analyze_r_split(struct program_t *prg, u64 seek, int depth)
 		code_analyze_r_split(prg, aop.fail, depth-1);
 
 	return 0;
-
-#if 0 
-	struct aop_t aop;
-	struct block_t *blk;
-	u64 oseek = seek;
-	u64 tmp = config.seek;
-	unsigned int sz = 0, ret;
-	int bsz = 0;// block size
-	char buf[4096]; // bytes of the code block
-	unsigned char *ptr = (unsigned char *)&buf;
-	int callblocks =(int) config_get_i("graph.callblocks");
-	//int jmpblocks = (int) config_get_i("graph.jmpblocks");
-    int refblocks = (int) config_get_i("graph.refblocks");
-	struct block_t *blf = NULL;
-	
-	if (arch_aop == NULL)
-		return -1;
-	// too deep! chop branch here!
-	if (depth<=0)
-		return 0;
-	if (config.interrupted)
-		return 0;
-
-	/* if already analyzed skip */
-	if (program_block_get(prg,seek))
-		return 0;
-
-	radare_seek(tmp, SEEK_SET);
-	bsz = 0;
-	config.seek = seek;
-	radare_read(0);
-	aop.eob = 0;
-
-	ret = radare_read(0);
-
-	/* Walk for all bytes of current block */
-	for(bsz = 0;(!aop.eob) && (bsz+4 <config.block_size); bsz+=sz) {
-		if (config.interrupted)
-			break;
-
-		/// Miro si l'adreca on soc es base d'algun bloc
-		//blf = program_block_get ( prg , config.seek+bsz );
-
-		sz = arch_aop(config.seek+bsz, config.block+bsz, &aop);
-		if (sz<=0) {
-			//eprintf("Invalid opcode (%02x %02x)\n", config.block[0], config.block[1]);
-			// eprintf("x");
-			//D analyze_progress(0,1,0,0);
-			break;
-		}
-#if 1
-#if 0
-		/* splitting code lives here */
-		if ( blf != NULL ) {	
-			//printf ("Address %llx already analed\n", config.seek+bsz );
-			aop.eob = 1;
-			aop.jump = blf->tnext; //config.seek+bsz;
-			aop.fail = blf->fnext;
-//printf("POLLA EN VINAGRE RULES\n");
-			break;
-		}
-#endif
-		blf = program_block_split_new (prg, config.seek+bsz);
-		if ( blf != NULL ) {
-			eprintf("Block splitted at address 0x%08llx\n", config.vaddr+config.seek+bsz);
-			
-			bsz = blf->n_bytes;
-			aop.eob = 1;
-			if (blf->tnext)
-				aop.jump = blf->tnext;
-			if (blf->fnext)
-				aop.fail = blf->fnext;
-			break;
-		}		
-#endif
-
-		/* continue normal analysis */
-#if 0 
-		if (!callblocks && aop.type == AOP_TYPE_CALL) {
-			program_block_add_call(prg, oseek, aop.jump);
-                	if (callblocks)
-				aop.eob = 1;
-			else aop.eob = 0;
-		}
-		if (!jmpblocks && (aop.type == AOP_TYPE_JMP || aop.type == AOP_TYPE_CJMP))
-			aop.eob = 0;
-
-		switch(aop.type) {
-		case AOP_TYPE_JMP:
-		case AOP_TYPE_CJMP:
-		case AOP_TYPE_CALL:
-			program_block_add_call(prg, oseek, aop.jump);
-			break;
-		case AOP_TYPE_PUSH:
-			/* TODO : add references */
-			if (config_get("graph.refblocks"))
-				program_block_add_call(prg, oseek, aop.ref);
-			break;
-		}
-
-		memcpy(ptr+bsz, config.block+bsz, sz); // append bytes
-	}
-#else
-		if (aop.type == AOP_TYPE_CALL) {
-			program_block_add_call(prg, oseek, aop.jump);
-			if (callblocks)
-				aop.eob = 1;
-			else aop.eob = 0;
-		} else
-			if (aop.type == AOP_TYPE_PUSH && aop.ref !=0) {
-				/* TODO : add references */
-				if (refblocks) {
-					program_block_add_call(prg, oseek, aop.ref);
-					aop.eob = 1;
-					aop.jump = aop.ref;
-					aop.fail = oseek+bsz+sz;
-				}
-				//block_add_ref(prg, oseek, aop.ref);
-				//block_add_call(prg, oseek, aop.ref);
-			}
-		memcpy(ptr+bsz, config.block+bsz, sz); // append bytes
-	}
-#endif
-	config.seek = tmp;
-
-	blk = program_block_get_new(prg, oseek);
-
-	blk->bytes = (unsigned char *)malloc(bsz+1);
-	if (blk->bytes == NULL) {
-		eprintf("analyze.c: Cannot allocate\n");
-	}
-	blk->n_bytes = bsz;
-	memcpy(blk->bytes, buf, bsz);
-	blk->tnext = aop.jump;
-	blk->fnext = aop.fail;
-	
-	blk->type = BLK_TYPE_HEAD;
-	if (aop.jump && !aop.fail)
-		blk->type = BLK_TYPE_BODY;
-	else
-	if (aop.jump && aop.fail)
-		blk->type = BLK_TYPE_BODY;
-	else
-	if (aop.type == AOP_TYPE_RET)
-		blk->type = BLK_TYPE_LAST;
-
-	oseek = seek;
-
-	/* walk childs */
-	if (blk->tnext && (blf == NULL) )
-		code_analyze_r_split(prg, blk->tnext, depth-1);
-	if (blk->fnext  )
-		code_analyze_r_split(prg, blk->fnext, depth-1);
-
-	return 0;
-#endif 
 }
 
-int code_analyze_r_nosplit(struct program_t *prg, u64 seek, int depth)
+int code_analyze_r_nosplit(struct program_t *prg, ut64 seek, int depth)
 {
 	struct aop_t aop;
 	struct block_t *blk;
-	u64 oseek = seek;
-	u64 tmp = config.seek;
+	ut64 oseek = seek;
+	ut64 tmp = config.seek;
 	unsigned int sz = 0, ret;
 	int bsz = 0;// block size
 	char buf[4096]; // bytes of the code block
 	unsigned char *ptr = (unsigned char *)&buf;
+	// TODO: make those vars global.. its slow to have config_* functions in a recursive fun
 	int callblocks = (int)config_get_i("graph.callblocks");
+	//int pushblocks = (int)config_get_i("anal.push");
 	//int jmpblocks = (int) config_get_i("graph.jmpblocks");
 	int refblocks = (int)config_get_i("graph.refblocks");
 
@@ -505,27 +360,32 @@ int code_analyze_r_nosplit(struct program_t *prg, u64 seek, int depth)
 
 		sz = arch_aop(config.seek+bsz, config.block+bsz, &aop);
 		if (sz<=0) {
-			//                        eprintf("Invalid opcode (%02x %02x)\n", config.block[0], config.block[1]);
+			// eprintf("Invalid opcode (%02x %02x)\n", config.block[0], config.block[1]);
 			break;
 		}
 
+		if (aop.type == AOP_TYPE_JMP) {
+			//eprintf("GO DIE at 0x%08llx\n", seek);
+			aop.eob = 1;
+		} else
 		if (aop.type == AOP_TYPE_CALL) {
 			program_block_add_call(prg, oseek, aop.jump);
+
 			if (callblocks)
 				aop.eob = 1;
 			else aop.eob = 0;
 		} else
-			if (aop.type == AOP_TYPE_PUSH && aop.ref !=0) {
-				/* TODO : add references */
-				if (refblocks) {
-					program_block_add_call(prg, oseek, aop.ref);
-					aop.eob = 1;
-					aop.jump = aop.ref;
-					aop.fail = oseek+bsz+sz;
-				}
-				//block_add_ref(prg, oseek, aop.ref);
-				//block_add_call(prg, oseek, aop.ref);
+		if (aop.type == AOP_TYPE_PUSH && aop.ref != 0) {
+			/* TODO : add references */
+			if (refblocks) {
+				program_block_add_call(prg, oseek, aop.ref);
+				aop.eob = 1;
+				aop.jump = aop.ref;
+				aop.fail = oseek+bsz+sz;
 			}
+			//block_add_ref(prg, oseek, aop.ref);
+			//block_add_call(prg, oseek, aop.ref);
+		}
 		memcpy(ptr+bsz, config.block+bsz, sz); // append bytes
 	}
 	config.seek = tmp;
@@ -535,6 +395,7 @@ int code_analyze_r_nosplit(struct program_t *prg, u64 seek, int depth)
 	blk->bytes = (unsigned char *)malloc(bsz+1);
 	if (blk->bytes == NULL) {
 		eprintf("analyze.c: Cannot allocate\n");
+		return 0;
 	}
 	blk->n_bytes = bsz;
 	memcpy(blk->bytes, buf, bsz);
@@ -557,7 +418,6 @@ int code_analyze_r_nosplit(struct program_t *prg, u64 seek, int depth)
 		code_analyze_r_nosplit(prg, blk->tnext, depth-1);
 	if (blk->fnext)
 		code_analyze_r_nosplit(prg, blk->fnext, depth-1);
-
 	return 0;
 }
 
@@ -589,11 +449,11 @@ void analyze_spcc(const char *name)
 }
 
 /* CALLBACK defined with graph.split which is false by default */
-int (*code_analyze_r)(struct program_t *prg, u64 seek, int depth) = &code_analyze_r_nosplit;
+int (*code_analyze_r)(struct program_t *prg, ut64 seek, int depth) = &code_analyze_r_nosplit;
 
-struct program_t *code_analyze(u64 seek, int depth)
+struct program_t *code_analyze(ut64 seek, int depth)
 {
-	u64 bsize = config.block_size;
+	ut64 bsize = config.block_size;
 	struct program_t *prg = program_new(NULL);
 
 	if (prg == NULL)
@@ -606,23 +466,11 @@ struct program_t *code_analyze(u64 seek, int depth)
 
 	radare_controlc();
 
-#if 0 
-	if (config_get("graph.split"))
-		code_analyze_r = &code_analyze_r_split;
-	else	code_analyze_r = &code_analyze_r_nosplit;
-
-	/* XXX fix hirroble bug in deep overflow :O */
-	if (depth>10) depth=10;
-	//if (depth>10) depth=10;
-
-	code_analyze_r(prg, seek, --depth);
-#endif 
 	if (depth>30) depth=30;
 	if (config_get("graph.split")) {
 		code_analyze_r_nosplit(prg, seek, --depth);
 		code_analyze_r_split(prg, seek, --depth);
-	} else
-		code_analyze_r_nosplit(prg, seek, --depth);
+	} else code_analyze_r_nosplit(prg, seek, --depth);
 
 	// TODO: construct xrefs from tnext/fnext info
 	radare_controlc_end();
@@ -650,13 +498,13 @@ TODO: use maps here! must be mixed with flags and so
 
 #endif
 
-int radare_analyze(u64 seek, int size, int depth, int rad)
+int radare_analyze(ut64 seek, int size, int depth, int rad)
 {
 	char cmd[1024];
 	char str[1024];
 	u8 word[128];
-	u64 tmp = config.seek;
-	u32 num, nume; // little endian
+	ut64 tmp = config.seek;
+	ut32 num, nume; // little endian
 	int count=0;
 	int i;
 	int str_i=0;
@@ -693,13 +541,13 @@ int radare_analyze(u64 seek, int size, int depth, int rad)
 		if (str_i>2) {
 			str[str_i] = '\0';
 			if (rad) {
-				u64 addr = (u64)(seek+i-str_i);
+				ut64 addr = (ut64)(seek+i-str_i);
 				cons_printf("Cs %d @ 0x%08llx\n", strlen(str)+1, addr);
 				flag_filter_name(str);
 				cons_printf("f str.%s @ 0x%08llx\n", str, addr);
 				//cons_printf("; TODO (if exists) f str_%s\n", str);
 			} else {
-				print_addr((u64)(seek+i-str_i));
+				print_addr((ut64)(seek+i-str_i));
 				cons_strcat("   ");
 				C	cons_printf("string "C_BYELLOW"\"%s\""C_RESET"\n", str);
 				else	cons_printf("string \"%s\"\n", str);
@@ -742,33 +590,32 @@ int radare_analyze(u64 seek, int size, int depth, int rad)
 				//cons_printf("0xffffffff (-1)\n");
 			} else {
 				if (rad) {
-					u32 n = (config.endian)?num:nume;
+					ut32 n = (config.endian)?num:nume;
 					str[0]='\0';
-					string_flag_offset(str, (u64)n, 0);
+					string_flag_offset(NULL, str, (ut64)n, 0);
 					if (!strnull(str)) {
 						/* reference by pointer */
-						cons_printf("Cx 0x%08llx @ 0x%08llx ; %s\n", (u64)(seek+i-3), (u64)n, str);
+						cons_printf("Cx 0x%08llx @ 0x%08llx ; %s\n", (ut64)(seek+i-3), (ut64)n, str);
 					} else
-					if (n == (u32)seek)
+					if (n == (ut32)seek)
 						cons_printf(" ;  (self pointer)\n");
 					else radare_analyze(n, size, --depth, rad);
 				} else {
 					if (lastnull>1)
 						cons_printf("(last null repeated %d times)\n", lastnull);
 					lastnull = 0;
-				//	for(j=config_get_i("cfg.analdepth"); j>depth;j--)
+				//	for(j=config_get_i("anal.depth"); j>depth;j--)
 				//		cons_strcat("   ");
 					print_addr(seek+i-3);
 					C {
-					C cons_printf(C_TURQOISE);
+						cons_printf(C_TURQOISE);
 						if (config.endian)
 						cons_printf("int be="C_YELLOW"0x%08x"C_RESET" le=0x%08x ",
 							num, nume);
 						else
 						cons_printf("int be=0x%08x le="C_YELLOW"0x%08x"C_RESET" ",
 							num, nume);
-					} else
-						cons_printf("int be=0x%08x le=0x%08x ",
+					} else cons_printf("int be=0x%08x le=0x%08x ",
 							num, nume);
 					if (num<0xffff)
 						cons_printf("(be= %d )", num);
@@ -780,12 +627,12 @@ int radare_analyze(u64 seek, int size, int depth, int rad)
 					if (nume>-0xfffff && nume<0xfffff)
 						cons_printf("(%d)\n", nume);
 					else {
-						u32 n = (config.endian)?num:nume;
+						ut32 n = (config.endian)?num:nume;
 						C cons_printf(C_TURQOISE);
 						sprintf(cmd, ":fd @0x%08x", n);
 						radare_cmd(cmd, 0);
 
-						if (n == (u32)seek)
+						if (n == (ut32)seek)
 							cons_printf("  (self pointer)\n");
 						else radare_analyze(n, size, --depth, rad);
 
@@ -803,8 +650,6 @@ int radare_analyze(u64 seek, int size, int depth, int rad)
 	/* restore */
 	config.seek = tmp;
 	radare_read(0);
-	//if (!rad)
-	//D cons_strcat("\n");
 	config.verbose = v;
 
 	return 0;
@@ -862,11 +707,11 @@ int analyze_var_get(int type)
 	return ctr;
 }
 
-int analyze_progress(int _o, int _x, int _p, int _v)
+void analyze_progress(int _o, int _x, int _p, int _v)
 {
 	char buf[80];
 	int i, j = 0;
-	static int refresh = 0;
+	//static int refresh = 0;
 	static int o=0,x=0,p=0,v=0;
 	static int ch = '/';
 	static int lastlen = 0;
@@ -897,30 +742,31 @@ int analyze_progress(int _o, int _x, int _p, int _v)
 
 int analysis_interrupted = 0;
 
-int analyze_function(u64 from, int recursive, int report)
+int analyze_function(ut64 from, int recursive, int report)
 {
 	struct aop_t aop;
 	struct list_head *head;
 	struct block_t *b0;
 	struct program_t *prg;
 	int ret;
-	char buf[1024];
-	static int interrupted = 0;
+	char buf[4096];
+	//static int interrupted = 0;
 	/*--*/
-	u8 *bytes;
-	//u64 from = config.vaddr + config.seek;
-	u64 seek = from; // to place comments
-	u64 end  = 0;
+	ut8 *bytes;
+	//ut64 from = config.vaddr + config.seek;
+	ut64 seek = from; // to place comments
+	ut64 end  = 0;
 	int i, inc = 0;
-	u64 to;
-	u64 funsize;
+	ut64 to;
+	ut64 funsize;
 	int len =0;
-	int ref;
+	int ref, count;
 	int ncalls = 0;
 	int nrefs = 0;
 	int framesize = 0;
 	int nblocks = 0;
 	char tmpstr[16], fszstr[256];
+ut64 addr= seek;
 
 	if (recursive <0)
 		return -1;
@@ -928,7 +774,7 @@ int analyze_function(u64 from, int recursive, int report)
 		return -1;
 
 	from += config.vaddr-config.paddr;
-//eprintf("ANAL FROM (%llx)\n", from);
+
 	if (arch_aop == NULL)
 		return -1;
 	fszstr[0]='\0';
@@ -962,7 +808,7 @@ int analyze_function(u64 from, int recursive, int report)
 		nblocks++;
 	}
 	to = end;
-	len=(int)(1+to-seek); //from;
+	len = (int)(1+to-seek); //from;
 	if (len<0)
 		return -1;
 
@@ -992,7 +838,7 @@ int analyze_function(u64 from, int recursive, int report)
 		break;
 	case 1:
 		buf[0]='\0';
-		string_flag_offset(buf, from, 0);
+		string_flag_offset(NULL, buf, from, 0);
 		cons_printf("offset = 0x%08llx\n", from);
 		cons_printf("label = %s\n", buf);
 		cons_printf("size = %lld\n", to-seek);
@@ -1016,11 +862,9 @@ int analyze_function(u64 from, int recursive, int report)
 	//D eprintf(".");
 	D analyze_progress(0,0,1,0);
 
-//eprintf("LEN=%d\n", len);
-	for(;seek< to; seek+=inc) {
-		u64 delta = seek+config.vaddr-from;
-	//eprintf("0x%08llx\n", seek+config.vaddr);
-		if (delta > len) {
+	for(;seek < to; seek+=inc) {
+		ut64 delta = seek+config.vaddr-from;
+		if (delta >= len) {
 			//eprintf("analyze_function: oob %lld > %lld\n", delta, len);
 			break;
 		}
@@ -1030,6 +874,9 @@ int analyze_function(u64 from, int recursive, int report)
 			break;
 		}
 		switch(aop.type) {
+		case AOP_TYPE_PUSH:
+			analyze_function(aop.ref-config.vaddr, recursive-1, report);
+			break;
 		case AOP_TYPE_CALL:
 			switch(report) {
 			case 2:
@@ -1037,11 +884,11 @@ int analyze_function(u64 from, int recursive, int report)
 				break;
 			case 0:
 				buf[0]='\0';
-				string_flag_offset(buf, aop.jump, 0);
+				string_flag_offset(NULL, buf, aop.jump, 0);
 				// if resolved as sym_ add its call
 				cons_printf("Cx 0x%08llx @ 0x%08llx ; %s\n", aop.jump, seek+config.vaddr, buf);
 			}
-			analyze_function(aop.jump-config.vaddr, --recursive, report);
+			analyze_function(aop.jump-config.vaddr, recursive-1, report);
 			ncalls++;
 			break;
 		case AOP_TYPE_SWI:
@@ -1049,6 +896,7 @@ int analyze_function(u64 from, int recursive, int report)
 			case 2: cons_printf("CC -syscall %s @ 0x%08llx\n", "(todo)", seek);
 				break;
 			case 0: cons_printf("CC syscall %s @ 0x%08llx\n", "(todo)", seek);
+				break;
 			}
 			break;
 		default:
@@ -1064,10 +912,10 @@ int analyze_function(u64 from, int recursive, int report)
 					break;
 				case 0:
 					buf[0]='\0';
-					string_flag_offset(buf, aop.jump, 0);
+					string_flag_offset(NULL, buf, aop.jump, 0);
 					// if resolved as sym_ add its call
 					cons_printf("CX 0x%08llx @ 0x%08llx ; %s\n",
-						seek, (u64)aop.ref, buf);
+						seek, (ut64)aop.ref, buf);
 				}
 				nrefs++;
 			}
@@ -1078,7 +926,7 @@ int analyze_function(u64 from, int recursive, int report)
 		switch(aop.stackop) {
 		case AOP_STACK_LOCAL_SET:
 			if (report == 2) {
-				sprintf(buf, "CC -* @ 0x%08llx\n", ref, seek);
+				sprintf(buf, "CC -* @ 0x%08llx\n", seek);
 				cons_strcat(buf);
 			} else
 			if (!report) {
@@ -1168,17 +1016,21 @@ int analyze_function(u64 from, int recursive, int report)
 
 		/* recursivity */
 		if (recursive) {
-			recursive--;
 			switch(aop.type) {
-	#if 0
+			case AOP_TYPE_RET:
+			case AOP_TYPE_TRAP:
+				seek = to;
+				break;
+	#if 1
 			case AOP_TYPE_JMP: // considered as new function
 				radare_seek(aop.jump, SEEK_SET);
-				analyze_function(1);
+				//analyze_function(1);
+				analyze_function(seek-config.vaddr, recursive-1, report);
 				break;
 	#endif
 			case AOP_TYPE_CALL: // considered as new function
 				radare_seek(aop.jump, SEEK_SET);
-				analyze_function(seek-config.vaddr, recursive, report);
+				analyze_function(seek-config.vaddr, recursive-1, report);
 				break;
 			}
 		}
@@ -1188,10 +1040,12 @@ int analyze_function(u64 from, int recursive, int report)
 	switch(report) {
 	case 0:
 		if (fszstr[0])
-			cons_printf("CC framesize =%s @ 0x%08llx\n", fszstr, from);
-		cons_printf("CC args = %d @ 0x%08llx\n", analyze_var_get(VAR_TYPE_ARG), from);
-		cons_printf("CC vars = %d @ 0x%08llx\n", analyze_var_get(VAR_TYPE_LOCAL), from);
-		cons_printf("CC drefs = %d @ 0x%08llx\n", nrefs);
+			cons_printf("CC framesize = %s @ 0x%08llx\n", fszstr, from);
+		count = analyze_var_get(VAR_TYPE_ARG);
+		if (count>0) cons_printf("CC args = %d @ 0x%08llx\n", count, from);
+		count = analyze_var_get(VAR_TYPE_LOCAL);
+		if (count>0) cons_printf("CC vars = %d @ 0x%08llx\n", count, from);
+		if (nrefs>0) cons_printf("CC drefs = %d @ 0x%08llx\n", nrefs);
 		cons_printf("fs *\n");
 		break;
 	case 1:
@@ -1212,3 +1066,43 @@ int analyze_function(u64 from, int recursive, int report)
 
 	return 0;
 }
+
+void analyze_preludes (char *input) {
+	char cmd[1024];
+	char *prelude;
+	long len;
+	const char *sfrom, *sto, *cmdhit;
+
+	switch(config.arch) {
+	case ARCH_X86: // X86-64 is implicit here
+		prelude = "5589e5";
+		break;
+	default:
+		eprintf ("Analyze preludes (ap) not supported for this architecture\n");
+		return;
+	}
+	sfrom = config_get("search.from");
+	sto   = config_get("search.to");
+	cmdhit= config_get("cmd.hit");
+
+	len = get_math (input);
+	if (len <1) {
+		len = config.size;
+		if (len <0) {
+			eprintf ("Dunno filesize..gimme some ranges\n");
+			return;
+		}
+	}
+	/* go search */
+	config_set_i("search.from", config.seek);
+	config_set_i("search.to", config.seek+ len);
+	config_set("cmd.hit", ".af*");
+	sprintf(cmd, "/x %s", prelude);
+	radare_cmd(cmd, 0);
+
+	/* restore */
+	config_set("search.from", sfrom);
+	config_set("search.to", sto);
+	config_set("cmd.hit", cmdhit);
+	radare_cmd("f-hit*", 0);
+}
