@@ -56,7 +56,9 @@
 static int current_offset = 0;
 int color;
 
+#undef TRUE
 #define TRUE (0==0)
+#undef FALSE
 #define FALSE (0==1)
 
 #define DISARM_COMMENTPOS 32
@@ -583,7 +585,7 @@ static CondPosT decode_branch (Branch branch)
 	addr+=8;
 
 	label[0]='\0';
-	string_flag_offset(label, current_offset+addr, 0);
+	string_flag_offset(NULL, label, current_offset+addr, 0);
 	sprintf(disasmstr, "%sb%s  0x%x ; %s%s",
 			(color)?"\x1b[32m":"",
 			condtext[(bytes->byte3&0xf0)>>4][0],
