@@ -26,7 +26,7 @@
 #include <string.h>
 
 // NOTE: bytes should be at least 16 bytes?
-int arch_m68k_aop(u64 addr, const unsigned char *bytes, struct aop_t *aop)
+int arch_m68k_aop(ut64 addr, const unsigned char *bytes, struct aop_t *aop)
 {
 
 	return 4;
