@@ -25,7 +25,6 @@
 //#include <dir.h>
 #include <math.h>
 #include <float.h>
-#pragma hdrstop
 
 #include "disasm.h"
 
@@ -80,7 +79,7 @@ static int       nresult;              // Current length of disassembly
 static int       addcomment;           // Comment value of operand
 
 // Copy of input parameters of function Disasm()
-static char      *cmd;                 // Pointer to binary data
+static unsigned char      *cmd;                 // Pointer to binary data
 static char      *pfixup;              // Pointer to possible fixups or NULL
 static ulong     size;                 // Remaining size of the command buffer
 static t_disasm  *da;                  // Pointer to disassembly results
@@ -655,7 +654,7 @@ static void DecodeJF(void) {
   if (addr==0 || seg==0) da->zeroconst=1;
   if (mode>=DISASM_FILE) {
     nresult+=sprintf(da->result+nresult,"%s 0x%04X:0x%08X",
-    (lowercase==0?"FAR":"far"),seg,addr);
+    (lowercase==0?"FAR":"far"), (unsigned int)seg, (unsigned int)addr);
   };
 };
 
