@@ -58,11 +58,11 @@ extern void show_help_message();
 /* Call a command from the command table */
 int commands_parse (const char *_cmdline)
 {
+	// XXX making non-const a const ptr??
 	char *cmdline = (char *)_cmdline;
 	command_t *cmd = NULL;
 
-	for(;cmdline[0]&&(cmdline[0]=='\x1b'||cmdline[0]==' ');
-	cmdline=cmdline+1);
+	for(;cmdline[0]&&(cmdline[0]=='\x1b'||cmdline[0]==' ');cmdline=cmdline+1);
 
 	// null string or comment
 	if (cmdline[0]==0||cmdline[0]=='#'||cmdline[0]==';')
@@ -72,11 +72,7 @@ int commands_parse (const char *_cmdline)
 		if (cmd->sname == cmdline[0])
 		    return cmd->hook(cmdline+1);
 	}
-
-	/* default hook */
-	if (cmd)
-		return cmd->hook(cmdline);
-	return -1;
+	return cmd?cmd->hook(cmdline):-1;
 }
 
 void show_help_message()
@@ -100,14 +96,6 @@ void show_help_message()
 	cons_printf(" ?[?]<expr>        calc     evaluate math expression\n");
 }
 
-	//COMMAND('c', " [times]",       "count   limit of search hits and 'w'rite loops", count),
-	//COMMAND('e', " [0|1]",       "endian  change endian mode (0=little, 1=big)", endianess),
-	//COMMAND('g', " [x,y]",         "gotoxy  move screen cursor to x,y", gotoxy),
-	//COMMAND('l', " [offset]",      "limit   set the top limit for searches", limit),
-	//COMMAND('Y', " [length]",      "Ypaste  copy n bytes from clipboard to cursor", yank_paste),
-	//COMMAND('_', " [oneliner]",    "perl/py interpret a perl/python script (use #!perl)", interpret_perl),
-	//COMMAND('%', "ENVVAR [value]", "setenv  gets or sets a environment variable", envvar),
-
 command_t commands[] = {
 	COMMAND('a', "[ocdgv?]",       "analyze  perform code/data analysis", analyze),
 	COMMAND('b', " [blocksize]",   "bsize    change the block size", blocksize),
@@ -118,7 +106,6 @@ command_t commands[] = {
 	COMMAND('f', "[crogd|-][name]","flag     flag the current offset (f? for help)", flag),
 	COMMAND('H', " [cmd]",         "Hack     performs a hack", hack),
 	COMMAND('i', "",               "info     prints status information", info),
-	//COMMAND('m', " [size] [dst]",  "move     copy size bytes from here to dst", move),
 	COMMAND('o', " [file]",        "open     open file", open),
 	COMMAND('p', "[fmt] [len]",    "print    print data block", print),
 	COMMAND('q', "[!]",            "quit     close radare shell", quit),
@@ -148,7 +135,7 @@ command_t commands[] = {
 
 CMD_DECL(macro)
 {
-	switch(input[0]) {
+	switch (input[0]) {
 	case ')':
 		radare_macro_break(input+1);
 		break;
@@ -178,6 +165,7 @@ CMD_DECL(macro)
 		radare_macro_add(input);
 		break;
 	}
+	return 0;
 }
 
 CMD_DECL(config_eval)
@@ -200,7 +188,6 @@ CMD_DECL(config_eval)
 	default:
 		config_eval(input+i);
 	}
-
 	return 0;
 }
 
@@ -212,12 +199,13 @@ CMD_DECL(analyze)
 	struct list_head *head;
 	struct list_head *head2;
 	struct xrefs_t *c0;
+	char *file;
 	const char *ptr;
 	int i, sz, n_calls=0;
 	int depth_i;
 	int delta = 0;
 	int depth = input[0]?atoi(input+1):0;
-	u64 oseek = config.seek;
+	ut64 oseek = config.seek;
 
 	if (input[0]=='c'&&depth<1) // only for 'ac'
 		depth = config_get_i("graph.depth");
@@ -237,7 +225,7 @@ CMD_DECL(analyze)
 			" at+ [addr] [times] ; add trace for address N times\n"
 			" at [addr]          ; show trace info at address\n"
 			" att [tag]          ; select trace tag (no arg unsets)\n"
-			" at%%                ; TODO\n"
+			" at%                ; TODO\n"
 			" atr                ; show traces as range commands (ar+)\n"
 			" atd                ; show disassembly trace\n"
 			" atD                ; show dwarf trace (at*|rsc dwarf-traces $FILE)\n");
@@ -245,9 +233,8 @@ CMD_DECL(analyze)
 			break;
 		case 't':
 			sz = atoi(input+2);
-			if (sz>0) {
-				trace_tag_set(sz-1);
-			} else trace_tag_set(-1);
+			if (sz>0) trace_tag_set(sz-1);
+			else trace_tag_set(-1);
 			break;
 		case 'd':
 			trace_show(2, trace_tag_get());
@@ -257,7 +244,7 @@ CMD_DECL(analyze)
 			break;
 		case '+':
 			ptr = input+3;
-			u64 addr = get_offset(input+3);
+			ut64 addr = get_offset(input+3);
 			ptr = strchr(ptr, ' ');
 			if (ptr != NULL) {
 				//eprintf("at(0x%08llx)=%d (%s)\n", addr, atoi(ptr+1), ptr+1);
@@ -269,15 +256,15 @@ CMD_DECL(analyze)
 			trace_reset();
 			break;
 		case ' ': {
-			u64 foo = get_math(input+1);
-			struct trace_t *t = (struct trace_t *)trace_get(foo, trace_tag_get());
+			struct trace_t *t = (struct trace_t *)trace_get(get_math (input+1), trace_tag_get());
 			if (t != NULL) {
 				cons_printf("offset = 0x%llx\n", t->addr);
 				cons_printf("opsize = %d\n", t->opsize);
 				cons_printf("times = %d\n", t->times);
 				cons_printf("count = %d\n", t->count);
 				//TODO cons_printf("time = %d\n", t->tm);
-			} } break;
+			} }
+			break;
 		case '*':
 			trace_show(1, trace_tag_get());
 			break;
@@ -290,7 +277,7 @@ CMD_DECL(analyze)
 		break;
 	case 'd':
 		// XXX do not uses depth...ignore analdepth?
-		radare_analyze(config.seek, config.block_size, config_get_i("cfg.analdepth"), input[1]=='*');
+		radare_analyze(config.seek, config.block_size, config_get_i("anal.depth"), input[1]=='*');
 		break;
 #if 0
 	/* TODO: reset analyze engine ? - remove rdbs, xrefs, etc...  reset level as argument? maybe cool 0 for just vm, 1 for rdbs, 2 for xrefs */
@@ -323,11 +310,6 @@ CMD_DECL(analyze)
 			}
 		} else eprintf("oops\n");
 		break;
-#if 0
-	case 'F':
-		analyze_function(config.seek, (int)config_get_i("cfg.analdepth"), 0); // XXX move to afr ?!?
-		break;
-#endif
 	case 'f':
 		switch(input[1]) {
 		case '?':
@@ -338,16 +320,16 @@ CMD_DECL(analyze)
 			eprintf(" .af*       - import function analysis (same as Vdf)\n");
 			break;
 		case 'r':
-			analyze_function(config.seek, config_get_i("cfg.analdepth"),0);
+			analyze_function(config.seek, config_get_i("anal.depth"), 0);
 			break;
 		case 'u':
-			analyze_function(config.seek, 0,2);
+			analyze_function(config.seek, config_get_i("anal.depth"), 2);
 			break;
 		case '*':
-			analyze_function(config.seek, 0,0);
+			analyze_function(config.seek, config_get_i("anal.depth"), 0);
 			break;
 		default:
-			analyze_function(config.seek, 0,1);
+			analyze_function(config.seek, 0, 1);
 		}
 		break;
 	case 'g':
@@ -361,7 +343,7 @@ CMD_DECL(analyze)
 			eprintf(" agdv [file] - use agd > file.dot && !!dot -Tpng -ofile.png file.dot && rsc view file.png\n");
 			break;
 		case 'd':
-			{ char *file = strchr(input+2, ' ');
+			file = strchr(input+2, ' ');
 			if (input[2]=='v') {
 				char *oprof = strdup(config_get("asm.profile"));
 				int ocolor = config_get_i("scr.color");
@@ -400,21 +382,49 @@ CMD_DECL(analyze)
 					graph_viz(prg, input[2]!='*');
 				}
 			}
-			}
 			break;
 		default:
 #if HAVE_GUI
-		// use graph.depth by default if not set
-		config.graph = 1;
-		prg = code_analyze(config.vaddr + config.seek, depth ); //config_get_i("graph.depth"));
-		list_add_tail(&prg->list, &config.rdbs);
-		grava_program_graph(prg, NULL);
-		config.graph = 0;
+			// use graph.depth by default if not set
+			config.graph = 1;
+			prg = code_analyze(config.vaddr + config.seek, depth ); //config_get_i("graph.depth"));
+			list_add_tail(&prg->list, &config.rdbs);
+			grava_program_graph(prg, NULL);
+			config.graph = 0;
 #else
-		eprintf("Compiled without gui. Try with ag*\n");
+			eprintf("Compiled without gui. Try with ag*\n");
 #endif
 		}
 		break;
+	case 'C':
+		{
+			char *b, *str;
+			ut64 seek = config.seek;
+			ut32 len, i = 0;
+			if (input[1]=='f'){
+ 				// TODO: use get_math(input+2) ???
+				struct data_t *data = data_get(config.seek);
+				if (data) len = data->size;
+				else len = 0;
+			} else len = (ut32)get_math(input+1);
+			if (len<1) len = config.block_size;
+			b = malloc(len);
+			radare_read_at(seek, b, len);
+			while (i<len) {
+				if (arch_aop(seek+i, b+i, &aop)<1)
+					break;
+				if (aop.type == AOP_TYPE_CALL) {
+					const char *str = flag_name_by_offset(aop.jump);
+					cons_printf("call 0x%08llx", aop.jump);
+					if (str&&*str)
+						cons_printf(" ; %s\n", str);
+					cons_printf("\n");
+				}
+				i += aop.length;
+			}
+			free(b);
+		}
+		break;
 	case 'c': {
 		int c = config.verbose;
 		char cmd[1024];
@@ -441,11 +451,11 @@ CMD_DECL(analyze)
 			}
 			i++;
 		}
-				config.verbose=c;
+		config.verbose=c;
 		} break;
 	case 'm':
 		input = input+1;
-		while(input[0]==' ')input=input+1;
+		while(input[0]==' ') input=input+1;
 		if (strchr(input,'-')) {
 			print_mem_del(input+1);
 		} else
@@ -454,7 +464,8 @@ CMD_DECL(analyze)
 			" am          ; list all memory formats registered\n"
 			" am foo xxd  ; add a named memory format\n"
 			" am -foo     ; delete named memory foo\n"
-			" pm x[foo]x  ; use the 'foo' named memory format from 'pm'\n");
+			" pm x[foo]x  ; use the 'foo' named memory format from 'pm'\n"
+			" pm x[*foo]x ; use the 'foo' (follow pointer) named memory format from 'pm'\n");
 		} else
 		if (input[0]=='\0') {
 			print_mem_list();
@@ -465,11 +476,8 @@ CMD_DECL(analyze)
 				print_mem_add(input, s+1);
 			} else {
 				const char *fmt = print_mem_get(input);
-				if (fmt) {
-					print_mem(config.seek, config.block, config.block_size, fmt, config.endian);
-				} else {
-					print_mem_add(NULL, NULL);
-				}
+				if (fmt) print_mem(config.seek, config.block,
+					config.block_size, fmt, config.endian);
 			}
 		}
 		break;
@@ -578,6 +586,9 @@ CMD_DECL(analyze)
 			config.seek += sz;
 		}
 		break;
+	case 'p':
+		analyze_preludes(input+1);
+		break;
 	case 'v':
 		switch(input[1]) {
 		case 'e':
@@ -663,8 +674,8 @@ CMD_DECL(analyze)
 	case 'x': {
 		char buf[4096];
 		char file[1024];
-		u64 seek = config.seek;
-		u64 base = 0;
+		ut64 seek = config.seek;
+		ut64 base = 0;
 		strcpy(file, config.file);
 #if DEBUGGER
 		if (config.debug) {
@@ -676,15 +687,13 @@ CMD_DECL(analyze)
 #endif
 		{
 		char buf[4096];
-			
 		sprintf(buf,"xrefs -a %s -b %lld %s %lld",
 			config_get("asm.arch"), base, file, seek);
 		radare_system(buf);
 		}
 #if DEBUGGER
-		if (config.debug) {
+		if (config.debug)
 			unlink(file);
-		}
 #endif
 		} break;
 	default:
@@ -693,9 +702,11 @@ CMD_DECL(analyze)
 		cons_printf(" aos [num]    show size of N opcodes\n");
 		cons_printf(" am [n] [fmt] analyze memory format manager (am? and pm? fmi)\n");
 		cons_printf(" ab [num]     analyze N code blocks\n");
+		cons_printf(" ap [size]    analyze function preludes from here to <size> (default filesize)\n");
 		cons_printf(" af [size]    analyze function\n");
 		//cons_printf(" aF [size]    analyze function (recursively)\n");
 		cons_printf(" ac [num]     disasm and analyze N code blocks\n");
+		cons_printf(" aC[f] [num]  analyze calls of N bytes or 'f'unction size\n");
 		cons_printf(" ad [num]     analyze N data blocks \n");
 		cons_printf(" ag [depth]   graph analyzed code (agd = dot format) (check 'gu?')\n");
 		cons_printf(" ar [args]    analyze ranges\n");
@@ -713,11 +724,11 @@ CMD_DECL(project)
 {
 	char *arg = input + 2;
 	char* str;
-	switch(input[0]) {
+	switch (input[0]) {
 	case 'o':
 		if (input[1])
 			project_open(arg);
-		else{
+		else {
 			str = strdup ( config_get("file.project") );
 			project_open(str);
 			free ( str );
@@ -749,7 +760,6 @@ CMD_DECL(project)
 		" Pi [file]  info\n");
 		break;
 	}
-
 	return 0;
 }
 
@@ -809,16 +819,13 @@ CMD_DECL(menu)
 			sprintf(buf2, "e %s.%s", pfx, buf);
 			radare_cmd(buf2, 0);
 		} else
-		if (!pfx[0]) {
-			strcpy(pfx, buf);
-		} else {
+		if (pfx[0]) {
 			cons_printf("%s.%s = ", pfx, buf);
 			sprintf(buf2, "e %s.%s", pfx, buf);
 			radare_cmd(buf2, 0);
 			goto noprint;
-		}
+		} else strcpy(pfx, buf);
 	}
-
 	return 0;
 }
 #endif
@@ -830,8 +837,6 @@ CMD_DECL(hack)
 	return radare_hack(input);
 }
 
-
-
 CMD_DECL(graph)
 {
 	char *text = input;
@@ -948,7 +953,7 @@ CMD_DECL(graph)
 			list_for_each_prev(head, &(prg->blocks)) {
 				char str[128];
 				struct block_t *b0 = list_entry(head, struct block_t, list);
-				string_flag_offset(str, b0->addr, -1);
+				string_flag_offset(NULL, str, b0->addr, -1);
 				cons_printf("0x%08llx : %s ",
 					b0->addr, str);
 				if (b0->tnext != 0)
@@ -1056,9 +1061,9 @@ CMD_DECL(graph)
 				struct program_t *mr = list_entry(pos, struct program_t, list);
 				fflush(stdout);
 				offstr[0]='\0';
-				string_flag_offset(offstr, mr->entry, 0);
+				string_flag_offset(NULL, offstr, mr->entry, 0);
 
-				cons_printf("%02d 0x%08llx %s\n", i, (u64)mr->entry, offstr);
+				cons_printf("%02d 0x%08llx %s\n", i, (ut64)mr->entry, offstr);
 				i++;
 			}
 			return 0;
@@ -1066,7 +1071,6 @@ CMD_DECL(graph)
 		}
 	}
 
-
 	return 0;
 }
 
@@ -1086,12 +1090,12 @@ CMD_DECL(hash)
 	int i;
 	char buf[1024];
 	char *str;
-	u64 bs = config.block_size;
-	u64 obs = config.block_size;
+	ut64 bs = config.block_size;
+	ut64 obs = config.block_size;
 
-	str = strchr(input, ' ');
+	str = strchr (input, ' ');
 	if (str)
-		bs = get_math(str+1);
+		bs = get_math (str+1);
 
 	for(i=0;input[i];i++) if (input[i]==' ') { input[i]='\0'; break; }
 
@@ -1102,7 +1106,7 @@ CMD_DECL(hash)
 		radare_set_block_size_i(obs);
 	} else {
 		snprintf(buf, 1000, "rahash -a '%s' -S %lld -L %lld '%s'", // | head -n 1", 
-			input, (u64)config.seek, (u64)bs, config.file);
+			input, (ut64)config.seek, (ut64)bs, config.file);
 		io_system(buf);
 	}
 	return 0;
@@ -1137,16 +1141,31 @@ CMD_DECL(open)
 		io_map_list();
 		break;
 	case '?':
-		cons_printf("Usage: o[-] [file] [offset]\n");
-		cons_printf(" o /bin/ls                  ; open file\n");
-		cons_printf(" o /lib/libc.so 0xC848000   ; map file at offset\n");
-		cons_printf(" o- /lib/libc.so            ; unmap\n");
+		cons_printf("Usage: o[-] [file] [offset] [delta] [size]\n");
+		cons_printf(" o                         ; list opened files\n");
+		cons_printf(" o /bin/ls                 ; open file\n");
+ 		cons_printf("; map file at offset skiping 100 bytes until 200 of the file\n");
+		cons_printf(" o /lib/libc.so 0xC848000 100 200\n");
+		cons_printf(" o- /lib/libc.so           ; unmap\n");
 		break;
 	case ' ':
 		arg = strchr(ptr+1, ' ');
 		if (arg != NULL) {
+			char *arg2, *arg3;
+			ut64 size = 0LL;
+			ut64 delta = 0LL;
 			arg[0]='\0';
-			io_map(ptr+1, get_math(arg+1));
+			arg2 = strchr(arg+1, ' ');
+			if (arg2) {
+				*arg2='\0';
+				arg3 = strchr(arg2+1, ' ');
+				if (arg3) {
+					*arg3='\0';
+					size = get_math(arg3+1);
+				}
+				delta = get_math(arg2+1);
+			}
+			io_map(ptr+1, get_math(arg+1)-config.vaddr, delta, size);
 		} else {
 			/* XXX */
 			config.file = ptr+1;
@@ -1154,7 +1173,6 @@ CMD_DECL(open)
 		}
 	}
 	free(ptr);
-
 	return 0;
 }
 
@@ -1202,17 +1220,10 @@ CMD_DECL(envvar)
 	ptr[0]='\0';
 	for(;*text&&iswhitespace(*text);text=text+1);
 	for(ptr=ptr+1;*ptr&&iswhitespace(*ptr);ptr=ptr+1);
-
-	if ((!memcmp(ptr,"-", 2))
-	|| (!memcmp(ptr,"(null)", 5))) {
+	if ((!memcmp(ptr,"-", 2)) || (!memcmp(ptr,"(null)", 5)))
 		unsetenv(text2);
-		//D cons_printf("%%%s=(null)\n", text2);
-	} else {
-		setenv(text2, ptr, 1);
-		//D cons_printf("%%%s='%s'\n", text2, ptr);
-	}
+	else setenv(text2, ptr, 1);
 	ptro[0]=' ';
-
 	env_update();
 	free(text2);
 
@@ -1249,7 +1260,6 @@ CMD_DECL(blocksize)
 	default:
 		radare_set_block_size(input);
 	}
-
 	return 0;
 }
 
@@ -1296,9 +1306,9 @@ CMD_DECL(code)
 			break;
 		default:
 			{
-			u64 from = get_math(text+1);
+			ut64 from = get_math(text+1);
 			if (from >= config.vaddr*2)
-				from -=config.vaddr;
+				from -= config.vaddr;
 			data_xrefs_add(from, config.seek+config.vaddr, 0);
 			}
 			break;
@@ -1356,7 +1366,7 @@ CMD_DECL(code)
 			if (text[2]=='\0') {
 				eprintf("Oops. missing argument?\n");
 			} else {
-				u64 len, tmp = config.block_size;
+				ut64 len, tmp = config.block_size;
 				len = get_math(text+2);
 				radare_set_block_size_i(len);
 				data_add_arg(config.seek+(config.cursor_mode?config.cursor:0), DATA_FUN, text+2);
@@ -1396,7 +1406,7 @@ CMD_DECL(code)
 		struct data_t *d;
 		int fmt, len;
 		char *arg = text + 1;
-		u64 tmp = config.block_size;
+		ut64 tmp = config.block_size;
 
 		for(; *arg==' ';arg=arg+1);
 		if (arg[0]=='\0') {
@@ -1460,7 +1470,6 @@ CMD_DECL(code)
 		" CF*                    ; list function ranges as ar cmds\n"
 		" C*                     ; list metadata database\n");
 	}
-
 	return 0;
 }
 
@@ -1532,6 +1541,13 @@ CMD_DECL(print)
 				bs = config.block_size;
 				radare_set_block_size_i(data->size);
 			}
+		} else {
+			if (strchr(input, ' ')) {
+				ut64 num = get_math (input+1);
+				bs = config.block_size;
+				if (input[0]!='d')
+					radare_set_block_size_i(num);
+			}
 		}
 		fmt = format_get(input[0]); //, fmt);
 		if (fmt == FMT_ERR)
@@ -1569,7 +1585,6 @@ CMD_DECL(resize)
 		printf("0x"OFF_FMTx"\n", config.size);
 		return 0;
 	}
-
 	/* XXX not needed ? */
 	for(;*text&&!iswhitespace(*text);text=text+1);
 	for(;*text&&iswhitespace(*text);text=text+1);
@@ -1587,6 +1602,7 @@ CMD_DECL(flag)
 	char *text2;
 	char *eof = input + strlen(input)-1;
 
+	if (input[0]!='-')
 	for(;*text&&!iswhitespace(*text);text=text+1);
 	for(;*text&&iswhitespace(*text);text=text+1);
 	for(;iswhitespace(eof[0]);eof=eof-1) eof[0]='\0';
@@ -1619,7 +1635,7 @@ CMD_DECL(flag)
 	case 's': flag_space(input+1); break;
 	case 'u': flag_set_undef(input+2, config.seek, 0); break;
 	case 'm': flag_space_move(text); break;
-	case 'd': print_flag_offset(config.seek); cons_newline(); break;
+	case 'd': print_flag_offset(text, config.seek); cons_newline(); break;
 	case 'i':
 		text2=strchr(text,' ');
 		if (text2) {
@@ -1629,25 +1645,21 @@ CMD_DECL(flag)
 		} else eprintf("Usage: fi hit0_ hit1_\n");
 		break;
 	case 'N':
-		if (input[1]=='\0') {
-			flag_ctr = 0;
-		} else {
+		if (input[1]!='\0') {
 			char bu[128];
 			snprintf(bu, 127, "%s_%d",text, flag_ctr++); 
 			while(bu[0]==' ')strcpy(bu, bu+1);
 			eprintf("0x%08llx %s\n", config.seek, bu);
 			flag_set(bu, config.seek,0);
-		}
+		} else flag_ctr = 0;
 		break;
 	case '\0': flag_list(text); break;
 	//case '*': flag_set("*",0,0); break;
 	case '-': flag_remove(text+1); break;
 	case ' ':
-		if (input[1]=='-') {
-			flag_remove(text+1);
-		} else {
-			u64 here = config.seek;//+config.vaddr;
-			u64 size = config.block_size;
+		if (input[1]!='-') {
+			ut64 here = config.seek;//+config.vaddr;
+			ut64 size = config.block_size;
 			char *s = strchr(text, ' ');
 			char *s2 = NULL;
 			if (s) {
@@ -1665,9 +1677,8 @@ CMD_DECL(flag)
 			if (s2) *s2=' ';
 			if (size != config.block_size)
 				radare_set_block_size_i(size);
-		}
+		} else flag_remove(text+1);
 	}
-
 	return ret;
 }
 
@@ -1690,13 +1701,11 @@ CMD_DECL(undowrite)
 		break;
 	case ' ':
 	case '\0':
-		if (input[0] == '\0')
-			undo_write_list(0);
-		else {
+		if (input[0] != '\0') {
 			if (input[1]=='-')
 				undo_write_set(atoi(input+2), 1);
 			else undo_write_set(atoi(input+1), 0);
-		}
+		} else undo_write_list(0);
 		break;
 	case '?':
 	default:
@@ -1717,7 +1726,7 @@ CMD_DECL(undowrite)
 
 CMD_DECL(seek)
 {
-	u64 new_off = 0;
+	ut64 new_off = 0;
 	struct aop_t aop;
 	char *input2, *text;
 	int whence    = SEEK_SET;
@@ -1759,7 +1768,7 @@ CMD_DECL(seek)
 #endif
 
 	if (text[0] == '\0') {
-		D printf(OFF_FMT"\n", (u64)config.seek);
+		D printf(OFF_FMT"\n", (ut64)config.seek);
 		return 0;
 	}
 	for(;text[0]==' ';text = text +1);
@@ -1833,13 +1842,12 @@ CMD_DECL(seek)
 		whence   = SEEK_SET; // stupid twice
 	}
 
-	if (new_off<0) new_off = 0;
-	if (text[0]=='0' && new_off==0 || new_off != 0) {
+	//if (new_off<0) new_off = 0;
+	if (1 || (text[0]=='0'&&new_off==0) || new_off!=0) {
 		if (radare_seek(new_off, whence) < 0)
 			eprintf("Couldn't seek: %s\n", strerror(errno));
 	}
 	undo_push();
-
 	radare_read(0);
 
 	return 0;
@@ -1866,15 +1874,15 @@ CMD_DECL(info)
 	cons_printf(" vaddr   "OFF_FMTd"  ( 0x"OFF_FMTx" )", config.vaddr, config.vaddr); cons_newline();
 	cons_printf(" bsize   %d  ( 0x%x )", config.block_size, config.block_size); cons_newline();
 	cons_printf(" seek    "OFF_FMTd" 0x"OFF_FMTx,
-		(u64)config.seek, (u64)config.seek); cons_newline();
+		(ut64)config.seek, (ut64)config.seek); cons_newline();
 	cons_printf(" delta   %lld\n", config_get_i("cfg.delta")); 
 	cons_printf(" count   %lld\n", config_get_i("cfg.count")); 
 	//fflush(stdout);
 	//print_flag_offset(config.seek);
 	cons_printf(" size    "OFF_FMTd" \t 0x"OFF_FMTx,
-		(u64)config.size, (u64)config.size); cons_newline();
+		(ut64)config.size, (ut64)config.size); cons_newline();
 	cons_printf(" limit   "OFF_FMTd" \t 0x"OFF_FMTx,
-		(u64)config.limit, (u64)config.limit); cons_newline();
+		(ut64)config.limit, (ut64)config.limit); cons_newline();
 
 	if (config.debug) {
 		cons_printf("Debugger:\n");
@@ -1902,7 +1910,7 @@ CMD_DECL(compare)
 		break;
 	case 'f':
 		if (input[1]!=' ') {
-			eprintf("Please. use 'wf [file]'\n");
+			eprintf ("Please. use 'cf [file]'\n");
 			return 0;
 		}
 		fd = fopen(input+2, "r");
@@ -1918,7 +1926,7 @@ CMD_DECL(compare)
 		break;
 	case 'x':
 		if (input[1]!=' ') {
-			eprintf("Please. use 'wx 00 11 22'\n");
+			eprintf("Please. use 'cx 00 11 22'\n");
 			return 0;
 		}
 		buf = (unsigned char *)malloc(strlen(input+2));
@@ -1968,10 +1976,10 @@ CMD_DECL(compare)
 CMD_DECL(write)
 {
 	int ret;
-	u64 delta = 0;
-	u64 off;
-	u64 here = config.seek + (config.cursor_mode?config.cursor:0);
-	u64 back = config.seek;
+	ut64 delta = 0;
+	ut64 off;
+	ut64 here = config.seek + (config.cursor_mode?config.cursor:0);
+	ut64 back = config.seek;
 
 	radare_seek(here, SEEK_SET);
 	switch (input[0]) {
@@ -2058,7 +2066,7 @@ CMD_DECL(write)
 			int ret, delta = 0;
 			unsigned char data[256];
 			char *oinput, *ptr, *aux = strdup ( config_get("asm.arch") );
-			u64 seek = config.seek + config.vaddr;
+			ut64 seek = config.seek + config.vaddr;
 			oinput= input = strdup(input+2);
 			while(input) {
 				ptr = strchr(input, ';');
@@ -2104,6 +2112,9 @@ CMD_DECL(write)
 		case '4':
 			ret = radare_write_op(input+3, input[1]);
 			break;
+		case 'n':
+			ret = radare_write_op ("ff", 'x');
+			break;
 		case '\0':
 		case '?':
 		default:
@@ -2115,6 +2126,7 @@ CMD_DECL(write)
 			"  woa  addition            +=\n"
 			"  wos  substraction        -=\n"
 			"  wom  multiply            *=\n"
+			"  won  negate all bits (same as wox ff)\n"
 			"  wod  divide              /=\n"
 			"  wox  xor                 ^=\n"
 			"  woo  or                  |=\n"
@@ -2143,7 +2155,7 @@ CMD_DECL(write)
 			eprintf("Please. use 'wT prefix-file [size]'.\n");
 			return 0;
 		} else {
-			u64 bsize = config.block_size;
+			ut64 bsize = config.block_size;
 			static counter = 0;
 			char file[1024];
 			char *off = strchr(input+2, ' ');
@@ -2222,20 +2234,21 @@ CMD_DECL(sections)
 {
 	switch(input[0]) {
 	case '?':
-		eprintf("Usage: S[cbtf=*] len [base [comment]] @ address\n");
-		eprintf(" S                ; list sections\n");
-		eprintf(" S*               ; list sections (in radare commands\n");
-		eprintf(" S=               ; list sections (in visual)\n");
-		eprintf(" S 4096 0x80000 rwx section_text  @ 0x8048000 ; adds new section\n");
-		eprintf(" S 4096 0x80000   ; 4KB of section at current seek with base 0x.\n");
-		eprintf(" S 10K @ 0x300    ; create 10K section at 0x300\n");
-		eprintf(" S -0x300         ; remove this section definition\n");
-		eprintf(" Sd 0x400 @ here  ; set ondisk start address for current section\n");
-		eprintf(" Sc rwx _text     ; add comment to the current section\n");
-		eprintf(" Sb 0x100000      ; change base address\n");
-		eprintf(" St 0x500         ; set end of section at this address\n");
-		eprintf(" Sf 0x100         ; set from address of the current section\n");
-		eprintf(" Sp 7             ; set rwx (r=4 + w=2 + x=1)\n");
+		eprintf("Usage: S[cbtf=*] len [base [comment]] @ address\n"
+		" S                ; list sections\n"
+		" S*               ; list sections (in radare commands)\n"
+		" S=               ; list sections (in nice ascii-art bars)\n"
+		" Sa @ 0x804840    ; show section at address\n"
+		" S 4096 0x80000 rwx section_text @ 0x8048000 ; adds new section\n"
+		" S 4096 0x80000   ; 4KB of section at current seek with base 0x.\n"
+		" S 10K @ 0x300    ; create 10K section at 0x300\n"
+		" S -0x300         ; remove this section definition\n"
+		" Sd 0x400 @ here  ; set ondisk start address for current section\n"
+		" Sc rwx _text     ; add comment to the current section\n"
+		" Sb 0x100000      ; change base address\n"
+		" St 0x500         ; set end of section at this address\n"
+		" Sf 0x100         ; set from address of the current section\n"
+		" Sp 7             ; set rwx (r=4 + w=2 + x=1)\n");
 		break;
 	case ' ':
 		switch(input[1]) {
@@ -2247,18 +2260,18 @@ CMD_DECL(sections)
 			int i;
 			char *ptr = strdup(input+1);
 			const char *comment = NULL;
-			u64 from = config.seek;
-			u64 to   = config.seek + config.block_size;
-			u64 base = config.vaddr; //config_get_i("io.vaddr");
-			u64 ondisk = config.paddr;
+			ut64 from = config.seek;
+			ut64 to   = config.seek + config.block_size;
+			ut64 base = config.vaddr; //config_get_i("io.vaddr");
+			ut64 ondisk = 0LL;
 			
 			i = set0word(ptr);
 			switch(i) {
-			case 2: // get comment
+			case 3: // get comment
 				comment = get0word(ptr, 2);
-			case 1: // get base address
-				base = get_math(get0word(ptr, 1));
-			case 0: // get length
+			case 2: // get base address
+				ondisk = get_math(get0word(ptr, 1));
+			case 1: // get length
 				to = from + get_math(get0word(ptr,0));
 			}
 			section_add(from, to, base, ondisk, 7, comment);
@@ -2270,6 +2283,9 @@ CMD_DECL(sections)
 	case '=':
 		section_list_visual(config.seek, config.block_size);
 		break;
+	case 'a':
+		section_list(config.seek, 2);
+		break;
 	case '\0':
 		section_list(config.seek, 0);
 		break;
@@ -2294,8 +2310,8 @@ CMD_DECL(sections)
 	case 'f':
 		eprintf("TODO\n");
 		break;
-
 	}
+	return 0;
 }
 
 CMD_DECL(examine)
@@ -2317,7 +2333,7 @@ CMD_DECL(examine)
 
 CMD_DECL(prev)
 {
-	u64 off;
+	ut64 off;
 	if (input[0] == '\0')
 		off = config.block_size;
 	else	off = get_math(input);
@@ -2337,7 +2353,7 @@ CMD_DECL(prev)
 
 CMD_DECL(next)
 {
-	u64 off;
+	ut64 off;
 	if (input[0] == '\0')
 		off = config.block_size;
 	else	off = get_math(input);
@@ -2395,14 +2411,15 @@ CMD_DECL(search) {
 	char *input2 = (char *)strdup(input);
 	char *text   = input2;
 	int  len,i = 0,j = 0;
-	u64 seek = config.seek;
+	ut64 seek = config.seek;
 	char *ptr;
 
 	switch(text[0]) {
 	case '\0':
 	case '?':
-		eprintf(
+		cons_printf(
 		" / \\x7FELF       ; plain string search (supports \\x).\n"
+		" /i str-no-case  ; case insensitive text search\n"
 		" /. [file]       ; search using the token file rules (/.? for help)\n"
 		" /: [file] [...] ; search using the 'keyname keyword' file format\n"
 		" /-              ; unsetenv SEARCH[N] and MASK[N] environ vars( deprecated )\n"
@@ -2416,7 +2433,7 @@ CMD_DECL(search) {
 		" /s [str] [str]  ; replace first string with the second one\n"
 		" /S [hex] [hex]  ; replace first hexpair string with the second one\n"
 		" /p len          ; search pattern of length = len\n"
-		" /P count        ; search patterns of size $$b matching count bytes at least against first block\n"
+		" /P count        ; search patterns of size $$b matching at least N bytes of curblock\n"
 		" /v numexpr      ; search a value (32 or 64 bit size) uses cfg.bigendian\n"
 		" /w foobar       ; search a widechar string (f\\0o\\0o\\0b\\0..)\n"
 		" /x A0 B0 43        ; hex byte pair binary search. (space between bytes are optional)\n"
@@ -2547,6 +2564,7 @@ CMD_DECL(search) {
 	case '/':
 		search_range("0");
 		break;
+	case 'i':
 	case '0':
 	case '1':
 	case '2':
@@ -2567,28 +2585,31 @@ CMD_DECL(search) {
 		else ptr = ptr +1;
 		setenv(buf, ptr, 1);
 		sprintf(buf, "%d", idx);
+		if (text[0]=='i')
+			search_nocase = 1;
+		else search_nocase = 0;
 		search_range(buf);
 		} break;
 	case 'v':
 		{
 		char buf[64];
-		u8 *b;
-		u64 n, _n;
-		u32 n32, _n32;
+		ut8 *b;
+		ut64 n, _n;
+		ut32 n32, _n32;
 		radare_cmd("f -hit0*", 0);
 		_n = n = get_math(input+2);
 		if (n & 0xffffffff00000000LL) {
-			endian_memcpy_e(&n, &_n, 8, !config_get_i("cfg.bigendian"));
-			b=&n;
+			endian_memcpy_e((ut8*)&n, (const ut8*)&_n, 8, !config_get_i("cfg.bigendian"));
+			b=(ut8*)&n;
 			sprintf(buf,
 			"\\x%02x\\x%02x\\x%02x\\x%02x"
 			"\\x%02x\\x%02x\\x%02x\\x%02x",
 			b[0],b[1],b[2],b[3],
 			b[4],b[5],b[6],b[7]);
 		} else {
-			_n32 = n32=(u32)n;
-			endian_memcpy_e(&n32, &_n32, 4, !config_get_i("cfg.bigendian"));
-			b=&n32;
+			_n32 = n32=(ut32)n;
+			endian_memcpy_e((ut8*)&n32, (const ut8*)&_n32, 4, !config_get_i("cfg.bigendian"));
+			b=(ut8*)&n32;
 			sprintf(buf,
 			"\\x%02x\\x%02x\\x%02x\\x%02x",
 			b[0],b[1],b[2],b[3]);
@@ -2619,15 +2640,23 @@ CMD_DECL(search) {
 	case 'x':
 		free(input2);
 		len = strlen(input+2);
+		if (len<2) {
+			eprintf("Too short len\n");
+			return;
+		}
 		input2 = (char *)malloc((len<<1)+10);
 		input2[0] = '\0';
 		for(i=0;i<len;i+=j) {
 			char buf[10];
-			int ch = hexpair2bin(input+2+i);
+			int ch;
+			ch = hexpair2bin(input+2+i);
 			if (i+4<len && input[2+i+2]==' ')
 				j = 3;
 			else j = 2;
-			if (ch == -1) break;
+			if (ch == -1) {
+				eprintf ("Invalid hexadecimal string\n");
+				goto __beach;
+			}
 			if (is_printable(ch)) {
 				buf[0] = ch;
 				buf[1] = '\0';
@@ -2646,6 +2675,7 @@ CMD_DECL(search) {
 		search_range("0");
 		break;
 	}
+__beach:
 	free(input2);
 
 	return 0;
@@ -2686,18 +2716,21 @@ CMD_DECL(rtr)
 	default:
 		rtr_cmd(input);
 	}
+	return 0;
 }
 
 CMD_DECL(help)
 {
 	if (strlen(input)>0) {
 		// XXX switch/case this!
-		if (input[0]=='!') {
+		switch(input[0]) {
+		case '!':
 			if (config.last_cmp != 0)
 				radare_cmd(input+1, 0);
-		} else
-		if (input[0]=='s') {
-			u64 from, to, step;
+			break;
+		case 's':
+			{
+			ut64 from, to, step;
 			char *p, *p2;
 			for(input=input+1;input[0]==' ';input=input+1);
 			p = strchr(input, ' ');
@@ -2714,16 +2747,19 @@ CMD_DECL(help)
 					cons_printf("%lld ", from);
 				cons_newline();
 			}
-		} else
-		if (input[0]=='q') {
+			}
+			break;
+		case 'q':
 			config.last_cmp = get_math(input+1);
-		} else
-		if (input[0]=='v') {
-			u64 res = get_math(input+1);
+			break;
+		case 'v':
+			{
+			ut64 res = get_math(input+1);
 			config.last_cmp = res;
 			cons_printf("0x%08llx\n", res);
-		} else
-		if (input[0]=='?') {
+			}
+			break;
+		case '?':
 			if (input[1]=='?') {
 				cons_printf("0x%llx\n", config.last_cmp);
 			} else
@@ -2731,11 +2767,13 @@ CMD_DECL(help)
 				cons_strcat(
 				"Usage: ?[?[?]] <expr>\n"
 				" ? eip             ; get value of eip flag (or any math expression)\n"
-				" ?i offset A:      ; user input for u64 numeric values, result in $$?\n"
+				" ?i offset A:      ; user input for ut64 numeric values, result in $$?\n"
+				" ?f 3.4545         ; calculate hex, double and float value for hex or float num\n"
 				" ?v eip+33+[esp]   ; show hex value of math expression\n"
 				" ?q eip+33+[esp]   ; quite math expression evaluation (no output) changes $$?\n"
 				" ?z`str            ; sets false if string is zero length\n"
 				" ?x 303132         ; show hexpair as a printable string\n"
+				" ?x 0x804800       ; show hexpair ruled by asm.bits and cfg.bigendian\n"
 				" ?s from to step   ; print numeric sequence for(;from<=to;from+=step)\n"
 				" ?X eip            ; show hex result of math expression\n"
 				" ? 0x80+44         ; calc math expression\n"
@@ -2767,8 +2805,9 @@ CMD_DECL(help)
 			} else
 			if (config.last_cmp == 0)
 				radare_cmd(input+1, 0);
-		} else
-		if (input[0]=='<') {
+			break;
+		case '<':
+			{
 			char buf[1024];
 			cons_printf("%s?\n", input+1);
 			cons_flush();
@@ -2782,48 +2821,122 @@ CMD_DECL(help)
 			if (!strcmp(buf, "n")) {
 				config.last_cmp = 0;
 			} else config.last_cmp = get_math(buf);
-		} else
-		if (input[0]=='t') {
+			}
+			break;
+		case 't':
+			{
 			struct r_prof_t prof;
 			r_prof_start(&prof);
 			radare_cmd(input+1, 0);
 			r_prof_end(&prof);
-			config.last_cmp = (u64)prof.result;
+			config.last_cmp = (ut64)prof.result;
 			eprintf("%lf\n", prof.result);
-		} else
-		if (input[0]=='e') {
+			}
+			break;
+		case 'e':
+			{
 			char *str = input+1;
-			if (str[0]==' ') str = str+1;
+			if (str[0]==' ')
+				str = str+1;
 			if (str[0]) {
 				cons_strcat(str);
 				cons_newline();
+			} }
+			break;
+		case 'f':
+			{
+			//double n1, n2;
+			/* TODO: Add support for doubles 'F'? */
+			char *str = input+((input[1]==' ')?2:1);
+			char buf[64];
+			ut32 _ut32, __ut32;
+			float _f;
+			ut8 *p = (ut8*) &_f;
+			if (str[0]=='0'&&str[1]=='x') {
+				sscanf(str, "0x%x", &_ut32);
+				memcpy(&_f, &_ut32, sizeof(float));
+				sprintf(buf, "%f", _f);
+				/* consistency check */
+				//n1 = (double)_f;
+				//sscanf(buf, "%g", (double *)&n2);
+			} else {
+				sscanf(str, "%f", &_f);
+				memcpy(&_ut32, &_f, sizeof(ut32));
+				sprintf(buf, "0x%x", _ut32);
+				/* consistency check */
+				//sscanf(str, "%lf", &n1);
+				//n2 = (double)_f;
 			}
-		} else
-		if (input[0]=='i') {
+			cons_printf("%s\n", buf);
+			/* it is consistent ? */
+			//if (n1 != n2) cons_printf("%s,\n", buf);
+			//else cons_printf("%s\n", buf);
+			}
+			break;
+		case 'i':
+			{
 			char str[128];
 			eprintf("%s: ", input+2);
 			fflush(stderr);
 			fgets(str, 127, stdin);
 			config.last_cmp = get_math(str);
-		} else
-		if (input[0]=='z') {
+			}
+			break;
+		case 'z':
+			{
 			char *ptr;
 			for(ptr=input+1;*ptr==' ';ptr=ptr+1);
 			config.last_cmp = strlen(ptr);
-		} else
-		if (input[0]=='x') {
-			char buf[1024];
-			int len = hexstr2binstr(input+1, buf);
-			if (len>0)
-				print_data(0, "", buf, len, FMT_ASC);
-		} else
-		if (input[0]=='X') {
-			u64 res = get_math(input+1);
+			}
+			break;
+		case 'x':
+			if (input[2]=='0' && input[3]=='x') {
+	// TODO: endian memcpy!!
+				switch(config_get_i("asm.bits")) {
+				case 64:
+					{
+					ut64 n = get_math(input+2);
+					u8 np[8];
+					endian_memcpy_e((u8*)&np, (u8*)&n, 8, !config.endian);
+					cons_printf("%02x %02x %02x %02x %02x %02x %02x %02x\n",
+						np[0], np[1], np[2], np[3], np[4], np[5], np[6], np[7]);
+					}
+					break;
+				case 32:
+					{ ut32 n = get_math(input+2);
+					u8 np[4];
+					endian_memcpy_e((u8*)&np, (u8*)&n, 4, !config.endian);
+					cons_printf("%02x %02x %02x %02x\n",
+						np[0], np[1], np[2], np[3]);
+					}
+					break;
+				case 16:
+					{ ut16 n = get_math(input+2);
+					u8 np[2];
+					endian_memcpy_e((u8*)&np, (u8*)&n, 2, !config.endian);
+					cons_printf("%02x %02x\n", np[0], np[1]);
+					}
+					break;
+				}
+			} else {
+				char buf[1024];
+				int len;
+				len = hexstr2binstr(input+1, buf);
+				if (len>0)
+					print_data(0, "", buf, len, FMT_ASC);
+			}
+			break;
+		case 'X':
+			{
+			ut64 res = get_math(input+1);
 			if (res > 0xffffffff)
 				cons_printf("0x%llx\n", -res);
 			else cons_printf("0x%llx\n", res);
-		} else {
-			u64 res = get_math(input);
+			}
+			break;
+		default:
+			{
+			ut64 res = get_math(input);
 			config.last_cmp = res;
 		//	if (!strchr(input, '!') && !strchr(input, '=')) {
 			D { 
@@ -2849,9 +2962,9 @@ CMD_DECL(help)
 				cons_newline();
 			}// else cons_printf("0x%llx\n", res);
 			//}
+			}
 		}
 	} else show_help_message();
-
 	return 0;
 }
 
