@@ -54,7 +54,7 @@ extern int arm_mode;
 extern int mips_mode;
 int ollyasm_enable = 0;
 
-static char *udis_mem = NULL;
+static const ut8 *udis_mem = NULL;
 static int udis_mem_ptr = 0;
 static int input_hook_x(ud_t* u)
 {
@@ -66,7 +66,7 @@ static int input_hook_x(ud_t* u)
 	return config.block[ud_idx++];
 }
 
-void udis_set_pc(u64 pc)
+void udis_set_pc(ut64 pc)
 {
 //	ud_idx = 0;
 	ud_set_pc(&ud_obj, pc);
@@ -122,7 +122,7 @@ void udis_init()
 }
 
 static int jump_n = 0;
-static u64 jumps[10]; // only 10 jumps allowed
+static ut64 jumps[10]; // only 10 jumps allowed
 
 void udis_jump(int n)
 {
@@ -135,12 +135,12 @@ void udis_jump(int n)
 }
 
 /* -- disassemble -- */
-char *disarm (RawInstruction rawinstruction, u64 offset);
+char *disarm (RawInstruction rawinstruction, ut64 offset);
 
-int udis_arch_string(int arch, char *string, const u8 *buf, int endian, u64 seek, int bytes, int myinc)
+int udis_arch_string(int arch, char *string, const u8 *buf, int endian, ut64 seek, int bytes, int myinc)
 {
 	//unsigned char *b = config.block + bytes;
-	const u8 *b = buf; //config.block + bytes;
+	const ut8 *b = buf; //config.block + bytes;
 	int ret = 0;
 
 	//ud_idx = bytes;
@@ -154,6 +154,12 @@ int udis_arch_string(int arch, char *string, const u8 *buf, int endian, u64 seek
 	//b = config.block + bytes;
 	string[0]='\0';
 	switch(arch) {
+	case ARCH_Z80:
+		ret = z80_udis (string, buf, bytes, seek);
+		break;
+	case ARCH_8051:
+		ret = dis51_udis (string, buf, bytes, seek);
+		break;
 	case ARCH_ARM:
 		arm_mode = 32;
 		force_thumb = 0;
@@ -166,17 +172,18 @@ int udis_arch_string(int arch, char *string, const u8 *buf, int endian, u64 seek
 		break;
 	case ARCH_CSR:
 		//if (bytes<config.block_size) {
-		arch_csr_disasm(string, (const unsigned char *)b, (u64)seek);
+		arch_csr_disasm(string, (const ut8 *)b, (ut64)seek);
 		break;
 	case ARCH_X86:
 		if (ollyasm_enable) {
 			t_disasm da;
-			ud_disassemble(&ud_obj);
-			lowercase=1;
-			ret = Disasm(b, MAXCMDSIZE, seek, &da, DISASM_FILE);
-			sprintf(string, "%s", da.result);
+			ud_disassemble (&ud_obj);
+			lowercase = 1;
+			ret = Disasm ((char*)b, MAXCMDSIZE, seek, &da, DISASM_FILE);
+			sprintf (string, "%s", da.result);
 			if (da.error)
-				sprintf(string, "%i -- %s - %02x %02x %02x (error)",ret,da.dump,b[0],b[1],b[2]);
+				sprintf (string, "%i -- %s - %02x %02x %02x (error)",
+					ret, da.dump, b[0], b[1], b[2]);
 		} else {
 //			udis_init();
 			//ud_obj.insn_offset = seek+myinc; //+bytes;
@@ -198,9 +205,9 @@ int udis_arch_string(int arch, char *string, const u8 *buf, int endian, u64 seek
 	       /* initialize DisasmPara */
 	       dp.opcode = opcode;
 	       dp.operands = operands;
-	       dp.iaddr = seek; //config.vaddr + config.seek + i;
+	       dp.iaddr = (ppc_word*) (size_t)seek; //config.vaddr + config.seek + i;
 	       endian_memcpy(bof, b, 4);
-	       dp.instr = bof;
+	       dp.instr = (ppc_word*) bof;
 	       // memcpy(bof, b, 4);
 	       // dp.instr = b; //config.block + i;
 	       PPC_Disassemble(&dp);
@@ -261,7 +268,7 @@ int udis_arch_string(int arch, char *string, const u8 *buf, int endian, u64 seek
 	return ret;
 }
 
-int udis_arch_opcode(int arch, const u8 *b, int endian, u64 seek, int bytes, int myinc)
+int udis_arch_opcode(int arch, const u8 *b, int endian, ut64 seek, int bytes, int myinc)
 {
 	char buf[128];
 	//unsigned char *b = config.block + bytes; //(seek-config.seek); //+ bytes;
@@ -280,6 +287,9 @@ int udis_arch_opcode(int arch, const u8 *b, int endian, u64 seek, int bytes, int
 		/* ultra ugly hack */
 		ret = udis_arch_string(arch, buf, b, endian, seek, bytes, myinc);
 		break;
+	case ARCH_BF:
+		ret = arch_bf_dis(b, seek+myinc, 1024);
+		break;
 	case ARCH_PPC:
 	case ARCH_JAVA:
 	case ARCH_M68K:
@@ -289,11 +299,10 @@ int udis_arch_opcode(int arch, const u8 *b, int endian, u64 seek, int bytes, int
 	case ARCH_ARM:
 	case ARCH_MIPS:
 	case ARCH_SPARC:
+	case ARCH_8051:
+	case ARCH_Z80:
 		ret = udis_arch_string(arch, buf, b, endian, seek+myinc, bytes, myinc);
 		break;
-	case ARCH_BF:
-		ret = arch_bf_dis(b, seek+myinc, 1024);
-		break;
 	default:
 		strcpy(buf, "Unknown architecture");
 		break;
@@ -314,7 +323,7 @@ void udis_arch(int arch, int len, int rows)
 }
 
 /* aop disassembler */
-int arch_aop_aop(u64 addr, const u8 *bytes, struct aop_t *aop)
+int arch_aop_aop(ut64 addr, const u8 *bytes, struct aop_t *aop)
 {
 	switch(aop->type) {
 	case AOP_TYPE_NOP:
@@ -339,9 +348,11 @@ int arch_aop_aop(u64 addr, const u8 *bytes, struct aop_t *aop)
 struct radis_arch_t {
 	const char *name;
 	int id;
-	int (*fun)(u64 addr, const u8 *bytes, struct aop_t *aop);
+	int (*fun)(ut64 addr, const u8 *bytes, struct aop_t *aop);
 } radis_arches [] = {
 	{ "bf"      , ARCH_BF    , &arch_bf_aop  }   ,
+	{ "z80"     , ARCH_Z80   , &arch_z80_aop }   , 
+	{ "8051"    , ARCH_8051  , &arch_8051_aop }   , 
 	{ "intel"   , ARCH_X86   , &arch_x86_aop }   , 
 	{ "intel16" , ARCH_X86   , &arch_x86_aop }   , 
 	{ "intel32" , ARCH_X86   , &arch_x86_aop }   , 
@@ -377,7 +388,7 @@ void radis_update_i(int id)
 void radis_update()
 {
 	int i;
-	char *arch = config_get("asm.arch");
+	const char *arch = config_get("asm.arch");
 	for(i=0;radis_arches[i].name;i++) {
 		if (!strcmp(arch, radis_arches[i].name)) {
 			config.arch = radis_arches[i].id;
@@ -445,22 +456,20 @@ static void print_stackptr(struct aop_t *aop, int zero)
 	cons_printf("%3d%c", stack_ptr, changed);
 }
 
-const char *get_section_name_at(u64 addr)
+const char *get_section_name_at(ut64 addr)
 {
-	char *str;
-	str = flag_get_here_filter2(addr - config.vaddr, "maps.", "section.");
-	return str;
+	return flag_get_here_filter2 (addr - config.vaddr, "maps.", "section.");
 }
 
-void radis_str(int arch, const u8 *block, int len, int rows,char *cmd_asm, int flags)
+void radis_str(int arch, const u8 *block, int len, int rows, const char *cmd_asm, int flags)
 {
 	struct aop_t aop;
 	int i,idata,delta;
-	u64 seek = 0;
-	u64 sk = 0;
+	ut64 seek = 0;
+	ut64 sk = 0;
 	int bytes = 0;
 	int myrow = 0;
-	u64 myinc = 0;
+	ut64 myinc = 0;
 	struct data_t *foo;
 	unsigned char b[32];
 	char buf[1024];
@@ -473,7 +482,7 @@ void radis_str(int arch, const u8 *block, int len, int rows,char *cmd_asm, int f
 	int folder = 0; // folder level
 
 	// XXX
-	endian        = (int) config_get("cfg.bigendian");
+	endian        = (int)(size_t) config_get("cfg.bigendian");
 	show_nbytes   = (int) config_get_i("asm.nbytes");
 	if (show_nbytes>16 ||show_nbytes<0)
 		show_nbytes = 16;
@@ -518,8 +527,11 @@ void radis_str(int arch, const u8 *block, int len, int rows,char *cmd_asm, int f
 	if (rrows>0) rrows ++;
 	/* DISASSEMBLY LOOP */
 	while (!config.interrupted) {
-		if (bytes >= length ) break;
-		if (rrows>0 && --rrows == 0) break;
+		if (rrows>0) {
+			--rrows;
+  			if (rrows == 0)
+				break;
+		} else if (bytes >= length ) break;
 		if (bytes>=config.block_size)
 			break;
 		seek = config.seek+bytes + config.vaddr;
@@ -530,10 +542,10 @@ void radis_str(int arch, const u8 *block, int len, int rows,char *cmd_asm, int f
 		D {
 			if (flags & RADIS_FLAGSLINE) {
 				char buf[1024];
-				char *ptr = buf;
+				const char *ptr = buf;
 				//char *ptr = flag_name_by_offset( sk );
 				buf[0]='\0';
-				string_flag_offset(buf, seek, -1);
+				string_flag_offset(NULL, buf, seek, -1);
 				if (ptr == NULL && config.vaddr)
 					ptr = flag_name_by_offset( seek );
 				if (ptr && ptr[0]) {
@@ -552,13 +564,15 @@ void radis_str(int arch, const u8 *block, int len, int rows,char *cmd_asm, int f
 				}
 			}
 			if (flags & RADIS_COMMENTS)
-				myrow+=data_printd(bytes);
+				myrow += data_printd(bytes);
+			if ((reflines) &&  (flags & RADIS_LINES))
+				code_lines_print(reflines, sk, 0);
+			if (data_xrefs_print(seek, -1))
 			if ((reflines) &&  (flags & RADIS_LINES))
 				code_lines_print(reflines, sk, 0);
 		}
 		if (flags & RADIS_SECTION) {
-			char *str;
-			str = get_section_name_at(seek-config.vaddr);
+			const char *str = get_section_name_at(seek-config.vaddr);
 			if (*str)
 				cons_printf("%s:", str);
 		}
@@ -578,7 +592,7 @@ void radis_str(int arch, const u8 *block, int len, int rows,char *cmd_asm, int f
 				//arch_x86_aop((unsigned long)ud_insn_off(&ud_obj), (const unsigned char *)config.block+bytes, &aop);
 				//ud_set_pc(&ud_obj, seek);
 				//arch_x86_aop((unsigned long)ud_insn_off(&ud_obj), (const unsigned char *)b, &aop);
-				arch_x86_aop((u64)seek, (const u8*)block+bytes, &aop);
+				arch_x86_aop((ut64)seek, (const u8*)block+bytes, &aop);
 				myinc += ud_insn_len(&ud_obj);
 				break;
 			case ARCH_ARM16:
@@ -590,6 +604,12 @@ void radis_str(int arch, const u8 *block, int len, int rows,char *cmd_asm, int f
 				// endian stuff here
 				myinc += arch_arm_aop(seek, (const u8 *)block+bytes, &aop);
 				break;
+			case ARCH_Z80:
+				myinc += arch_z80_aop(seek, (const u8 *)block+bytes, &aop);
+				break;
+			case ARCH_8051:
+				myinc += arch_8051_aop(seek, (const u8 *)block+bytes, &aop);
+				break;
 			case ARCH_MIPS:
 				arch_mips_aop(seek, (const u8 *)block+bytes, &aop);
 				myinc += aop.length;
@@ -843,7 +863,7 @@ void radis_str(int arch, const u8 *block, int len, int rows,char *cmd_asm, int f
 			char buf[128];
 			sprintf(buf, "%lld", seek);
 			setenv("HERE", buf, 1);
-			radare_cmd(cmd_asm, 0);
+			radare_cmd_raw (cmd_asm, 0);
 		}
 		if (myinc<1)
 			myinc = 1;
@@ -872,9 +892,9 @@ void radis_str(int arch, const u8 *block, int len, int rows,char *cmd_asm, int f
 				char flag[1024];
 				//const char *flag = nullstr;
 
-				string_flag_offset(flag, seek, -1);
 				if (flag[0]=='\0')
-					string_flag_offset(flag, seek-config.vaddr, -1);
+					string_flag_offset(NULL, flag, seek-config.vaddr, -1);
+				else string_flag_offset(NULL, flag, seek, -1);
 #if 0
 				f = flag_by_offset( seek );
 				if (f != NULL)
@@ -998,7 +1018,7 @@ void radis_str(int arch, const u8 *block, int len, int rows,char *cmd_asm, int f
 
 			/* show references */
 			if (aop.ref) {
-				string_flag_offset(buf, aop.ref, 0);
+				string_flag_offset(NULL, buf, aop.ref, 0);
 				cons_printf(" ; %s",buf);
 				if (buf[0]=='\0')
 					cons_printf("(0x%08llx)",aop.ref);
@@ -1010,11 +1030,11 @@ void radis_str(int arch, const u8 *block, int len, int rows,char *cmd_asm, int f
 				if (aop.jump) {
 					if (++jump_n<10) {
 						jumps[jump_n-1] = aop.jump;
-						if (string_flag_offset(buf, aop.jump-config.vaddr, -1))
+						if (string_flag_offset(NULL, buf, aop.jump-config.vaddr, -1))
 							cons_printf("  ; %d = %s", jump_n, buf);
 						else cons_printf("  ; %d = 0x%08llx", jump_n, aop.jump);
 					} else {
-						if (string_flag_offset(buf, aop.jump-config.vaddr, -1))
+						if (string_flag_offset(NULL, buf, aop.jump-config.vaddr, -1))
 							cons_printf("  ; %s", buf);
 						else cons_printf("  ; 0x%08llx", aop.jump);
 					}
@@ -1051,11 +1071,10 @@ void radis_str(int arch, const u8 *block, int len, int rows,char *cmd_asm, int f
 
 		cons_newline();
 		if (f && f->cmd != NULL)
-			radare_cmd(f->cmd, 0);
-
+			radare_cmd_raw (f->cmd, 0);
 		/* */
 		if (flags & RADIS_FLAGSALL) {
-			u64 ptr = flag_delta_between(sk,sk+myinc);
+			ut64 ptr = flag_delta_between(sk,sk+myinc);
 			if (ptr != 0LL) {
 				myinc = ptr;
 				myrow++;
@@ -1071,7 +1090,7 @@ void radis_str(int arch, const u8 *block, int len, int rows,char *cmd_asm, int f
 void radis_str_e(int arch, const u8 *block, int len, int rows)
 {
 	const char *cmd_asm;
-	int flags, linescall;
+	int flags, linescall, asmflags;
 	int size, bytes, offset, splits, comments, lines, section,
 	traces, reladdr, flagsline, functions, stackptr, flagsall;
 	// TODO: add xrefsto here
@@ -1083,11 +1102,11 @@ void radis_str_e(int arch, const u8 *block, int len, int rows)
 		bytes    = (int) config_get_i("asm.bytes");
 		offset   = (int) config_get_i("asm.offset");
 		functions= (int) config_get_i("asm.functions");
+		asmflags = (int) config_get_i("asm.flags");
 		flagsall = (int) config_get_i("asm.flagsall");
 		if (config.verbose) {
 			section  = (int) config_get_i("asm.section");
 			splits   = (int) config_get_i("asm.split");
-			flags    = (int) config_get_i("asm.flags");
 			flagsline= (int) config_get_i("asm.flagsline");
 			traces   = (int) config_get_i("asm.trace");
 		}
@@ -1107,10 +1126,10 @@ void radis_str_e(int arch, const u8 *block, int len, int rows)
 		if (section) flags |= RADIS_SECTION;
 		if (splits) flags |= RADIS_SPLITS;
 		if (stackptr) flags |= RADIS_STACKPTR;
-		if (flags) flags |= RADIS_FLAGS;
+		if (asmflags) flags |= RADIS_FLAGS;
 		if (flagsline) flags |= RADIS_FLAGSLINE;
 		if (flagsall) flags |= RADIS_FLAGSALL;
-		//if (xrefsto) flags |= RADIS_XREFSTO;
+//		if (xrefsto) flags |= RADIS_XREFSTO;
 		if (lines) flags |= RADIS_LINES;
 		if (linescall) flags |= RADIS_LINESCALL;
 		if (reladdr) flags |= RADIS_RELADDR;
