@@ -89,6 +89,7 @@ static void config_old_init()
 	config.width       = cons_get_columns();
 	config.last_seek   = 0;
 	config.file        = NULL;
+	config.realfile    = 0;
 	config.scrfit      = 1;
 	config.block_size  = DEFAULT_BLOCK_SIZE;
 	config.cursor      = 0;
@@ -102,7 +103,7 @@ static void config_old_init()
 	config.scrdelta    = 0;
 	config.lang        = 0;
 	config.fd          = -1;
-	config.zoom.size   = config.size;
+	config.zoom.to     = config.size;
 	config.zoom.from   = 0;
 	config.zoom.enabled= 0;
 	config.zoom.piece  = config.size/config.block_size;
@@ -141,8 +142,9 @@ void config_list(const char *str)
 	struct list_head *i;
 	int len = 0;
 
-	if (str&&str[0]) {
-		str = strclean(str);
+	if (str && str[0]) {
+		while(str[0]&&iswhitechar(str[0]))
+			str = str + 1;
 		len = strlen(str);
 	}
 
@@ -151,9 +153,7 @@ void config_list(const char *str)
 		if (str) {
 			if (strncmp(str, bt->name,len) == 0)
 				cons_printf("%s = %s\n", bt->name, bt->value);
-		} else {
-			cons_printf("%s = %s\n", bt->name, bt->value);
-		}
+		} else cons_printf("%s = %s\n", bt->name, bt->value);
 	}
 }
 
@@ -190,7 +190,7 @@ const char *config_get(const char *name)
 	return NULL;
 }
 
-u64 config_get_i(const char *name)
+ut64 config_get_i(const char *name)
 {
 	struct config_node_t *node;
 
@@ -198,10 +198,10 @@ u64 config_get_i(const char *name)
 	if (node) {
 		if (node->i_value != 0)
 			return node->i_value;
-		return (u64)get_math(node->value);
+		return (ut64)get_math(node->value);
 	}
 
-	return (u64)0LL;
+	return (ut64)0LL;
 }
 
 struct config_node_t *config_set(const char *name, const char *value)
@@ -222,7 +222,7 @@ struct config_node_t *config_set(const char *name, const char *value)
 		free(node->value);
 		if (node->flags & CN_BOOL) {
 			int b = (!strcmp(value,"true")||!strcmp(value,"1"));
-			node->i_value = (u64)(b==0)?0:1;
+			node->i_value = (ut64)(b==0)?0:1;
 			//node->value = estrdup(node->value, b?"true":"false");
 			node->value = strdup(b?"true":"false");
 		} else {
@@ -273,7 +273,7 @@ int config_rm(const char *name)
 	return 0;
 }
 
-struct config_node_t *config_set_i(const char *name, const u64 i)
+struct config_node_t *config_set_i(const char *name, const ut64 i)
 {
 	char buf[128];
 	struct config_node_t *node;
@@ -296,7 +296,7 @@ struct config_node_t *config_set_i(const char *name, const u64 i)
 		if (config_new.lock) {
 			eprintf("(locked: no new keys can be created)");
 		} else {
-			sprintf(buf, "%d", (unsigned int)i);//OFF_FMTd, (u64) i);
+			sprintf(buf, "%d", (unsigned int)i);//OFF_FMTd, (ut64) i);
 			node = config_node_new(name, buf);
 			node->flags = CN_RW | CN_OFFT;
 			node->i_value = i;
@@ -354,7 +354,7 @@ void config_eval(char *str)
 			const char * str = config_get(foo);
 			if (config_get_notfound)
 				config_list(name);
-			else cons_printf("%s\n", (((int)str)==1)?"true":(str==0)?"false":str);
+			else cons_printf("%s\n", ((PTRCAST (str))==1)?"true":(str==0)?"false":str);
 		}
 	}
 }
@@ -366,10 +366,10 @@ static int config_bigendian_callback(void *data)
 	return 1;
 }
 
-static int config_scr_interactive_callback(void *data)
+static int config_scr_floodprot_callback(void *data)
 {
 	struct config_node_t *node = data;
-	cons_interactive = node->i_value?1:0;
+	cons_floodprot = node->i_value?1:0;
 	return 1;
 }
 
@@ -384,7 +384,7 @@ static int config_filterfile_callback(void *data)
 {
 	struct config_node_t *node = (struct config_node_t *)data;
 	if (!node->value || node->value[0]=='\0') {
-		efree(&cons_filterline);
+		efree((void **)&cons_filterline);
 	} else cons_filterline = estrdup(cons_filterline, node->value);
 	return 1;
 }
@@ -393,11 +393,19 @@ static int config_teefile_callback(void *data)
 {
 	struct config_node_t *node = (struct config_node_t *)data;
 	if (!node->value || node->value[0]=='\0') {
-		efree(&cons_teefile);
+		efree((void **)&cons_teefile);
 	} else cons_teefile = estrdup(cons_teefile, node->value);
 	return 1;
 }
 
+static int config_ppm_callback(void *data)
+{
+	struct config_node_t *node = (struct config_node_t *)data;
+#if DEBUGGER
+	debug_procpidmem = node->i_value;
+#endif
+}
+
 static int config_zoom_callback(void *data)
 {
 	struct config_node_t *node = (struct config_node_t *)data;
@@ -410,6 +418,23 @@ static int config_zoom_callback(void *data)
 	return 1;
 }
 
+static int config_zoomfrom_callback(void *data)
+{
+	struct config_node_t *node = data;
+	if (node->i_value < 0)
+		return 0;
+	config.zoom.from = node->i_value;
+	return 1;
+}
+
+static int config_zoomto_callback(void *data)
+{
+	struct config_node_t *node = data;
+	if (node->i_value < 0)
+		return 0;
+	config.zoom.to = node->i_value;
+}
+
 static int config_zoombyte_callback(void *data)
 {
 	struct config_node_t *node = data;
@@ -441,48 +466,35 @@ static int config_core_callback(void *data)
 
 	if (!strcmp(node->name, "core.jmp")) {
 		hist_goto(node->value);
-	} else
-		if (!strcmp(node->name, "core.echo")) {
-			cons_printf("%s\n", node->value);
-		} else
-			if (!strcmp(node->name, "core.cmp")) {
-				hist_cmp(node->value);
-			} else
-				if (!strcmp(node->name, "core.load")) {
-					hist_load(node->value);
-				} else
-					if (!strcmp(node->name, "core.dump")) {
-						hist_dump(node->value);
-					} else
-						if (!strcmp(node->name, "core.list")) {
-							hist_show();
-						} else
-							if (!strcmp(node->name, "core.reset")) {
-								hist_reset();
-							} else
-								if (!strcmp(node->name, "core.je")) {
-									hist_cgoto(node->value, OP_JE);
-								} else
-									if (!strcmp(node->name, "core.jne")) {
-										hist_cgoto(node->value, OP_JNE);
-									} else
-										if (!strcmp(node->name, "core.ja")) {
-											hist_cgoto(node->value, OP_JA);
-										} else
-											if (!strcmp(node->name, "core.jb")) {
-												hist_cgoto(node->value, OP_JB);
-											} else
-												if (!strcmp(node->name, "core.loop")) {
-													hist_loop(node->value);
-												} else
-													if (!strcmp(node->name, "core.break")) {
-														// ignored
-														//hist_break();
-													} else
-														if (!strcmp(node->name, "core.label")) {
-															hist_add_label(node->value);
-														}
-													// TODO needs more work
+	} else if (!strcmp(node->name, "core.echo")) {
+		cons_printf("%s\n", node->value);
+	} else if (!strcmp(node->name, "core.cmp")) {
+		hist_cmp(node->value);
+	} else if (!strcmp(node->name, "core.load")) {
+		hist_load(node->value);
+	} else if (!strcmp(node->name, "core.dump")) {
+		hist_dump(node->value);
+	} else if (!strcmp(node->name, "core.list")) {
+		hist_show();
+	} else if (!strcmp(node->name, "core.reset")) {
+		hist_reset();
+	} else if (!strcmp(node->name, "core.je")) {
+		hist_cgoto(node->value, OP_JE);
+	} else if (!strcmp(node->name, "core.jne")) {
+		hist_cgoto(node->value, OP_JNE);
+	} else if (!strcmp(node->name, "core.ja")) {
+		hist_cgoto(node->value, OP_JA);
+	} else if (!strcmp(node->name, "core.jb")) {
+		hist_cgoto(node->value, OP_JB);
+	} else if (!strcmp(node->name, "core.loop")) {
+		hist_loop(node->value);
+	} else if (!strcmp(node->name, "core.break")) {
+		// ignored
+		//hist_break();
+	} else if (!strcmp(node->name, "core.label")) {
+		hist_add_label(node->value);
+	}
+	// TODO needs more work
 }
 #endif
 
@@ -609,7 +621,7 @@ static int config_bits_callback(void *data)
 {
 	char buf[128];
 	struct config_node_t *node = data;
-	char *arch = config_get("asm.arch");
+	const char *arch = config_get("asm.arch");
 	if (strstr(arch, "intel")) {
 		if ((node->i_value != 8 && node->i_value != 16)
 		&& (node->i_value != 32 && node->i_value != 64))
@@ -630,8 +642,12 @@ static int config_debug_callback(void *data)
 {
 	struct config_node_t *node = data;
 
-	if (node) // && node->i_value)
+	if (node) { // && node->i_value)
 		config.debug = node->i_value;
+	}
+	if (!config.debug) {
+#warning TODO import register values from debugger into the vm
+	}
 
 	return 0;
 }
@@ -689,6 +705,24 @@ static int config_color_callback(void *data)
 	return 1;
 }
 
+static int config_unksize_callback(void *data)
+{
+	static ut64 backup = -1LL;
+	struct config_node_t *node = data;
+	if (node)
+		config.unksize = (int)node->i_value;
+	if (config.unksize) {
+		if (config.size != -1)
+			backup = config.size;
+		config.size = -1;
+		config_set_i("cfg.limit", -1);
+	} else {
+		config.size = backup;
+		config_set_i("cfg.limit", 0);
+	}
+	return 1;
+}
+
 #if 0
 int config_asm_dwarf(void *data)
 {
@@ -706,7 +740,7 @@ static int config_paddr_callback(void *data)
 	struct config_node_t *node = data;
 
 	if (node) {
-		config.paddr = (u64)node->i_value;
+		config.paddr = (ut64)node->i_value;
 		section_set(config.seek, -1, config.paddr, -1, -1, NULL);
 	}
 	return 1;
@@ -717,7 +751,7 @@ static int config_vaddr_callback(void *data)
 	struct config_node_t *node = data;
 
 	if (node) {
-		config.vaddr = (u64)node->i_value;
+		config.vaddr = (ut64)node->i_value;
 		section_set(config.seek, -1, config.vaddr, -1, -1, NULL);
 	}
 	return 1;
@@ -851,6 +885,10 @@ void config_init(int first)
 	config_set("asm.os", "openbsd");
 #elif __Windows__
 	config_set("asm.os", "linux");
+#elif __sun
+	config_set("asm.os", "solaris");
+#else
+	config_set("asm.os", "unknown");
 #endif
 	config_set("asm.split", "true"); // split code blocks
 	config_set("asm.splitall", "false"); // split code blocks
@@ -884,9 +922,25 @@ void config_init(int first)
 	config_set("search.verbose", "true");
 	config_set_i("search.limit", 0);
 
+	node = config_set("dbg.procpidmem", "false");
+	node->callback = &config_ppm_callback;
+
+	config_set("dbg.env_ldso", "false");
+	config_set("file.dbg_env", "");
+	config_set("file.dbg_arg", "");
+
 	config_set("file.id", "false");
+
+	// TODO: file.analyze -> anal.enable ?
 	config_set("file.analyze", "false");
-	config_set("file.analdata", "false");
+//	config_set("anal.enable", "false");
+
+	config_set_i("anal.depth", 3); // dunno why >4 causes bad code analysis :??
+	config_set("anal.data", "false");
+	config_set("anal.funhdr", "false");
+	config_set("anal.push", "false");
+	config_set("anal.calls", "false");
+
 	config_set("file.type", "");
 	config_set("file.flag", "false");
 	config_set("file.trace", "");//trace.log");
@@ -916,11 +970,13 @@ void config_init(int first)
 	config_set("trace.dup", "false");
 	config_set("trace.cmtregs", "false");
 
-	config_set("cfg.editor", "vi");
+	config_set("cfg.editor", "vim");
 	node = config_set("cfg.debug", "false");
 	node->callback = &config_debug_callback;
 	config_set("cfg.noscript", "false");
 	config_set_i("cfg.tzdelta", 0);
+	node = config_set("cfg.unksize", config.unksize?"true":"false");
+	node->callback = &config_unksize_callback;
 	config_set("cfg.sections", "true");
 	config_set("cfg.encoding", "ascii"); // cp850
 	config_set_i("cfg.delta", 4096); // cp850
@@ -935,7 +991,6 @@ void config_init(int first)
 
 	config.endian = config_get_i("cfg.bigendian");
 	config_set("cfg.inverse", "false");
-	config_set_i("cfg.analdepth", 6);
 	config_set("file.insert", "false");
 	config_set("file.insertblock", "false");
 	config_set("file.undowrite", "true");
@@ -985,6 +1040,7 @@ void config_init(int first)
 	config_set("dbg.stepo", "false"); // step over for !contu (debug_step())
 	config_set("dbg.dwarf", "false");
 	config_set("dbg.maps", "true");
+	config_set("dbg.framereg", "ebp");
 	config_set("dbg.contall", "true");
 	config_set("dbg.sections", "true");
 	config_set("dbg.strings", "false");
@@ -1013,12 +1069,12 @@ void config_init(int first)
 	config_set("dbg.hwbp", "true"); // hardware breakpoints by default // ALSO BSD
 #endif
 	config_set("dbg.bep", "loader"); // loader, main
-	config_set("dir.home", getenv("HOME"));
+	config_set("dir.home", get_home_directory());
 
 	/* dir.monitor */
 	ptr = getenv("MONITORPATH");
 	if (ptr == NULL) {
-		sprintf(buf, "%s/.radare/monitor/", getenv("HOME"));
+		sprintf(buf, "%s/.radare/monitor/", get_home_directory());
 		ptr = (const char *)&buf;
 	}
 	config_set("dir.monitor", ptr);
@@ -1027,7 +1083,7 @@ void config_init(int first)
 	/* dir.spcc */
 	ptr = getenv("SPCCPATH");
 	if (ptr == NULL) {
-		sprintf(buf, "%s/.radare/spcc/", getenv("HOME"));
+		sprintf(buf, "%s/.radare/spcc/", get_home_directory());
 		ptr = buf;
 	}
 	config_set("dir.spcc", ptr);
@@ -1037,7 +1093,7 @@ void config_init(int first)
 #else
 	config_set("dir.plugins", LIBDIR"/radare/");
 #endif
-	snprintf(buf, 1023, "%s/.radare/rdb/", getenv("HOME"));
+	snprintf(buf, 1023, "%s/.radare/rdb/", get_home_directory());
 	config_set("dir.project", buf); // ~/.radare/rdb/
 	config_set("dir.tmp", get_tmp_dir());
 	config_set("graph.bgcolor", "white");
@@ -1065,7 +1121,9 @@ void config_init(int first)
 	node = config_set("zoom.enable", "false");
 	node->callback = &config_zoom_callback;
 	node = config_set_i("zoom.from", 0);
+	node->callback = &config_zoomfrom_callback;
 	node = config_set_i("zoom.to", config.size);
+	node->callback = &config_zoomto_callback;
 	node = config_set("zoom.byte", "head");
 	node->callback = &config_zoombyte_callback;
 
@@ -1074,39 +1132,40 @@ void config_init(int first)
 	node = config_set("scr.html", "false");
 	node->callback = &config_scrhtml_callback;
 	config_set_i("scr.accel", 0);
-	node = config_set("scr.interactive", "true");
-	node->callback = &config_scr_interactive_callback;
+	node = config_set("scr.floodprot", "true");
+	node->callback = &config_scr_floodprot_callback;
 
 	node = config_set("scr.palette", cons_palette_default);
 	node->callback = &config_palette_callback;
 	cons_palette_init(config_get("scr.palette"));
+
 #define config_set_scr_pal(x,y) \
 	node = config_set("scr.pal."x"", y); \
 	node->callback = &config_palette_callback; \
-	node->callback(node);
+	node->callback(node)
 
-	config_set_scr_pal("prompt","yellow")
-		config_set_scr_pal("default","white")
-		config_set_scr_pal("changed","green")
-		config_set_scr_pal("jumps","green")
-		config_set_scr_pal("calls","green")
-		config_set_scr_pal("push","green")
-		config_set_scr_pal("trap","red")
-		config_set_scr_pal("cmp","yellow")
-		config_set_scr_pal("ret","red")
-		config_set_scr_pal("nop","gray")
-		config_set_scr_pal("metadata","gray")
-		config_set_scr_pal("header","green")
-		config_set_scr_pal("printable","bwhite")
-		config_set_scr_pal("lines0","white")
-		config_set_scr_pal("lines1","yellow")
-		config_set_scr_pal("lines2","bwhite")
-		config_set_scr_pal("address","green")
-		config_set_scr_pal("ff","red")
-		config_set_scr_pal("00","white")
-		config_set_scr_pal("7f","magenta")
+	config_set_scr_pal("prompt","yellow");
+	config_set_scr_pal("default","white");
+	config_set_scr_pal("changed","green");
+	config_set_scr_pal("jumps","green");
+	config_set_scr_pal("calls","green");
+	config_set_scr_pal("push","green");
+	config_set_scr_pal("trap","red");
+	config_set_scr_pal("cmp","yellow");
+	config_set_scr_pal("ret","red");
+	config_set_scr_pal("nop","gray");
+	config_set_scr_pal("metadata","gray");
+	config_set_scr_pal("header","green");
+	config_set_scr_pal("printable","bwhite");
+	config_set_scr_pal("lines0","white");
+	config_set_scr_pal("lines1","yellow");
+	config_set_scr_pal("lines2","bwhite");
+	config_set_scr_pal("address","green");
+	config_set_scr_pal("ff","red");
+	config_set_scr_pal("00","white");
+	config_set_scr_pal("7f","magenta");
 
-		config_set("scr.seek", "eip");
+	config_set("scr.seek", "eip");
 	node = config_set("scr.color", (config.color)?"true":"false");
 	node->callback = &config_color_callback;
 	config_set("scr.grephigh", "");
@@ -1204,7 +1263,6 @@ void config_visual_menu()
 {
 	char cmd[1024];
 	struct list_head *pos;
-#define MAX_FORMAT 2
 	const char *ptr;
 	char *fs = NULL;
 	char *fs2 = NULL;
@@ -1218,7 +1276,7 @@ void config_visual_menu()
 	char old[1024];
 	old[0]='\0';
 
-	while(1) {
+	while (1) {
 		cons_gotoxy(0,0);
 		cons_clear();
 
@@ -1230,149 +1288,165 @@ void config_visual_menu()
 			last_print_format = tmp;
 		}
 
+		switch (menu) {
+		case 0: // flag space
+			cons_printf("\n Eval spaces:\n\n");
+			hit = 0;
+			j = i = 0;
+			list_for_each(pos, &(config_new.nodes)) {
+				struct config_node_t *bt = list_entry(pos, struct config_node_t, list);
+				if (option==i) {
+					fs = bt->name;
+					hit = 1;
+				}
+				show = 0;
+				if (old[0]=='\0') {
+					str_ccpy(old, bt->name, '.');
+					show = 1;
+				} else if (strccmp(old, bt->name, '.')) {
+					str_ccpy(old, bt->name, '.');
+					show = 1;
+				}
+
+				if (show) {
+					if( (i >=option-delta) && ((i<option+delta)||((option<delta)&&(i<(delta<<1))))) {
+						cons_printf(" %c  %s\n", (option==i)?'>':' ', old);
+						j++;
+					}
+					i++;
+				}
+			}
+			if (!hit && j>0) {
+				option = j-1;
+				continue;
+			}
+			cons_printf("\n Sel:%s \n\n", fs);
+			break;
+		case 1: // flag selection
+			cons_printf("\n Eval variables: (%s)\n\n", fs);
+			hit = 0;
+			j = i = 0;
+			// TODO: cut -d '.' -f 1 | sort | uniq !!!
+			list_for_each(pos, &(config_new.nodes)) {
+				struct config_node_t *bt = list_entry(pos, struct config_node_t, list);
+				if (option==i) {
+					fs2 = bt->name;
+					hit = 1;
+				}
+				if (!strccmp(bt->name, fs, '.')) {
+					if( (i >=option-delta) && ((i<option+delta)||((option<delta)&&(i<(delta<<1))))) {
+						// TODO: Better align
+						cons_printf(" %c  %s = %s\n", (option==i)?'>':' ', bt->name, bt->value);
+						j++;
+					}
+					i++;
+				}
+			}
+			if (!hit && j>0) {
+				option = i-1;
+				continue;
+			}
+			if (fs2 != NULL)
+				cons_printf("\n Selected: %s\n\n", fs2);
+		}
+
 		if (fs&&!memcmp(fs, "asm.", 4))
 			radare_cmd_raw("pd 5", 0);
-
-		switch(menu) {
-			case 0: // flag space
-				cons_printf("\n Eval spaces:\n\n");
-				hit = 0;
-				j = i = 0;
-				list_for_each(pos, &(config_new.nodes)) {
-					struct config_node_t *bt = list_entry(pos, struct config_node_t, list);
-					if (option==i) {
-						fs = bt->name;
-						hit = 1;
-					}
-					show = 0;
-					if (old[0]=='\0') {
-						strccpy(old, bt->name, '.');
-						show = 1;
-					} else if (strccmp(old, bt->name, '.')) {
-						strccpy(old, bt->name, '.');
-						show = 1;
-					}
-
-					if (show) {
-						if( (i >=option-delta) && ((i<option+delta)||((option<delta)&&(i<(delta<<1))))) {
-							cons_printf(" %c  %s\n", (option==i)?'>':' ', old);
-							j++;
-						}
-						i++;
-					}
-				}
-				if (!hit && j>0) {
-					option = j-1;
-					continue;
-				}
-				cons_printf("\n Sel:%s \n\n", fs);
-				break;
-			case 1: // flag selection
-				cons_printf("\n Eval variables: (%s)\n\n", fs);
-				hit = 0;
-				j = i = 0;
-				// TODO: cut -d '.' -f 1 | sort | uniq !!!
-				list_for_each(pos, &(config_new.nodes)) {
-					struct config_node_t *bt = list_entry(pos, struct config_node_t, list);
-					if (option==i) {
-						fs2 = bt->name;
-						hit = 1;
-					}
-					if (!strccmp(bt->name, fs, '.')) {
-						if( (i >=option-delta) && ((i<option+delta)||((option<delta)&&(i<(delta<<1))))) {
-							// TODO: Better align
-							cons_printf(" %c  %s = %s\n", (option==i)?'>':' ', bt->name, bt->value);
-							j++;
-						}
-						i++;
-					}
-				}
-				if (!hit && j>0) {
-					option = i-1;
-					continue;
-				}
-				if (fs2 != NULL)
-					cons_printf("\n Selected: %s\n\n", fs2);
-		}
 		cons_flushit();
-		ch = cons_readchar();
-		ch = cons_get_arrow(ch); // get ESC+char, return 'hjkl' char
+ 		// get ESC+char, return 'hjkl' char
+		ch = cons_get_arrow (cons_readchar ());
 		switch(ch) {
-			case 'j':
-				option++;
-				break;
-			case 'k':
-				if (--option<0)
-					option = 0;
-				break;
-			case 'h':
-			case 'b': // back
-				menu = 0;
-				option = _option;
-				break;
-			case 'q':
-				if (menu<=0) return; menu--;
-				break;
-			case '*':
-			case '+':
+		case 'J':
+			option+=5;
+			if (option>=i)
+				option = i-1;
+			break;
+		case 'K':
+			option-=5;
+			if (--option<0)
+				option = 0;
+			break;
+		case 'j':
+			if (++option>=i)
+				option = i-1;
+			break;
+		case 'k':
+			if (--option<0)
+				option = 0;
+			break;
+		case 'h':
+		case 'b': // back
+			menu = 0;
+			option = _option;
+			break;
+		case 'q':
+			if (menu<=0)
+				return;
+			menu--;
+			break;
+		case '*':
+		case '+':
+			if (fs2 != NULL)
+				config_visual_hit_i(fs2, +1);
+			continue;
+		case '/':
+		case '-':
+			if (fs2 != NULL)
+				config_visual_hit_i(fs2, -1);
+			continue;
+		case 'l':
+		case 'e': // edit value
+		case ' ':
+		case '\r':
+		case '\n': // never happens
+			if (menu == 1) {
 				if (fs2 != NULL)
-					config_visual_hit_i(fs2, +1);
-				continue;
-			case '/':
-			case '-':
-				if (fs2 != NULL)
-					config_visual_hit_i(fs2, -1);
-				continue;
-			case 'l':
-			case 'e': // edit value
-			case ' ':
-			case '\r':
-			case '\n': // never happens
-				if (menu == 1) {
-					if (fs2 != NULL)
-						config_visual_hit(fs2);
-				} else {
-					flag_space_set(fs);
-					menu = 1;
-					_option = option;
-					option = 0;
-				}
-				break;
-			case '?':
-				cons_clear00();
-				cons_printf("\nVe: Visual Eval help:\n\n");
-				cons_printf(" q     - quit menu\n");
-				cons_printf(" j/k   - down/up keys\n");
-				cons_printf(" h/b   - go back\n");
-				cons_printf(" e/' ' - edit/toggle current variable\n");
-				cons_printf(" +/-   - increase/decrease numeric value\n");
-				cons_printf(" :     - enter command\n");
-				cons_flushit();
-				cons_any_key();
-				break;
-			case ':':
-				cons_set_raw(0);
+					config_visual_hit (fs2);
+			} else {
+				flag_space_set (fs);
+				menu = 1;
+				_option = option;
+				option = 0;
+			}
+			break;
+		case '?':
+			cons_clear00 ();
+			cons_strcat (
+			"\nVe: Visual Eval help:\n\n"
+			" q     - quit menu\n"
+			" ?     - show this help\n"
+			" j/k   - down/up keys\n"
+			" J/K   - down/up fast keys (same as re/av.page)\n"
+			" h/b   - go back\n"
+			" e/' ' - edit/toggle current variable\n"
+			" +/-   - increase/decrease numeric value\n"
+			" :     - enter command\n");
+			cons_flushit ();
+			cons_any_key ();
+			break;
+		case ':':
+			cons_set_raw(0);
 #if HAVE_LIB_READLINE
-				char *ptr = readline(VISUAL_PROMPT);
-				if (ptr) {
-					strncpy(cmd, ptr, sizeof(cmd));
-					radare_cmd(cmd, 1);
-					free(ptr);
-				}
+			char *ptr = readline (VISUAL_PROMPT);
+			if (ptr) {
+				strncpy (cmd, ptr, sizeof(cmd));
+				radare_cmd (cmd, 1);
+				free (ptr);
+			}
 #else
+			cmd[0] = '\0';
+			dl_prompt = ":> ";
+			if (cons_fgets (cmd, 1000, 0, NULL) <0)
 				cmd[0]='\0';
-				dl_prompt = ":> ";
-				if (cons_fgets(cmd, 1000, 0, NULL) <0)
-					cmd[0]='\0';
-				//line[strlen(line)-1]='\0';
-				radare_cmd(cmd, 1);
+			//line[strlen(line)-1]='\0';
+			radare_cmd (cmd, 1);
 #endif
-				cons_set_raw(1);
-				if (cmd[0])
-					cons_any_key();
-				cons_gotoxy(0,0);
-				cons_clear();
-				continue;
+			cons_set_raw (1);
+			if (cmd[0])
+				cons_any_key ();
+			cons_gotoxy (0,0);
+			cons_clear ();
+			continue;
 		}
 	}
 }
