@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2008, 2009
+ * Copyright (C) 2008, 2009, 2010
  *       pancake <@youterm.com>
  *
  * radare is free software; you can redistribute it and/or modify
@@ -42,7 +42,7 @@ static char *cons_buffer = NULL;
 char *cons_filterline = NULL;
 char *cons_teefile = NULL;
 int cons_is_html = 0;
-int cons_interactive = 1;
+int cons_floodprot = 1;
 int _print_fd = 1;
 int cons_lines = 0;
 int cons_noflush = 0;
@@ -53,7 +53,7 @@ void stdout_open(char *file)
 {
 	int fd = open(file, O_RDWR|O_CREAT, 0644);
 	if (fd==-1) {
-eprintf("CANNOT OPEN FILE(%s)\n", file);
+		eprintf("stdout_open: Cannot open '%s'\n", file);
 		return;
 	}
 	cons_stdout_file = fd;
@@ -124,10 +124,8 @@ const char *cons_color_names[CONS_COLORS_SIZE+1] = {
 
 void cons_invert(int set)
 {
-  if (set)
-    cons_strcat("\x1b[7m");
-  else
-    cons_strcat("\x1b[0m");
+	if (set) cons_strcat("\x1b[7m");
+	else cons_strcat("\x1b[0m");
 }
 
 const char *cons_colors[CONS_COLORS_SIZE+1] = {
@@ -298,16 +296,16 @@ int cons_readchar()
 #if __WINDOWS__
 	DWORD out;
 	BOOL ret;
-	LPDWORD mode;
+	DWORD mode;
 	HANDLE h = GetStdHandle(STD_INPUT_HANDLE);
-	GetConsoleMode(h, &mode);
-	SetConsoleMode(h, 0); // RAW
+	GetConsoleMode (h, &mode);
+	SetConsoleMode (h, 0); // RAW
 	ret = ReadConsole(h, buf,1, &out, NULL);
 	if (!ret)
 		return -1;
-	SetConsoleMode(h, mode);
+	SetConsoleMode (h, mode);
 #else
-	if (read(0, buf, 1)==-1)
+	if (read (0, buf, 1)==-1)
 		return -1;
 #endif
 	return buf[0];
@@ -816,10 +814,10 @@ begin:
 			if (tok) {
 				toklen = strlen(tok);
 				len -= (linelen-toklen);
-				strbcpy(buf, tok);
+				str_cpy(buf, tok);
 				memcpy(buf+toklen, "\n", 1);
 				if (!n) return len;
-				strbcpy(buf+toklen+1, buf+linelen); //n+1);
+				str_cpy(buf+toklen+1, buf+linelen); //n+1);
 				buf = buf+toklen+1;
 				goto begin;
 			}
@@ -837,7 +835,7 @@ begin:
 
 	if (donotline) {
 		/* ok */
-		strbcpy(buf, n+1);
+		str_cpy(buf, n+1);
 		len-=linelen;
 		goto begin;
 	} else {
@@ -856,10 +854,10 @@ begin:
 			if (tok) {
 				toklen = strlen(tok);
 				len -= (linelen-toklen);
-				strbcpy(buf, tok);
+				str_cpy(buf, tok);
 				memcpy(buf+toklen, "\n", 1);
 				if (!n) return len;
-				strbcpy(buf+toklen+1, buf+linelen); //n+1);
+				str_cpy(buf+toklen+1, buf+linelen); //n+1);
 				buf = buf+toklen+1;
 				goto begin;
 			}
@@ -887,8 +885,8 @@ void cons_grep(const char *str)
 	/* set grep string */
 	greptoken = -1;
 	grepline = -1;
-	efree(&grepstr);
-	efree(&grephigh);
+	efree((void **)&grepstr);
+	efree((void **)&grephigh);
 
 	if (str != NULL && *str) {
 		if (*str == '!') {
@@ -925,8 +923,8 @@ void cons_grep(const char *str)
 	} else {
 		greptoken = -1;
 		grepline = -1;
-		efree(&grepstr);
-		efree(&grephigh);
+		efree((void **)&grepstr);
+		efree((void **)&grephigh);
 	}
 #if 0
 	if (grephigh == NULL || *grephigh == '\0')
@@ -938,8 +936,8 @@ void cons_grepbuf_end()
 {
 	greptoken = -1;
 	grepline = -1;
-	efree(&grepstr);
-	efree(&grephigh);
+	efree((void **)&grepstr);
+	efree((void **)&grephigh);
 }
 
 char *str_replace(char *str, char *from, char *to, int str_len)
@@ -1028,14 +1026,13 @@ int ansistrlen(const char *str)
 
 char *ansistrchrn(char *str, int n)
 {
-	int i=0, len = 0;
-	while(str[i]) {
+	int i, len = 0;
+	for (i=0;str[i];i++) {
 		if (n == len)
 			break;
 		if (str[i]==0x1b && str[i+1]=='[')
 			for(++i;str[i]&&str[i]!='J'&&str[i]!='m'&&str[i]!='H';i++);
 		else len++;
-		i++;
 	}
 	return str+i;
 }
@@ -1081,12 +1078,12 @@ void cons_fitbuf(char *buf, int len)
 			if (next>=buf+linelen) {
 				buf[0]='\n';
 				len -= linelen;
-				if (ptr) strbcpy(buf+1, ptr+1);
+				if (ptr) str_cpy(buf+1, ptr+1);
 				buf = buf+1;
 			} else {
 				len -= cons_skipx;
-				strbcpy(buf, next);
-				//strbcpy(buf, buf+cons_skipx);
+				str_cpy(buf, next);
+				//str_cpy(buf, buf+cons_skipx);
 			}
 			if (len<=0) break;
 		}
@@ -1095,7 +1092,7 @@ void cons_fitbuf(char *buf, int len)
 			linelen = (int)(ptr-buf);
 			len -= linelen;
 			if (lines>config.scrdelta+3 && cons_skipy && cons_skipy>=lines) {
-				strbcpy(buf, ptr+1);
+				str_cpy(buf, ptr+1);
 				lines++;
 				continue;
 			}
@@ -1107,8 +1104,8 @@ void cons_fitbuf(char *buf, int len)
 			if (linelen>cols) {
 				next = ansistrchrn(buf,cols);
 				next[0]='\n';
-				//strbcpy(buf+cols+1, ptr+1); // ok for b&w
-				strbcpy(next+1, ptr+1);
+				//str_cpy(buf+cols+1, ptr+1); // ok for b&w
+				str_cpy(next+1, ptr+1);
 				buf = next+1;
 			} else {
 				ptr[0]='\n';
@@ -1145,25 +1142,24 @@ void cons_render()
 {
 	FILE *fd;
 	char buf[1024];
-	int i,j;
-	int lines_counter = 0;
+	int i, j, lines_counter = 0;
 
 	if (cons_noflush)
 		return;
 
-	if (cons_interactive && cons_buffer) {
+	if (cons_floodprot && cons_buffer) {
 		cons_buffer_len = strlen(cons_buffer);
 		if (cons_buffer && (cons_buffer_len > CONS_MAX_USER)) {
-			if (yesno('n', "Do you want to print %d bytes? (y/N)", cons_buffer_len)== 0) {
+			eprintf("NOTE: Use 'e scr.floodprot=false' to disable this message\n");
+			if (!yesno('n', "Do you want to print %d bytes? (y/N)", cons_buffer_len)) {
 				cons_reset();
 				return;
 			}
 		}
 	}
 	if (grephigh == NULL) {
-		grephigh = config_get("scr.grephigh");
+		grephigh = (char *)config_get("scr.grephigh");
 		if (grephigh &&*grephigh) {
-			char *new, *old;
 			char bat[64];
 			sprintf(bat, "\x1b[7m%s\x1b[0m", grephigh);
 			cons_buffer = str_replace(cons_buffer, grephigh, bat, cons_buffer_sz);
@@ -1171,7 +1167,6 @@ void cons_render()
 		grephigh = NULL;
 	} else {
 		if (grephigh &&*grephigh) {
-			char *new, *old;
 			char bat[64];
 			sprintf(bat, "\x1b[7m%s\x1b[0m", grephigh);
 			cons_buffer = str_replace(cons_buffer, grephigh, bat, cons_buffer_sz);
@@ -1463,8 +1458,7 @@ int cons_get_real_columns()
 int yesno(int def, const char *fmt, ...)
 {
 	va_list ap;
-	int key = def;
-
+	char key = (char)def;
 	if (config.visual)
 		key='y';
 	else D {
@@ -1478,9 +1472,7 @@ int yesno(int def, const char *fmt, ...)
 		cons_set_raw(0);
 		if (key=='\n'||key=='\r')
 			key = def;
-	} else
-		key = 'y';
-
+	} else key = 'y';
 	return key=='y';
 }
 #endif
