@@ -40,7 +40,41 @@ static struct list_head data;
 static struct list_head comments;
 static struct list_head xrefs;
 
-int data_set_len(u64 off, u64 len)
+ut64 var_functions_show(int idx)
+{
+	const char *mark = nullstr;
+	int i = 0;
+	struct list_head *pos;
+	ut64 seek = config.seek;
+	ut64 addr = config.seek;
+	int window = 15;
+	int wdelta = (idx>5)?idx-5:0;
+
+	list_for_each(pos, &data) {
+		struct data_t *d = (struct data_t *)list_entry(pos, struct data_t, list);
+		if (d->type == DATA_FUN) {
+			if (i>=wdelta) {
+				if (i> window+wdelta) {
+					cons_printf("   ...\n");
+					break;
+				}
+				if (seek > d->from+config.vaddr && seek < d->to+config.vaddr)
+					mark = "<SEEK IS HERE>";
+				else mark = nullstr;
+				if (idx == i) {
+					addr = d->from;
+					cons_printf(" * ");
+				} else cons_printf("   ");
+				cons_printf("0x%08llx (%s) %s\n", d->from+config.vaddr,
+					flag_name_by_offset(d->from), mark);
+			}
+			i++;
+		}
+	}
+	return addr;
+}
+
+int data_set_len(ut64 off, ut64 len)
 {
 	struct list_head *pos;
 	list_for_each(pos, &data) {
@@ -54,10 +88,10 @@ int data_set_len(u64 off, u64 len)
 	return -1;
 }
 
-u64 data_prev(u64 off, int type)
+ut64 data_prev(ut64 off, int type)
 {
 	struct list_head *pos;
-	u64 ret = 0;
+	ut64 ret = 0;
 
 	list_for_each(pos, &data) {
 		struct data_t *d = (struct data_t *)list_entry(pos, struct data_t, list);
@@ -69,10 +103,10 @@ u64 data_prev(u64 off, int type)
 	return ret;
 }
 
-u64 data_prev_size(u64 off, int type)
+ut64 data_prev_size(ut64 off, int type)
 {
 	struct list_head *pos;
-	u64 ret = 0;
+	ut64 ret = 0;
 
 	list_for_each(pos, &data) {
 		struct data_t *d = (struct data_t *)list_entry(pos, struct data_t, list);
@@ -84,19 +118,20 @@ u64 data_prev_size(u64 off, int type)
 	return ret;
 }
 
-int data_get_fun_for(u64 addr, u64 *from, u64 *to)
+int data_get_fun_for(ut64 addr, ut64 *from, ut64 *to)
 {
 	struct list_head *pos;
 	int n_functions = 0;
 	int n_xrefs = 0;
 	int n_dxrefs = 0;
 	struct data_t *rd = NULL;
-	u64 lastfrom = 0LL;
+	ut64 lastfrom = 0LL;
 
 	list_for_each(pos, &data) {
 		struct data_t *d = (struct data_t *)list_entry(pos, struct data_t, list);
 		if (d->type == DATA_FUN) {
-			if (d->from < addr && d->from > lastfrom) {
+			//if (d->from < addr && d->from > lastfrom) {
+			if (d->from < addr && addr < d->to ) { //&& d->from > lastfrom) {
 				rd = d;
 			}
 		}
@@ -142,7 +177,7 @@ void data_info()
 	cons_printf("code_xrefs: %d\n", n_xrefs);
 }
 
-int data_set(u64 off, int type)
+int data_set(ut64 off, int type)
 {
 	struct list_head *pos;
 	list_for_each(pos, &data) {
@@ -155,7 +190,7 @@ int data_set(u64 off, int type)
 	return -1;
 }
 
-struct data_t *data_add_arg(u64 off, int type, const char *arg)
+struct data_t *data_add_arg(ut64 off, int type, const char *arg)
 {
 	struct data_t *d;
 	if (arg == NULL)
@@ -166,7 +201,7 @@ struct data_t *data_add_arg(u64 off, int type, const char *arg)
 	return d;
 }
 
-void data_del(u64 addr, int type,int len/* data or code */)
+void data_del(ut64 addr, int type, int len/* data or code */)
 {
 	struct data_t *d;
 	struct list_head *pos;
@@ -179,9 +214,9 @@ void data_del(u64 addr, int type,int len/* data or code */)
 	}
 }
 
-struct data_t *data_add(u64 off, int type)
+struct data_t *data_add(ut64 off, int type)
 {
-	u64 tmp;
+	ut64 tmp;
 	struct data_t *d = NULL;
 	struct list_head *pos;
 
@@ -228,9 +263,9 @@ struct data_t *data_add(u64 off, int type)
 	return d;
 }
 
-u64 data_seek_to(u64 offset, int type, int idx)
+ut64 data_seek_to(ut64 offset, int type, int idx)
 {
-	u64 ret = 0ULL;
+	ut64 ret = 0ULL;
 	struct list_head *pos;
 	int i = 0;
 	idx--;
@@ -248,7 +283,7 @@ u64 data_seek_to(u64 offset, int type, int idx)
 	return ret;
 }
 
-struct data_t *data_get(u64 offset)
+struct data_t *data_get(ut64 offset)
 {
 	struct list_head *pos;
 	list_for_each(pos, &data) {
@@ -259,7 +294,7 @@ struct data_t *data_get(u64 offset)
 	return NULL;
 }
 
-struct data_t *data_get_range(u64 offset)
+struct data_t *data_get_range(ut64 offset)
 {
 	struct list_head *pos;
 	list_for_each(pos, &data) {
@@ -271,7 +306,7 @@ struct data_t *data_get_range(u64 offset)
 }
 
 /* TODO: OPTIMIZE: perform cache here */
-struct data_t *data_get_between(u64 from, u64 to)
+struct data_t *data_get_between(ut64 from, ut64 to)
 {
 	int hex = 0;
 	int str = 0;
@@ -350,7 +385,7 @@ struct data_t *data_get_between(u64 from, u64 to)
 	return &ret;
 }
 
-int data_type_range(u64 offset)
+int data_type_range(ut64 offset)
 {
 	struct data_t *d = data_get_range(offset);
 	if (d != NULL)
@@ -358,7 +393,7 @@ int data_type_range(u64 offset)
 	return -1;
 }
 
-int data_type(u64 offset)
+int data_type(ut64 offset)
 {
 	struct list_head *pos;
 	list_for_each(pos, &data) {
@@ -369,7 +404,7 @@ int data_type(u64 offset)
 	return -1;
 }
 
-int data_end(u64 offset)
+int data_end(ut64 offset)
 {
 	struct list_head *pos;
 	list_for_each(pos, &data) {
@@ -380,7 +415,7 @@ int data_end(u64 offset)
 	return -1;
 }
 
-int data_size(u64 offset)
+int data_size(ut64 offset)
 {
 	struct list_head *pos;
 	list_for_each(pos, &data) {
@@ -420,7 +455,7 @@ int data_list(const char *mask)
 	list_for_each(pos, &data) {
 		d = (struct data_t *)list_entry(pos, struct data_t, list);
 		label[0]='\0';
-		string_flag_offset(label, d->from, 0);
+		string_flag_offset(NULL, label, d->from, 0);
 		arg = NULL;
 		switch(d->type) {
 		case DATA_FOLD_O: strcpy(pfx,"Cu"); break;
@@ -448,7 +483,7 @@ int data_list(const char *mask)
 	return 0;
 }
 /* -- metadata -- */
-int data_xrefs_print(u64 addr, int type)
+int data_xrefs_print(ut64 addr, int type)
 {
 	char str[1024];
 	int n = 0;
@@ -459,12 +494,25 @@ int data_xrefs_print(u64 addr, int type)
 		x = (struct xrefs_t *)list_entry(pos, struct xrefs_t, list);
 		if (x->addr == addr) {
 			str[0]='\0';
-			string_flag_offset(str, x->from, 0);
+			string_flag_offset(NULL, str, x->from, 0);
 			switch(type) {
-			case 0: if (x->type == type) { cons_printf("; 0x%08llx CODE xref 0x%08llx (%s)\n", addr, x->from, str); n++; } break;
-			case 1: if (x->type == type) { cons_printf("; 0x%08llx DATA xref 0x%08llx (%s)\n", addr, x->from), str; n++; } break;
-			default: { cons_printf("; 0x%08llx %s xref from 0x%08llx (%s)\n",
-				addr, (x->type==1)?"DATA":(x->type==0)?"CODE":"UNKNOWN",x->from, str); n++; };
+			case 0: if (x->type == type) {
+					cons_printf("; 0x%08llx CODE xref 0x%08llx (%s)\n",
+						addr, x->from, str);
+					n++;
+				}
+				break;
+			case 1: if (x->type == type) {
+					cons_printf("; 0x%08llx DATA xref 0x%08llx (%s)\n",
+						addr, x->from), str;
+					n++;
+				}
+				break;
+			default:
+				cons_printf("; 0x%08llx %s xref from 0x%08llx (%s)\n",
+					addr, (x->type==1)?"DATA":(x->type==0)?"CODE":"UNKNOWN",x->from, str);
+				n++;
+				break;
 			}
 		} else
 		if (xrefsto && x->from == addr) {
@@ -476,7 +524,7 @@ int data_xrefs_print(u64 addr, int type)
 	return n;
 }
 
-int data_xrefs_add(u64 addr, u64 from, int type)
+int data_xrefs_add(ut64 addr, ut64 from, int type)
 {
 	struct xrefs_t *x;
 	struct list_head *pos;
@@ -499,7 +547,7 @@ int data_xrefs_add(u64 addr, u64 from, int type)
 	return 1;
 }
 
-int data_xrefs_at(u64 addr)
+int data_xrefs_at(ut64 addr)
 {
 	int ctr = 0;
 	struct xrefs_t *x;
@@ -515,7 +563,7 @@ int data_xrefs_at(u64 addr)
 
 }
 
-void data_xrefs_del(u64 addr, u64 from, int data /* data or code */)
+void data_xrefs_del(ut64 addr, ut64 from, int data /* data or code */)
 {
 	struct xrefs_t *x;
 	struct list_head *pos;
@@ -528,11 +576,11 @@ void data_xrefs_del(u64 addr, u64 from, int data /* data or code */)
 	}
 }
 
-void data_comment_del(u64 offset, const char *str)
+void data_comment_del(ut64 offset, const char *str)
 {
 	struct comment_t *cmt;
 	struct list_head *pos, *n;
-	//u64 off = get_math(str);
+	//ut64 off = get_math(str);
 
 	list_for_each_safe(pos, n, &comments) {
 		cmt = list_entry(pos, struct comment_t, list);
@@ -557,13 +605,12 @@ void data_comment_del(u64 offset, const char *str)
 		    if (offset == cmt->offset) {
 			    if (str[0]=='*') {
 				    free((void *)cmt->comment);
-				    list_del(pos);
+				    list_del(&(cmt->list));
 				    free(cmt);
 				    pos = comments.next; // list_init
-				    //data_comment_del(offset, str);
 			    } else
 			    if (!strcmp(cmt->comment, str)) {
-				    list_del(&(pos));
+				    list_del(&(cmt->list));
 				    return;
 			    }
 		    }
@@ -573,7 +620,7 @@ void data_comment_del(u64 offset, const char *str)
 	}
 }
 
-void data_comment_add(u64 offset, const char *str)
+void data_comment_add(ut64 offset, const char *str)
 {
 	struct comment_t *cmt;
 	char *ptr;
@@ -583,7 +630,7 @@ void data_comment_add(u64 offset, const char *str)
 		return;
 
 	/* avoid dupped comments */
-	data_comment_del(offset, str);
+	data_comment_del(offset, str); // XXX segfault here with -O2
 
 	cmt = (struct comment_t *) malloc(sizeof(struct comment_t));
 	cmt->offset = offset;
@@ -594,7 +641,7 @@ void data_comment_add(u64 offset, const char *str)
 	list_add_tail(&(cmt->list), &(comments));
 }
 
-void data_del_range(u64 from, u64 to)
+void data_del_range(ut64 from, ut64 to)
 {
 	struct list_head *pos, *n;
 
@@ -634,7 +681,7 @@ void data_comment_list(const char *mask)
 	}
 }
 
-void data_xrefs_here(u64 addr)
+void data_xrefs_here(ut64 addr)
 {
 	int count = 0;
 	char label[1024];
@@ -645,7 +692,7 @@ void data_xrefs_here(u64 addr)
 		x = (struct xrefs_t *)list_entry(pos, struct xrefs_t, list);
 		if (addr = x->addr) {
 			label[0]='\0';
-			string_flag_offset(label, x->from, 0);
+			string_flag_offset(NULL, label, x->from, 0);
 			cons_printf("%d %s xref 0x%08llx @ 0x%08llx ; %s\n",
 				count+1, x->type?"data":"code", x->from, x->addr, label);
 			count++;
@@ -665,14 +712,14 @@ void data_xrefs_list(const char *mask)
 	list_for_each(pos, &xrefs) {
 		x = (struct xrefs_t *)list_entry(pos, struct xrefs_t, list);
 		label[0]='\0';
-		string_flag_offset(label, x->from, 0);
+		string_flag_offset(NULL, label, x->from, 0);
 		sprintf(str, "C%c 0x%08llx @ 0x%08llx ; %s\n", x->type?'d':'x', x->from, x->addr, label);
 		if (!mask || !*mask || str_grep(str, mask))
 			cons_printf(str);
 	}
 }
 
-char *data_comment_get(u64 offset, int lines)
+char *data_comment_get(ut64 offset, int lines)
 {
 	struct list_head *pos;
 	char *str = NULL;
@@ -742,8 +789,8 @@ void data_reflines_init(int lines, int linescall)
 
 int data_printd(int delta)
 {
-	int show_lines = (int)config_get("asm.lines");
-	u64 offset = (u64)config.seek + (u64)delta;// - config.vaddr;
+	int show_lines = PTRCAST (config_get("asm.lines"));
+	ut64 offset = (ut64)config.seek + (ut64)delta;// - config.vaddr;
 	int lines = 0;
 	const char *ptr;
 
@@ -780,7 +827,7 @@ int data_var_type_add(const char *typename, int size, const char *fmt)
 int data_var_type_del(const char *typename)
 {
 	struct list_head *pos;
-	u64 ret = 0;
+	ut64 ret = 0;
 
 	if (*typename==' ')typename=typename+1;
 
@@ -799,7 +846,7 @@ int data_var_type_list(const char *mask)
 {
 	char str[128];
 	struct list_head *pos;
-	u64 ret = 0;
+	ut64 ret = 0;
 
 	list_for_each(pos, &vartypes) {
 		struct var_type_t *d = (struct var_type_t *)list_entry(pos, struct var_type_t, list);
@@ -814,7 +861,7 @@ int data_var_type_list(const char *mask)
 struct var_type_t *data_var_type_get(const char *datatype)
 {
 	struct list_head *pos;
-	u64 ret = 0;
+	ut64 ret = 0;
 
 	list_for_each(pos, &vartypes) {
 		struct var_type_t *d = (struct var_type_t *)list_entry(pos, struct var_type_t, list);
