@@ -80,8 +80,16 @@ struct regs_off roff[] = {
 	{0, 0}
 };
 
+int debug_register_list()
+{
+	int i;
+	for(i=0; roff[i].reg != 0; i++) 
+		cons_printf("%s ", roff[i].reg);
+	cons_printf("\n");
+}
+
 /* return register offset */
-u64 get_reg(const char *reg)
+ut64 get_reg(const char *reg)
 {
 	int i;
 
@@ -92,9 +100,9 @@ u64 get_reg(const char *reg)
 	return -1;
 }
 
-u64 arch_syscall(int pid, int sc, ...)
+ut64 arch_syscall(int pid, int sc, ...)
 {
-        u64 ret = (addr_t)-1;
+        ut64 ret = (addr_t)-1;
 #if __linux__ || __BSD__
 	va_list ap;
         regs_t   reg, reg_saved;
@@ -406,14 +414,14 @@ int arch_inject(u8 *data, int size)
 	return 0;
 }
 
-u64 arch_pc(int tid)
+ut64 arch_pc(int tid)
 {
 	regs_t regs;
 	debug_getregs(tid, &regs);
 #if __x86_64__
-	return (u64)R_RIP(regs);
+	return (ut64)R_RIP(regs);
 #else
-	return (u64)(R_EIP(regs)&0xffffffff);
+	return (ut64)(R_EIP(regs)&0xffffffff);
 #endif
 }
 
@@ -425,7 +433,7 @@ int arch_stackanal()
 	%ebp+4 points to ret
 	*/
 	int ret, i,j;
-	u64 last;
+	ut64 last;
 	unsigned long long ptr;
 	unsigned long long ebp2;
 	regs_t regs;
@@ -506,7 +514,7 @@ int arch_stackanal()
 int arch_backtrace()
 {
 	int ret, i,j;
-	u64 last;
+	ut64 last;
 	unsigned long esp;
 	unsigned long ebp2;
 	unsigned char buf[4];
@@ -564,7 +572,7 @@ int arch_backtrace()
 			}
 
 			/* find flag name + offset */
-			string_flag_offset(flagname, addr, 0);
+			string_flag_offset(NULL, flagname, addr, 0);
 
 			/* print out */
 			D {
@@ -675,7 +683,7 @@ struct user_fpxregs_struct regs __attribute__((aligned(16)));
 // NO RAD FOR FPREGS (only 32&64 bit vars, TODO: needs bsd port)
 int arch_print_fpregisters(int rad, const char *mask)
 {
-	int ret = 0;
+	int t, ret = 0;
 
 #if __linux__
 	int i;
@@ -688,23 +696,37 @@ int arch_print_fpregisters(int rad, const char *mask)
 	cons_printf(" cwd = 0x%04lx  ; control   ", regs.cwd);
 	cons_printf(" swd = 0x%04lx  ; status\n", regs.swd);
 	cons_printf(" twd = 0x%04lx  ; tag       ", regs.twd);
-	cons_printf(" fip = 0x%04lx  ; eip of fpu opcode\n", regs.fip);
-	cons_printf(" fcs = 0x%04lx              ", regs.fcs);
-	cons_printf(" foo = 0x%04lx  ; stack\n", regs.foo);
-	cons_printf(" fos = 0x%04lx\n", regs.fos);
+	cons_printf(" fip = 0x%04lx  ; code ptr\n", regs.fip);
+	cons_printf(" fcs = 0x%04lx  ; code seg  ", regs.fcs);
+	cons_printf(" foo = 0x%04lx  ; data ptr\n", regs.foo);
+	cons_printf(" fos = 0x%04lx  ; data seg\n", regs.fos);
 
 #if __NetBSD__
 #define FADDR ((double*)&regs.__data[i*4])
+#define MMXREG ((unsigned int*)&regs.__data[i*4])
 #else
 #define FADDR ((double*)&regs.st_space[i*4])
+#define MMXREG ((unsigned int*)&regs.st_space[i*4])
 #endif
 
+	for(t=0;t<2;t++)
 	for(i=0;i<8;i++) {
-		u16 *a = (u16*)&regs.xmm_space;
+		ut64 mmx;
+		ut16 *mmxword = (ut16*) &mmx;
+		ut16 *a = (ut16*)&regs.xmm_space;
 		a = a + (i * 4);
+		mmx = *MMXREG;
 
-		cons_printf(" mm%d = %04x %04x %04x %04x    ", i, (int)a[0], (int)a[1], (int)a[2], (int)a[3]);
-		cons_printf(" st%d = %lg (0x%08llx)\n", i, *FADDR, *FADDR);
+		switch(t) {
+		case 0:
+			cons_printf(" mm%d  %04x %04x %04x %04x  ", i, mmxword[0], mmxword[1], mmxword[2], mmxword[3]);
+			cons_printf(" st%d  %lg 0x%08llx\n", i, *FADDR, *FADDR);
+			break;
+		case 1:
+			cons_printf(" xmm%d %04x %04x %04x %04x %04x %04x %04x %04x\n", i,
+				a[0], a[1], a[2], a[3], a[4], a[5], a[6], a[7]);
+			break;
+		}
 	}
 #if 0
   /* from ida sdk */
@@ -773,18 +795,18 @@ int arch_print_registers(int rad, const char *mask)
 	
 	if (rad == 1) {
 		//cons_printf("\n"); // stupid trick
-		cons_printf("f oeax @ 0x%llx\n", (u64)(R_OEAX(regs)&0xffffffff));
-		cons_printf("f eax @ 0x%llx\n", (u64)(R_EAX(regs)&0xffffffff));
-		cons_printf("f ebx @ 0x%llx\n", (u64)(R_EBX(regs)&0xffffffff));
-		cons_printf("f ecx @ 0x%llx\n", (u64)(R_ECX(regs)&0xffffffff));
-		cons_printf("f edx @ 0x%llx\n", (u64)(R_EDX(regs)&0xffffffff));
-		cons_printf("f ebp @ 0x%llx\n", (u64)(R_EBP(regs)&0xffffffff));
-		cons_printf("f esi @ 0x%llx\n", (u64)(R_ESI(regs)&0xffffffff));
-		cons_printf("f edi @ 0x%llx\n", (u64)(R_EDI(regs)&0xffffffff));
-		cons_printf("f oeip @ 0x%llx\n", (u64)(R_EIP(oregs)&0xffffffff));
-		cons_printf("f eip @ 0x%llx\n", (u64)(R_EIP(regs)&0xffffffff));
-		cons_printf("f oesp @ 0x%llx\n", (u64)(R_ESP(oregs)&0xffffffff));
-		cons_printf("f esp @ 0x%llx\n", (u64)(R_ESP(regs)&0xffffffff));
+		cons_printf("f oeax @ 0x%llx\n", (ut64)(R_OEAX(regs)&0xffffffff));
+		cons_printf("f eax @ 0x%llx\n", (ut64)(R_EAX(regs)&0xffffffff));
+		cons_printf("f ebx @ 0x%llx\n", (ut64)(R_EBX(regs)&0xffffffff));
+		cons_printf("f ecx @ 0x%llx\n", (ut64)(R_ECX(regs)&0xffffffff));
+		cons_printf("f edx @ 0x%llx\n", (ut64)(R_EDX(regs)&0xffffffff));
+		cons_printf("f ebp @ 0x%llx\n", (ut64)(R_EBP(regs)&0xffffffff));
+		cons_printf("f esi @ 0x%llx\n", (ut64)(R_ESI(regs)&0xffffffff));
+		cons_printf("f edi @ 0x%llx\n", (ut64)(R_EDI(regs)&0xffffffff));
+		cons_printf("f oeip @ 0x%llx\n", (ut64)(R_EIP(oregs)&0xffffffff));
+		cons_printf("f eip @ 0x%llx\n", (ut64)(R_EIP(regs)&0xffffffff));
+		cons_printf("f oesp @ 0x%llx\n", (ut64)(R_ESP(oregs)&0xffffffff));
+		cons_printf("f esp @ 0x%llx\n", (ut64)(R_ESP(regs)&0xffffffff));
 	} else
 	if (rad == 2) {
 			if (R_EAX(regs)!=R_EAX(oregs)) cons_printf("eax = 0x%08x (0x%08x) ", R_EAX(regs), R_EAX(oregs));
@@ -1062,13 +1084,13 @@ XXX: we must use mmap2 here
 	return ret;
 }
 
-u64 arch_alloc_page(unsigned long size, unsigned long *rsize)
+ut64 arch_alloc_page(unsigned long size, unsigned long *rsize)
 {
 #ifdef __linux__
-        regs_t   reg, reg_saved;
-        int     status;
-	char	bak[4];
-        addr_t  ret = (addr_t)-1;
+        regs_t reg, reg_saved;
+        int status;
+	char bak[4];
+        ut64 ret = U64_MAX;
 
 	/* save old registers */
         debug_getregs(ps.tid, &reg_saved);
@@ -1120,20 +1142,17 @@ u64 arch_alloc_page(unsigned long size, unsigned long *rsize)
 
 	return ret;
 #elif __WINDOWS__
-	addr_t ret;
+	ut64 ret;
 
 	*rsize = size;
 
-	ret = (addr_t)(LONG)VirtualAllocEx(WIN32_PI(hProcess), (LPVOID)NULL,
-		       		size, MEM_COMMIT,
-				PAGE_NOCACHE | PAGE_EXECUTE_READWRITE);
-
-	return (ret != (addr_t)NULL)? ret : -1; 
-
+	ret = (ut64)(size_t)VirtualAllocEx(WIN32_PI(hProcess), (LPVOID)NULL,
+		size, MEM_COMMIT, PAGE_NOCACHE | PAGE_EXECUTE_READWRITE);
 
+	return (ret != 0LL)? ret : U64_MAX;
 #else
 	eprintf("Not supported on this OS\n");
-	return 0;
+	return 0LL;
 #endif
 } 
 
@@ -1280,7 +1299,7 @@ void signal_set(int signum, addr_t address)
 }
 #endif
 
-int arch_mprotect(u64 addr, unsigned int size, int perms)
+int arch_mprotect(ut64 addr, unsigned int size, int perms)
 {
 #if __APPLE__
 	/* OSX: Apple Darwin */
@@ -1346,7 +1365,7 @@ int arch_mprotect(u64 addr, unsigned int size, int perms)
 #endif
 }
 
-u64 arch_set_sighandler(int signum, addr_t handler)
+ut64 arch_set_sighandler(int signum, addr_t handler)
 {
 #ifdef __linux__
         regs_t   reg, reg_saved;
@@ -1603,7 +1622,7 @@ void arch_view_bt(struct list_head *sf)
 	list_for_each(pos, sf) {
                 sf_e = list_entry(pos, struct sf_t, next);
 		label[0] = '\0';
-		string_flag_offset((char *)&label, sf_e->ret_addr, 0);
+		string_flag_offset(NULL, (char *)&label, sf_e->ret_addr, 0);
 		cons_printf("%02d 0x%08x (framesz=%03d varsz=%d) %s\n",
 			i++, (uint)sf_e->ret_addr,
 			(uint)sf_e->sz, (uint)sf_e->vars_sz, label);
