@@ -22,6 +22,8 @@
 #define USE_PTRACE 0
 #define EXCEPTION_PORT 0
 
+int debug_procpidmem = 0;
+
 #define __addr_t_defined
 
 #include "../mem.h"
@@ -171,6 +173,7 @@ int debug_attach(int pid)
 
 	task = inferior_task; // ugly global asignation
 	fprintf(stderr, "; pid = %d\ntask= %d\n", pid, inferior_task);
+ps.pid = pid;
 	//ps.pid = ps.tid = task; // XXX HACK
 
 	if (task_threads(task, &inferior_threads, &inferior_thread_count) != KERN_SUCCESS) {
@@ -221,12 +224,14 @@ int debug_contp(int tid)
 	return 0;
 }
 
-#if __arm || __arm__
+#if __arm || __arm__ || __POWERPC__
 void debug_arch_x86_trap_set(int pid, int foo)
 {
 	/* do nothing here */
 }
+
 #else
+
 // XXX intel specific
 #define EFLAGS_TRAP_FLAG 0x100
 void debug_arch_x86_trap_set(int pid, int foo)
@@ -250,7 +255,7 @@ int debug_os_steps()
 
 #define OLD_PANCAKE_CODE 1
 #if OLD_PANCAKE_CODE
-	printf("stepping from pc = %08x\n", (u32)get_offset("eip"));
+	printf("stepping from pc = %08x\n", (ut32)get_offset("eip"));
 	//ret = ptrace(PT_STEP, ps.tid, (caddr_t)get_offset("eip"), SIGSTOP);
 	ret = ptrace(PT_STEP, ps.tid, (caddr_t)1, SIGTRAP); //SIGINT);
 	if (ret != 0) {
@@ -378,13 +383,16 @@ int debug_fork_and_attach()
 }
 
 // XXX This must be implemented on all the os/debug.c instead of arch_mprotect
-int debug_os_mprotect(u64 addr, int len, int perms)
+int debug_os_mprotect(ut64 addr, int len, int perms)
 {
-	return vm_protect(pid_to_task(ps.tid),
+	int ret = vm_protect(pid_to_task(ps.tid),
 		(vm_address_t)addr,
 		(vm_size_t)len,
 		(boolean_t)0, /* maximum protection */
 		unix_prot_to_darwin(perms));
+	if (ret != KERN_SUCCESS)
+		printf("vm_protect failed\n");
+	return ret;
 }
 
 #if 0
@@ -423,7 +431,7 @@ http://web.mit.edu/darwin/src/modules/xnu/osfmk/man/vm_map.html
 
 #endif
 
-int debug_os_read_at(pid_t tid, void *buff, int len, u64 addr)
+int debug_os_read_at(pid_t tid, void *buff, int len, ut64 addr)
 {
 	unsigned int size= 0;
 	int err = vm_read_overwrite(pid_to_task(tid), (unsigned int)addr, len, (pointer_t)buff, &size);
@@ -434,16 +442,26 @@ int debug_os_read_at(pid_t tid, void *buff, int len, u64 addr)
 	return size;
 }
 
-int debug_os_write_at(pid_t tid, void *buff, int len, u64 addr)
+int debug_os_write_at(pid_t tid, void *buff, int len, ut64 addr)
 {
-	// XXX SHOULD RESTORE PERMS LATER!!!
-	vm_protect(pid_to_task(tid), addr, len, 0, VM_PROT_READ | VM_PROT_WRITE | VM_PROT_EXECUTE);
+	kern_return_t err;
 
-	kern_return_t err = vm_write(
+	// XXX SHOULD RESTORE PERMS LATER!!!
+	err = vm_protect(pid_to_task(tid), addr+(addr%4096), 4096, 0, VM_PROT_READ | VM_PROT_WRITE);
+	if (err != KERN_SUCCESS)
+		printf("cant change page perms to rw\n");
+
+	err = vm_write(
 		pid_to_task(tid),
 		(vm_address_t)(unsigned int)addr, // XXX not for 64 bits
 		(pointer_t)buff,
 		(mach_msg_type_number_t)len);
+	if (err != KERN_SUCCESS)
+		printf("cant write on memory\n");
+
+	vm_protect(pid_to_task(tid), addr+(addr%4096), 4096, 0, VM_PROT_READ | VM_PROT_EXECUTE);
+	if (err != KERN_SUCCESS)
+		printf("cant change page perms to rx\n");
 
 	if (err == KERN_SUCCESS)
 		return len;
@@ -621,6 +639,7 @@ void macosx_debug_regions (task_t task, mach_vm_address_t address, int max, int
 	mach_vm_size_t size, prev_size;
 	mach_port_t object_name;
 	mach_msg_type_number_t count;
+	ut64 addr;
 	int nsubregions = 0;
 	int num_printed = 0;
 	MAP_REG *mr;
@@ -671,9 +690,9 @@ void macosx_debug_regions (task_t task, mach_vm_address_t address, int max, int
 			print = 1;
 
 		mr = malloc(sizeof(MAP_REG));
-		mr->ini = prev_address;
-		mr->end = prev_address + prev_size;
-		mr->size = prev_size;
+		mr->ini = (ut32) prev_address;
+		mr->end = (ut32) (prev_address+ prev_size);
+		mr->size = (ut32) prev_size;
 		
 		sprintf(buf, "unk%d-%s-%s-%s", i,
 			   unparse_inheritance (prev_info.inheritance),
@@ -686,12 +705,15 @@ void macosx_debug_regions (task_t task, mach_vm_address_t address, int max, int
 
 		add_regmap(mr);
 
+#if 0
 		if (1==0 && rest) { /* XXX never pritn this info here */
+			addr = 0LL;
+			addr = (ut64) (ut32) prev_address;
 			if (num_printed == 0)
 				fprintf(stderr, "Region ");
 			else	fprintf(stderr, "   ... ");
 			fprintf(stderr, " 0x%08llx - 0x%08llx %s (%s) %s, %s, %s",
-					(u64)prev_address, (u64)(prev_address + prev_size),
+					addr, addr + prev_size,
 					unparse_protection (prev_info.protection),
 					unparse_protection (prev_info.max_protection),
 					unparse_inheritance (prev_info.inheritance),
@@ -710,6 +732,7 @@ void macosx_debug_regions (task_t task, mach_vm_address_t address, int max, int
 
 			num_printed++;
 		} else {
+#endif
 #if 0
 			prev_size += size;
 			nsubregions++;
@@ -721,7 +744,7 @@ void macosx_debug_regions (task_t task, mach_vm_address_t address, int max, int
 
 			num_printed++;
 #endif
-		}
+//		}
 
 		if ((max > 0) && (num_printed >= max))
 			done = 1;
