@@ -81,7 +81,7 @@ int debug_syms()
 // TODO : stupid helper
 int getv()
 {
-	return (int)config_get("cfg.verbose");
+	return PTRCAST (config_get("cfg.verbose"));
 #if 0
 	char *ptr = config_get("cfg.verbose");
 	if (ptr&&ptr[0]=='1')
@@ -119,12 +119,11 @@ int debug_tt_range(const char *arg)
 {
 	struct aop_t aop;
 	int pid, bpsz;
-	u64 pc, rpc; // program counter
+	ut64 pc, rpc; // program counter
 	u8 *sa; // swap area
 	u8 *cc; // breakpoint area
-	u64 sz;
-	char *cmd = NULL;
-	u64 ba = config.seek; // base address
+	const char *cmd = NULL;
+	ut64 sz, ba = config.seek; // base address
 	struct list_head *rgs, *pos;
 	struct range_t *r;
 	int status;
@@ -212,12 +211,12 @@ int debug_tt(const char *arg)
 {
 	struct aop_t aop;
 	int pid, bpsz;
-	u64 pc; // program counter
+	ut64 pc; // program counter
 	u8 *sa; // swap area
 	u8 *cc; // breakpoint area
 	const char *cmd = NULL;
-	u64 ba = config.seek; // base address
-	u64 sz = get_math(arg+1); // size
+	ut64 ba = config.seek; // base address
+	ut64 sz = get_math(arg+1); // size
 	int status;
 
 	// XXX
@@ -393,10 +392,9 @@ int debug_init()
 int debug_print_maps(char *arg)
 {
 	if(debug_init_maps(1) == 0) {
-		print_maps_regions((int)strchr(arg, '*'), arg[0]=='?');
+		print_maps_regions(PTRCAST (strchr(arg, '*')), arg[0]=='?');
 		return 0;
 	}
-
 	return -1;
 }
 
@@ -411,14 +409,65 @@ void debug_msg_set(const char *format, ...)
 	va_end(ap);
 }
 
+static void debug_environment_()
+{
+#if __UNIX__
+	char str[1024];
+	int length;
+	// hack '_' to emulate the real one of the process
+	if (ps.filename[0]!='/') {
+		getcwd(str, 1023);
+		length=strlen(str);
+		snprintf(str+length, 1023-length, "/%s", ps.filename);
+		setenv("_", str, 1);
+	} else setenv("_", ps.filename, 1);
+#endif
+}
+
 void debug_environment()
 {
 	const char *ptr;
 
-	// TODO proper environment handle
-	if (getv()) {} //eprintf("TODO: load environment from environ.txt or so.\n");
+	unsetenv("COLUMNS");
+	unsetenv("LINES");
+	// Load environment from file
+	ptr = config_get("file.dbg_env");
+	if (ptr && *ptr) {
+		char buf[4096], *eq;
+		FILE *fd;
+		int quoted, len;
+		eprintf("Loading file.dbg_env from '%s'\n", ptr);
+		environ = NULL;
+		// hack '_' to emulate the real one of the process
+		debug_environment_();
+		fd = fopen(ptr, "r");
+		if (fd != NULL) {
+			/* expected format is: [#comment][export ][VAR]="[VALUE]" */
+			while(!feof(fd)) {
+				fgets(buf, 4095, fd);
+				if (feof(fd))
+					break;
+				if (!buf[0]) continue;
+				len = strlen(buf);
+				quoted = (buf[len-2]=='"')?2:1;
+				buf[len-quoted]=0; // chop '\n'
+				if (!memcmp(buf, "export ", 7))
+					memcpy(buf, buf+7, strlen(buf+7)+1);
+				eq = strchr(buf, '=');
+				if (eq) {
+					*eq = 0;
+					if (eq[1]=='@') {
+						char *contents = slurp(eq+quoted, NULL);
+						setenv(buf, contents, 1);
+						free(contents);
+					} else setenv(buf, eq+quoted, 1);
+				}
+			}
+		} else eprintf("Cannot open '%s'\n", ptr);
+	} else debug_environment_();
 
-	/* LD.SO */
+	if (config_get_i("dbg.env_ldso")) {
+		/* LD.SO */
 #if 0
 Valid options for the LD_DEBUG environment variable are:
 
@@ -436,17 +485,18 @@ Valid options for the LD_DEBUG environment variable are:
 To direct the debugging output into a file instead of standard output
 a filename can be specified using the LD_DEBUG_OUTPUT environment variable.
 #endif
-	// LD_PRELOAD
-	// LD_BIND_NOW : resolve all symbols at startup
-	setenv("LD_BIND_NOW", "true", 1);
-	//setenv("LD_TRACE_LOADED_OBJECTS", "true", 1);
+		// LD_PRELOAD
+		// LD_BIND_NOW : resolve all symbols at startup
+		setenv("LD_BIND_NOW", "true", 1);
+		//setenv("LD_TRACE_LOADED_OBJECTS", "true", 1);
 
-	setenv("LD_WARN", "true", 1);
-	setenv("LD_DEBUG", "", 1);
-	setenv("LD_VERBOSE", "", 1);
-	setenv("CK_FORK","no",1); // to debug gstreamer check test programs
-	//setenv("LD_SHOW_AUXV", "true", 1);
-	//setenv("LD_PROFILE", "/lib/libc.so.6", 1);
+		setenv("LD_WARN", "true", 1);
+		setenv("LD_DEBUG", "", 1);
+		setenv("LD_VERBOSE", "", 1);
+		setenv("CK_FORK","no",1); // to debug gstreamer check test programs
+		//setenv("LD_SHOW_AUXV", "true", 1);
+		//setenv("LD_PROFILE", "/lib/libc.so.6", 1);
+	}
 
 	// TODO: Set resources limits (setrlimit)
 #if __linux__
@@ -542,16 +592,14 @@ int is_usercode(addr_t pc)
 	return 0;
 }
 
-char *alloc_usercode(u64 *base, u64 *len)
+char *alloc_usercode(ut64 *base, ut64 *len)
 {
 	struct list_head *pos;
 	u8 *buf = NULL;
 
 	if (ps.map_regs_sz==0) {
 		eprintf("Cannot determine usercode.\n");
-		return 1;
-	}
-
+	} else
 	list_for_each_prev(pos, &ps.map_reg) {
 		MAP_REG	*mr = (MAP_REG *)((char *) pos + \
 			sizeof(struct list_head) -  sizeof(MAP_REG));
@@ -576,7 +624,7 @@ int debug_stepret()
 	radare_controlc();
 	while(!config.interrupted && ps.opened && debug_stepo()) {
 		debug_read_at(ps.tid, bytes, 4, arch_pc(ps.tid));
-		arch_aop((u64)arch_pc(ps.tid), (const u8 *)bytes, &aop);
+		arch_aop((ut64)arch_pc(ps.tid), (const u8 *)bytes, &aop);
 		if (aop.type == AOP_TYPE_RET)
 			break;
 	}
@@ -588,13 +636,13 @@ int debug_stepret()
 int debug_contum(const char *input)
 {
 	struct aop_t aop;
-	u64 pc;
+	ut64 pc;
 	char a[32];
-	int ilen, dotrace = config_get("trace.log");
-	u64 base = 0, len = 0;
-	u8 *buf = alloc_usercode(&base, &len);
+	int ilen, dotrace = PTRCAST (config_get ("trace.log"));
+	ut64 base = 0LL, len = 0LL;
+	u8 *buf = alloc_usercode (&base, &len);
 	if (buf == NULL) {
-		eprintf("No user code?\n");
+		eprintf ("No user code?\n");
 		return 1;
 	}
 
@@ -727,7 +775,7 @@ int debug_ie(char *input)
 
 int debug_until(const char *addr)
 {
-	u64 off = 0LL;
+	ut64 off = 0LL;
 	char buf[128];
 	u8 ptr[128];
 
@@ -743,8 +791,8 @@ int debug_until(const char *addr)
 	else	off = get_math(addr);
 
 	if (off != 0) {
-		eprintf("entry at: 0x%llx\n", (u64) ps.entrypoint);
-		sprintf(buf, "0x%llx\n", (u64) off);
+		eprintf("entry at: 0x%llx\n", (ut64) ps.entrypoint);
+		sprintf(buf, "0x%llx\n", (ut64) off);
 		debug_cont_until(buf);
 	}
 
@@ -760,7 +808,7 @@ int debug_until(const char *addr)
 		debug_read_at(ps.tid, buf, 12, arch_pc(ps.tid));
 		if (!memcmp(buf, "\x31\xed\x5e\x89\xe1\x83\xe4\xf0\x50\x54\x52\x68", 12)) {
 			debug_read_at(ps.tid, &ptr, 4, arch_pc(ps.tid)+24);
-			off = (u64)(unsigned int)(ptr[0]) | (ptr[1]<<8) | (ptr[2] <<16) | (ptr[3]<<24);
+			off = (ut64)(unsigned int)(ptr[0]) | (ptr[1]<<8) | (ptr[2] <<16) | (ptr[3]<<24);
 			sprintf(buf, "0x%x", (unsigned int)off);
 			printf("==> 1 main at : %s\n", buf);
 			if (off !=0&&ptr[0]!=0xff)
@@ -769,7 +817,7 @@ int debug_until(const char *addr)
 		if (!memcmp(buf, "^\x89\xe1\x83\xe4\xf0PTRh", 10)) {
 			unsigned int addr;
 			debug_read_at(ps.tid, &addr, 4, arch_pc(ps.tid)+0x16);
-			off = (u64)addr;
+			off = (ut64)addr;
 			sprintf(buf, "0x%x", addr);
 			printf("==> 2 main at : %s\n", buf);
 			if (off !=0&&off!=0xffffffffLL)
@@ -812,10 +860,13 @@ int debug_info(char *arg)
 	int fd;
 	char buf[4096];
 
+#if 0
 	if (strchr(arg,'*')) {
 		cons_printf("f entry @ 0x%08x\n", ps.entrypoint);
 		//cons_printf("f bep @ %s\n", config_get("dbg.bep"));
 	} else {
+#endif 
+	if (!strchr(arg, '*')) {
 		cons_printf(" filename    %s\n", strget(ps.filename));
 
 		cons_printf(" pid         %d\n", ps.pid);
@@ -883,7 +934,7 @@ int debug_skip(int times)
 	unsigned char buf[16];
 	struct aop_t aop;
 	int len;
-	u64 pc = arch_pc(ps.tid);
+	ut64 pc = arch_pc(ps.tid);
 
 	if (ps.opened) {
 		debug_read_at(ps.tid, buf, 16, arch_pc(ps.tid));
@@ -906,7 +957,7 @@ int debug_skip(int times)
 
 int debug_stepu(const char *arg)
 {
-	u64 until = get_math(arg);
+	ut64 until = get_math(arg);
 	unsigned long pc = arch_pc(ps.tid); //WS_PC();
 
 	/* step until address */
@@ -1008,7 +1059,7 @@ int debug_stepbp(int times)
 			debug_bp_rm_addr(aop.fail);
 		debug_bp_rm_addr(pc+len);
 	} else {
-		u64 ptr = pc;
+		ut64 ptr = pc;
 		for(i=0;i<times;i++) {
 			debug_read_at(ps.tid, bytes, 32, ptr);
 			len = arch_aop(pc, bytes, &aop);
@@ -1027,8 +1078,8 @@ int debug_stepbp(int times)
 int debug_step(int times)
 {
 	unsigned char opcode[32];
-	u64 pc, off;
-	u64 old_pc = 0;
+	ut64 pc, off;
+	ut64 old_pc = 0;
 	const char *tracefile;
 	const char *flagregs;
 	int ret;
@@ -1059,9 +1110,8 @@ int debug_step(int times)
 				return 0;
 			}
 			ret = debug_dispatch_wait();
-			printf("DISPATH WAIT: %d\n", ret);
-	trace_add((addr_t)pc);
-	ps.steps++;
+			trace_add((addr_t)pc);
+			ps.steps++;
 		}
 		debug_print_wait("step");
 	} else {
@@ -1107,7 +1157,7 @@ int debug_step(int times)
 			//	radare_cmd("!dregs", 0);
 			//	config_set("scr.buf", "true");
 				arch_print_registers(0, "line");
-				ptr = cons_get_buffer();
+				ptr = (char *)cons_get_buffer();
 				if(ptr[0])ptr[strlen(ptr)-1]='\0';
 				sprintf(buf, "CC %d %s @ 0x%08llx", ps.steps, strget(ptr), pc);
 			//	config_set("scr.buf", "false"); // XXX
@@ -1144,13 +1194,13 @@ int debug_trace(char *input)
 	// TODO: file.trace ???
 	// TODO: show stack and backtrace only when is different
 	int counter = 0;
-	int tbt = (int)config_get("trace.bt");
-	long slip = (int)config_get_i("trace.sleep");
-	int smart = (int)config_get("trace.smart");
-	int tracebps = (int)config_get("trace.bps");
-	int tracelibs = (int)config_get("trace.libs");
-	int tracecalls = (int)config_get("trace.calls");
-	char *cmdtrace = config_get("cmd.trace");
+	int tbt = PTRCAST (config_get("trace.bt"));
+	long slip = PTRCAST (config_get_i("trace.sleep"));
+	int smart = PTRCAST (config_get("trace.smart"));
+	int tracebps = PTRCAST (config_get("trace.bps"));
+	int tracelibs = PTRCAST (config_get("trace.libs"));
+	int tracecalls = PTRCAST (config_get("trace.calls"));
+	const char *cmdtrace = config_get("cmd.trace");
 	int level = atoi(input+1);
 	unsigned long pc;
 
@@ -1186,7 +1236,7 @@ int debug_trace(char *input)
 		/* XXX : if bp in addr stop! */
 		counter++;
 		radare_controlc();
-		u64 pc = arch_pc(ps.tid);
+		ut64 pc = arch_pc(ps.tid);
 		radare_cmd(".!regs*", 0);
 		if (smart) {
 			cons_printf("[-] 0x%08llx\n", arch_pc(ps.tid));
@@ -1199,14 +1249,14 @@ int debug_trace(char *input)
 				char label[128];
 				struct aop_t aop;
 				char buf[32];
-				u64 addr = arch_pc(ps.tid);
+				ut64 addr = arch_pc(ps.tid);
 				radare_read_at(addr, buf, 32);
 				arch_aop(addr, buf, &aop);
 				switch(aop.type) {
 				case AOP_TYPE_CALL:
 				case AOP_TYPE_RCALL:
 					label[0] = '\0';
-					string_flag_offset(label, aop.jump, 0);
+					string_flag_offset(NULL, label, aop.jump, 0);
 					cons_printf("[>] call 0x%08llx ; %s\n", aop.jump, label);
 					show = 1;
 					break;
@@ -1452,7 +1502,7 @@ int debug_inject_buffer(unsigned char *fil, int sz)
 
 int debug_inject(char *file)
 {
-	u64 sz = 0;
+	ut64 sz = 0;
 	int fd = open(file, O_RDONLY);
 	unsigned char *fil = NULL;
 
@@ -1484,12 +1534,12 @@ int debug_inject(char *file)
 /* XXX : Stops at the following opcode at this address...debug_bp_restore() doesnt works? */
 int debug_cont_until(const char *input)
 {
-	u64 addr = input?get_math(input):0;
+	ut64 addr = input?get_math(input):0;
 
 	/* continue until address */
 	if (addr != 0) {
 		int bp;
-		eprintf("Continue until (%s) = 0x%08llx\n", input, addr);
+		eprintf("=== Continue until '%s' 0x%08llx\n", input+1, addr);
 		bp = debug_bp_set(NULL, addr,config_get_i("dbg.hwbp"));
 		debug_cont(NULL);
 		debug_bp_restore(bp);
@@ -1535,7 +1585,7 @@ int debug_cont(const char *input)
 		ret = debug_dispatch_wait();
 		//debug_msg_set("debug_dispatch_wait: RET = %d WS(event)=%d INT3_EVENT=%d INT_EVENT=%d CLONE_EVENT=%d\n", 
 	//		ret, WS(event), INT3_EVENT, INT_EVENT, CLONE_EVENT);
-		printf("debug_dispatch_wait: RET = %d WS(event)=%d INT3_EVENT=%d INT_EVENT=%d CLONE_EVENT=%d\n", 
+		printf("=== wait: RET = %d WS(event)=%d INT3_EVENT=%d INT_EVENT=%d CLONE_EVENT=%d\n", 
 			ret, WS(event), INT3_EVENT, INT_EVENT, CLONE_EVENT);
 	#warning TODO: add in debug_print_foo() disassembly of eip here and change process id or so
 		if (SKIP_USER_CONTROLC()) {
@@ -1643,7 +1693,7 @@ int debug_loop(char *addr_str)
 
 int debug_inject2(char *input)
 {
-	u64 pc, size;
+	ut64 pc, size;
 	int status;
 	char buf[1024];
 	char *bak;
