@@ -27,16 +27,16 @@ int debug_single_setregs(pid_t tid, regs_t *regs);
 int debug_print_wait(char *act);
 int debug_dispatch_wait();
 
-void debug_print_sigh(char *signame, unsigned long handler);
+void debug_print_sigh(const char *signame, unsigned long handler);
 
 /* basic io */
-int debug_read_at(pid_t tid, void *buff, int len, u64 addr);
-int debug_write_at(pid_t tid, void *buff, int len, u64 addr);
+int debug_read_at(pid_t tid, void *buff, int len, ut64 addr);
+int debug_write_at(pid_t tid, void *buff, int len, ut64 addr);
 
 /* file descriptors */
-u64 debug_fd_seek(int pid, int fd, u64 addr, int whence);
-u64 debug_fd_read(int pid, int fd, u64 addr, int len);
-u64 debug_fd_write(int pid, int fd, u64 addr, int len);
+ut64 debug_fd_seek(int pid, int fd, ut64 addr, int whence);
+ut64 debug_fd_read(int pid, int fd, ut64 addr, int len);
+ut64 debug_fd_write(int pid, int fd, ut64 addr, int len);
 int debug_fd_list(int pid, int rad);
 int debug_fd_dup2(int pid, int oldfd, int newfd);
 int debug_fd_open(int pid, char *file, int mode);
