@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2007, 2008
+ * Copyright (C) 2007, 2008, 2009
  *       pancake <youterm.com>
  *       th0rpe <nopcode.org>
  *
@@ -52,7 +52,8 @@
 #include <sys/stat.h>
 #include "../events.h"
 #include "../debug.h"
-#include "procs.h"
+
+#define debug_read_raw(x,y) ptrace(PTRACE_PEEKTEXT, x, y, 0)
 
 #if __FreeBSD__
 int th_init_freebsd(int pid)
@@ -97,9 +98,7 @@ int th_info_bsd(int pid)
 }
 #endif
 
-int debug_os_kill(int pid, int sig)
-{
-	//return 0;
+int debug_os_kill(int pid, int sig) {
 	/* prevent killall selfdestruction */
 	if (pid < 1)
 		return -1;
@@ -114,8 +113,7 @@ int debug_os_kill(int pid, int sig)
 #include <sys/uio.h>
 #include <sys/ktrace.h>
 
-int debug_ktrace()
-{
+int debug_ktrace() {
 #if __OpenBSD__ || __FreeBSD__
 	eprintf("No fktrace for OpenBSD. Needs to be implemented with ktrace(2)\n");
 #else
@@ -139,16 +137,16 @@ int debug_ktrace()
 	}
 
 	ret = arch_continue();
-	ret = read(pd[0],buf,1);
+	ret = read (pd[0], buf, 1);
 	if (ret>0) {
-		kill(ps.tid, SIGSTOP);
+		debug_os_kill(ps.tid, SIGSTOP);
 		printf("Have sweet dreamz my lil pon1e!!1!\n");
 
 		ret = fktrace(pd[1], KTROP_CLEAR, trp, ps.tid);
 		ret |= fktrace(pd[1], KTROP_CLEARFILE, trp, ps.tid);
 		printf("<fktrace> %d\n", ret);
 
-		ret = kill(ps.tid, 0);
+		ret = debug_os_kill(ps.tid, 0);
 		printf("<kill0> %d (alive)\n", ret);
 
 		ret = wait(&sta);
@@ -174,6 +172,7 @@ int debug_print_wait(char *act)
 		debug_bp_restore(-1);
 		break;
 	default:
+		//eprintf("=== exit value: %d\n", WEXITSTATUS(ps.ws));
 		if(WS(event) != EXIT_EVENT ) {
 			/* XXX: update thread list information here !!! */
 			eprintf("=== %s: tid: %d event: %d, signal: %d (%s). stop at 0x%08llx\n", 
@@ -199,6 +198,10 @@ int debug_init_maps(int rest)
 	char  line[1024];
 	char  *pos_c;
 	int   i, unk = 0;
+#if __FreeBSD__
+	int ign;
+	char unkstr[128];
+#endif
 
 #if 0
  /* FOR SUN */
@@ -392,7 +395,9 @@ int debug_status()
 
 static int is_alive(int pid)
 {
-	return kill(pid, 0)==0;
+	if (pid<1)
+		return 0;
+	return kill (pid, 0)==0;
 }
 
 #define MAGIC_EXIT 0x33
@@ -406,7 +411,7 @@ static int is_alive(int pid)
 int debug_fork_and_attach()
 {
 	int wait_val;
-	int i,pid;
+	int i, pid;
 
 	pid = vfork();
 	switch(pid) {
@@ -426,7 +431,6 @@ int debug_fork_and_attach()
 			eprintf("]\n");
 		}
 
-		// TODO: USE TM IF POSSIBLE TO ATTACH IT FROM ANOTHER CONSOLE!!!
 		debug_environment();
 		execv(ps.argv[0], ps.argv);
 		perror("fork_and_attach: execv");
@@ -455,9 +459,8 @@ int debug_fork_and_attach()
 		/* restore breakpoints */
 		debug_bp_reload_all();
 
-		if (config_get("dbg.stop")) 
-			kill(ps.pid, SIGSTOP);
-
+		if (config_get("dbg.stop"))
+			debug_os_kill (ps.pid, SIGSTOP);
 		ps.steps  = 1;
 	}
 
@@ -516,24 +519,44 @@ int debug_detach()
 
 	return 0;
 }
-#define debug_read_raw(x,y) ptrace(PTRACE_PEEKTEXT, x, y, 0)
 
-/* copied from patan */
+/* copied from patan .. slow and buggy on solaris */
 extern int errno;
+int debug_procpidmem;
+static int ppm_fd = -1;
 
-int debug_os_read_at(int pid, void *buff, int sz, u64 addr)
+int debug_os_read_at(int pid, void *buff, int sz, ut64 addr)
 {
-	unsigned long words = sz / sizeof(long) ;
-	unsigned long last = sz % sizeof(long) ;
-	long x, lr ;
-	int ret ;
+	#define RDX sizeof(long);
+	unsigned long words = sz / RDX;
+	unsigned long last = sz % RDX;
+	long x, lr;
+	int ret = 1;
+#if __linux__
+	// TODO: port to other architectures?
+	if (debug_procpidmem) {
+		if (ppm_fd == -1) {
+			char file[128];
+			snprintf(file, sizeof (file), "/proc/%d/mem", pid);
+			ppm_fd = open (file, O_RDONLY|O_LARGEFILE);
+			if (ppm_fd == -1) {
+				printf("Cannot open proc/pid/mem\n");
+				return -1;
+			}
+		}
+		lseek (ppm_fd, (off_t)addr, SEEK_SET);
+		ret = read (ppm_fd, buff, sz);
+		//close (fd);
+		return ret;
+	}
+#endif
 
 	if (sz<0)
 		return -1;
 
 	if (addr==-1)
 		return 0;
-
+retry:
 	for(x=0;x<words;x++) {
 		((long *)buff)[x] = debug_read_raw(pid, (void *)(&((long*)(long )addr)[x]));
 
@@ -548,18 +571,25 @@ int debug_os_read_at(int pid, void *buff, int sz, u64 addr)
 		if (lr == -1 && errno)
 			goto err;
 
-		memcpy(&((long *)buff)[x],&lr,last) ;
+		memcpy(&((long *)buff)[x], &lr, last);
 	}
 
+//long *buf =buff;
+//fprintf(stderr, "ROK 0x%08llx %08x\n", addr, *buf);
 	return sz;
 err:
+#if __sun
+	ret = 0;
+#else
 	ret = --x * sizeof(long);
+#endif
 
+//fprintf(stderr, "ERR 0x%08llx\n", addr);
 	return ret ;
 }
 
 
-int debug_os_write_at(int pid, u8 *buff, int sz, u64 addr)
+int debug_os_write_at(int pid, u8 *buff, int sz, ut64 addr)
 {
         long words = sz / sizeof(long) ;
         long last = (sz % sizeof(long))*8;
@@ -582,15 +612,12 @@ int debug_os_write_at(int pid, u8 *buff, int sz, u64 addr)
 	return sz;
 #endif
 //eprintf("%d ->%d (0x%x)\n",pid, (int)sz, (long)addr);
-
-
 	for(x=0;x<words;x++)
 		if (ptrace(PTRACE_POKEDATA,pid,&((long *)(long)addr)[x],((long *)buff)[x]))
 			goto err ;
 
 	if (last) {
-		lr = ptrace(PTRACE_PEEKTEXT,pid,&((long *)(long)addr)[x], 0) ;
-
+		lr = ptrace(PTRACE_PEEKTEXT,pid, &((long *)(long)addr)[x], 0) ;
 		/* Y despues me quejo que lisp tiene muchos parentesis... */
 		if ((lr == -1 && errno) ||
 		    (
@@ -600,13 +627,9 @@ int debug_os_write_at(int pid, u8 *buff, int sz, u64 addr)
 		   )
                 goto err;
 	}
-
 	return sz ;
-
         err:
-                return --x * sizeof(long) ;
-
-        //return ret ;
+	return --x * sizeof(long) ;
 }
 
 
@@ -670,7 +693,7 @@ int debug_getregs(pid_t pid, regs_t *reg)
 	return ptrace(PTRACE_GETREGS, pid, NULL, reg);
     #endif
 #else
-	memset(reg,  0 ,sizeof(regs_t));
+	memset(reg, 0 ,sizeof(regs_t));
 	return ptrace(PTRACE_GETREGS, pid, reg, sizeof(regs_t));
 #endif
 }
@@ -741,27 +764,27 @@ int debug_contp(int pid)
 
 		list_for_each_prev(pos, &ps.th_list) {
 			TH_INFO		*th = list_entry(pos, TH_INFO, list);
-			ptrace(PTRACE_CONT, th->tid, (u32)arch_pc(th->tid), 0);
+			ptrace(PTRACE_CONT, th->tid, (ut32)arch_pc(th->tid), 0);
 //eprintf("..thread continue: %d\n", th->tid);
 //			eprintf("cont-> %c %d: 0x%08llx state: 0x%x\n", (ps.th_active == th)?'*':' ', th->tid, th->addr, th->status);
 			n++;
 		}
-		ptrace(PTRACE_CONT, pid, (u32)arch_pc(pid), 0);
+		ptrace(PTRACE_CONT, pid, (ut32)arch_pc(pid), 0);
 //eprintf("..continue: %d\n", pid);
-		ptrace(PTRACE_CONT, ps.pid, (u32)arch_pc(ps.pid), 0);
+		ptrace(PTRACE_CONT, ps.pid, (ut32)arch_pc(ps.pid), 0);
 //eprintf("..continue: %d\n", ps.pid);
-		ptrace(PTRACE_CONT, ps.tid, (u32)arch_pc(ps.tid), 0);
+		ptrace(PTRACE_CONT, ps.tid, (ut32)arch_pc(ps.tid), 0);
 //eprintf("..continue: %d\n", ps.tid);
 		
 		return 0;
 	}
-	return ptrace(PTRACE_CONT, pid, (u32)arch_pc(pid), 0);
+	return ptrace(PTRACE_CONT, pid, (ut32)arch_pc(pid), 0);
 }
 
 int debug_contscp()
 {
 #if __linux__ || ( __BSD__ && defined(PT_SYSCALL))
-	return ptrace(PTRACE_SYSCALL, ps.tid, (u32)arch_pc(ps.tid), 0);
+	return ptrace(PTRACE_SYSCALL, ps.tid, (ut32)arch_pc(ps.tid), 0);
 #else
 	eprintf("contscp: not implemented for this platform\n");
 	return 0;
@@ -770,7 +793,7 @@ int debug_contscp()
 
 int debug_os_steps()
 {
-	return ptrace(PTRACE_SINGLESTEP, ps.tid, (u32)arch_pc(ps.tid), 0);
+	return ptrace(PTRACE_SINGLESTEP, ps.tid, (ut32)arch_pc(ps.tid), 0);
 }
 
 int debug_dispatch_wait()
@@ -817,17 +840,15 @@ int debug_dispatch_wait()
 			th->addr = arch_pc(ps.tid);
 			add_th(th);
 			ps.th_active = th;
-
 			ret = 1;
 		}
-	} else
-		ps.th_active = 0;
+	} else ps.th_active = 0;
 
 	if(WIFEXITED(status)) {
 		debug_msg_set("process %d finished\n", ps.tid);
 		eprintf("\n\n______________[ process finished ]_______________\n\n");
 		//ps.opened = 0;
-		kill(ps.tid, SIGKILL);
+		debug_os_kill (ps.tid, SIGKILL);
 		sleep(1);
 		eprintf("Use !load or ^C to reload\n");
 		config.interrupted=1;
@@ -879,17 +900,17 @@ int debug_dispatch_wait()
 		#endif
 			{
 			char buf[128];
-			u64 addr = debug_bp_restore_after();
+			ut64 addr = debug_bp_restore_after();
 			if (addr != 0) {
 				/* we should trace here */
 				debug_step(1);
-				sprintf(buf, "!bp -0x%08llx\n", (u64) addr); //-bpsize);
+				sprintf(buf, "!bp -0x%08llx\n", (ut64) addr); //-bpsize);
 				radare_cmd(buf,0);
 				eprintf("==> Trace address 0x%08llx\n", addr);
-				sprintf(buf,"!bpt 0x%08llx\n",(u64) addr); //-bpsize);
+				sprintf(buf,"!bpt 0x%08llx\n",(ut64) addr); //-bpsize);
 				radare_cmd(buf,0);
-return 1;
-				debug_cont(0);
+				return 1;
+				debug_cont(0); // ???
 			}
 			}
 #if 0
@@ -905,7 +926,6 @@ return 1;
 				/* search if changed permissions at region */
 				//printf("CHANGE REGIONS: 0x%x\n", WS_SI(si_addr));
 				/* TODO: manage access to protected pages */
-				;
 				debug_msg_set("pid %d Segmentation fault!\n", ps.tid);
 				eprintf("Segmentation fault!\n");
 			} else {
@@ -935,14 +955,17 @@ return 1;
 					debug_msg_set("pid %d clone()\n", ps.tid);
 					eprintf("CLONE HAS BEEN INVOKED\n");
 					break;
+				case 28:
+					//eprintf("Child has generated a Segmentation fault.\n");
+					eprintf("=== Last debug op not performed because of event (console resize?)\n");
+					break;
 				default:
 					debug_msg_set("pid %d unknown signal received (%d)\n", ps.tid, WS_SI(si_signo));
 					eprintf("Unknown signal %d received\n", WS_SI(si_signo));
 				}
 			}
 		}
-	} else
-		eprintf("What?\n");
+	} else eprintf("What?\n");
 
 	return 0; /* XXX: should stop */
 }
