@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2008, 2009
+ * Copyright (C) 2008-2010
  *       th0rpe <nopcode.org>
  *       pancake <youterm.com>
  *
@@ -29,6 +29,8 @@
 #include <winbase.h>
 #include <psapi.h>
 
+
+int debug_procpidmem = 0;
 #if __CYGWIN__
 
 /* CYGWIN declare only GetProcessID() 
@@ -351,7 +353,7 @@ err_fork:
 	return -1;
 }
 
-int debug_os_read_at(pid_t tid, void *buff, int len, u64 addr)
+int debug_os_read_at(pid_t tid, void *buff, int len, ut64 addr)
 {
 	int ret_len = 0;
 
@@ -361,7 +363,7 @@ int debug_os_read_at(pid_t tid, void *buff, int len, u64 addr)
 	return ret_len;
 }
 
-int debug_os_write_at(pid_t tid, void *buff, int len, u64 addr)
+int debug_os_write_at(pid_t tid, void *buff, int len, ut64 addr)
 {
 	int ret_len = -1;
 
