@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2006, 2007, 2008, 2009
+ * Copyright (C) 2006, 2007, 2008, 2009, 2010
  *       pancake <youterm.com>
  *
  * radare is free software; you can redistribute it and/or modify
@@ -28,25 +28,57 @@
 #include <string.h>
 #include <unistd.h>
 
+#if HAVE_LIB_READLINE
+#include <readline/readline.h>
+#endif
+
 struct list_head flags;
 static int flag_ptr = -1;
 
+#if USE_BTREE
+#include "btree.h"
+struct btree_node *flags_off = NULL;
+struct btree_node *flags_nam = NULL;
+/* compare names */
+static int ncmp(const void *a, const void *b)
+{
+	flag_t *fa = (flag_t *)a;
+	flag_t *fb = (flag_t *)b;
+	int ret = 0;
+	/* we cannot use a simple substraction coz ut64 > s32 :) */
+	if (fa->namehash > fb->namehash) ret = 1;
+	else if (fa->namehash < fb->namehash) ret = -1;
+	return ret;
+}
+
+/* compare offsets */
+static int cmp(const void *a, const void *b)
+{
+	flag_t *fa = (flag_t *)a;
+	flag_t *fb = (flag_t *)b;
+	int ret = 0;
+	/* we cannot use a simple substraction coz ut64 > s32 :) */
+	if (fa->offset > fb->offset) ret = 1;
+	else if (fa->offset < fb->offset) ret = -1;
+	return ret;
+}
+#endif
+
 #define FLAG_SPACES 32
 // XXX optimize... FLAG SPACES MUST BE A LINKED LIST TOO!
 static struct flags_spaces_t {
 	const char *name;
 } flag_spaces[FLAG_SPACES];
 
-static u64 flag_from_i = 0;
+static ut64 flag_from_i = 0;
 int flag_space_idx = -1;
 int flag_space_idx2 = -1;
 
 void flag_from(const char *str)
 {
-	if (str[0]) {
+	if (str[0])
 		flag_from_i = get_math(str);
-	} else
-		cons_printf("0x%08llx\n", flag_from_i);
+	else cons_printf("0x%08llx\n", flag_from_i);
 }
 
 void flag_init()
@@ -91,7 +123,7 @@ void flag_help()
 int flag_interpolation(const char *from, const char *to)
 {
 	int ret = 0;
-	u64 tmp = 0;
+	ut64 tmp = 0;
 	const char *str = NULL;
 	struct list_head *pos, *pos2;
 
@@ -108,29 +140,22 @@ int flag_interpolation(const char *from, const char *to)
 					}
 				}
 			}
-
-			if (tmp != 0) {
+			if (tmp != 0)
 				printf("%s (0x%08llx) -> %s (0x%08llx)   ; size = %lld\n",
 					flag->name, flag->offset, str, tmp, tmp-flag->offset);
-			}
 		}
 	}
 	return ret;
 }
 
-flag_t *flag_get_by_addr(u64 addr);
 void flag_rebase(const char *text)
 {
 	flag_t *f = flag_get_by_addr(config.seek);
 	if (f) {
-		u64 base;
 		if (text[0]=='-')
 			f->offset -= get_math(text+1);
 		else f->offset += get_math(text);
-//printf("REBASING to 0x%08llx\n", f->offset);
-	} //else {
-	//	eprintf("No flag found at 0x%08llx this address\n", config.seek);
-	//}
+	}
 }
 
 void flag_cmd(const char *text)
@@ -146,15 +171,24 @@ void flag_cmd(const char *text)
 	}
 }
 
-flag_t *flag_get_by_addr(u64 addr)
+flag_t *flag_get_by_addr(ut64 addr)
 {
 	struct list_head *pos;
+#if USE_BTREE
+	flag_t *item;
+	flag_t tmp = { .offset = addr };
+	item = btree_get(flags_off, &tmp, cmp);
+//eprintf("GET_N (0x%08llx) = %p\n", off, i);
+	return item;
+#else
 	list_for_each(pos, &flags) {
 		flag_t *flag = (flag_t *)list_entry(pos, flag_t, list);
+		if (config.interrupted) break;
 		if (flag->offset == addr)
 			return flag;
 	}
 	return NULL;
+#endif
 }
 
 flag_t *flag_get_i(int id)
@@ -177,7 +211,7 @@ void flag_show(flag_t *flag, int cmd_flag)
 	cons_printf("0x%08llx %lld %s\n",
 		flag->offset, flag->length, flag->name);
 	if (cmd_flag && flag->cmd!=NULL && *flag->cmd) {
-		u64 seek = config.seek;
+		ut64 seek = config.seek;
 		radare_seek(flag->offset, SEEK_SET);
 		radare_cmd_raw(flag->cmd, 0);
 		radare_seek(seek, SEEK_SET);
@@ -193,11 +227,11 @@ void flag_grep_help()
 	eprintf("  fgn imp.\n");
 }
 
-void flag_grep_np(const char *str, u64 addr, int next)
+void flag_grep_np(const char *str, ut64 addr, int next)
 {
 	struct list_head *pos;
 	flag_t *fag = NULL;
-	u64 newaddr;
+	ut64 newaddr;
 
 	newaddr  = next?0:0xffffffffffffffffLL;
 
@@ -226,7 +260,7 @@ void flag_grep_np(const char *str, u64 addr, int next)
 }
 
 // TODO: USE GLOB OR SO...
-void flag_grep(const char *grepstr) // TODO: add u64 arg to grep only certain address
+void flag_grep(const char *grepstr) // TODO: add ut64 arg to grep only certain address
 {
 	int cmd_flag = config_get_i("cmd.flag"); /* boolean */
 	char *grep;
@@ -267,7 +301,7 @@ flag_t *flag_get(const char *name)
 	return NULL;
 }
 
-u64 flag_get_addr(const char *name)
+ut64 flag_get_addr(const char *name)
 {
 	flag_t *foo = flag_get(name);
 	if (foo)
@@ -275,7 +309,7 @@ u64 flag_get_addr(const char *name)
 	return 0;
 }
 
-u64 flag_delta_between(u64 from, u64 to)
+ut64 flag_delta_between(ut64 from, ut64 to)
 {
 	struct list_head *pos;
 
@@ -292,7 +326,7 @@ flag_t *flag_get_next(int delta)
 {
 	flag_t *nice = NULL;
 	struct list_head *pos;
-
+//TODO: NOT YET BTREEIZED
 	if (delta == 1) {
 		list_for_each(pos, &flags) {
 			flag_t *flag = list_entry(pos, flag_t, list);
@@ -312,9 +346,7 @@ flag_t *flag_get_next(int delta)
 				if (nice) {
 					if (flag->offset > nice->offset)
 						nice = flag;
-				} else {
-					nice = flag;
-				}
+				} else nice = flag;
 			}
 		}
 	}
@@ -328,7 +360,7 @@ flag_t *flag_get_reset()
 }
 
 // TODO : use flag size
-int flags_between(u64 from, u64 to)
+int flags_between(ut64 from, ut64 to)
 {
 	int n=0;
 	struct list_head *pos;
@@ -419,34 +451,16 @@ void flags_setenv()
 	}
 }
 
-flag_t *flag_by_offset(u64 offset)
+/* XXX DEPRECATED BY by_addr() */
+flag_t *flag_by_offset(ut64 offset)
 {
-	struct list_head *pos;
-
-	list_for_each(pos, &flags) {
-		flag_t *flag = (flag_t *)list_entry(pos, flag_t, list);
-		if (flag->offset == offset)
-			return flag;
-		if (config.interrupted) break;
-	}
-
-	return NULL;
+	return flag_get_by_addr(offset);
 }
 
-const char *flag_name_by_offset(u64 offset)
+const char *flag_name_by_offset(ut64 offset)
 {
-	struct list_head *pos;
-
-	//if (offset > config.vaddr)
-	offset=offset-config.paddr+config.vaddr;
-	list_for_each(pos, &flags) {
-		flag_t *flag = (flag_t *)list_entry(pos, flag_t, list);
-		if (flag->offset == offset)
-			return flag->name;
-		if (config.interrupted) break;
-	}
-
-	return nullstr;
+	flag_t *f = flag_get_by_addr(offset);
+	return f?f->name:nullstr;
 }
 
 /* 
@@ -455,53 +469,56 @@ const char *flag_name_by_offset(u64 offset)
  * idx =  0 : just return the first one found
  * idx =  1 : continue searching for flag after the first one
  */
-int string_flag_offset(char *buf, u64 seek, int idx)
+// XXX: *buf can be easily overflowed
+#define FLAG_MAXCOUNT 10
+int string_flag_offset(const char *match, char *buf, ut64 seek, int idx)
 {
 	int delta;
 	flag_t *ref = NULL;
 	struct list_head *pos;
 	int found = 0;
+	int count = 0;
 	char buf2[256];
 
-#warning TODO: Implement delta
-#if 0
-		if ((flag->offset >= seek && flag->offset <= seek+128)) {
-		//|| (flag->offset >= seek && flag->offset <= seek+config.vaddr)) {
-			if (idx == -1) {
-				if (buf[0])
-					strcat(buf, ",");
-				strcat(buf, flag->name);
-				snprintf(buf2, 32, "+0x%llx", flag->offset-seek);
-				strcat(buf, buf2);
-			}
-		} else
-#endif
 	buf[0]='\0';
 
 	list_for_each(pos, &flags) {
-		if (config.interrupted) break;
 		flag_t *flag = (flag_t *)list_entry(pos, flag_t, list);
+		if (config.interrupted) break;
 		if (flag->offset == seek || flag->offset == seek+config.vaddr) {
 			found = 1;
-			if (idx<1) {
+			if (++count > FLAG_MAXCOUNT)
+				break;
+			if (idx != -2 && idx<1) {
 				if (buf[0])
 					strcat(buf, ",");
-				strcat(buf, flag->name);
+				strcat(buf, flag->name); // slow as shit
 			} else {
-				ref = flag;
-				break;
+				if (match && *match) {
+					if (strstr(flag->name, match))
+						ref = flag;
+				} else {
+					ref = flag;
+					break;
+				}
 			}
 		} else
 		if (flag->offset <= seek+config.vaddr && (!ref || flag->offset > ref->offset)) {
-			if (!found)
-				ref = flag;
+			if (!found) {
+				if (match && *match) {
+					if (strstr(flag->name, match))
+						ref = flag;
+				} else ref = flag;
+			}
 		}
 	}
 	if (idx==-1)
 		return buf[0]!='\0';
 	if (idx==-2) {
-		if (buf[0]=='\0') {
-			if (ref) {
+		if (buf[0]=='\0' && ref) {
+			if (match && *match) {
+				strcpy(buf, ref->name);
+			} else {
 				int delta = (int)((seek+config.vaddr)-ref->offset);
 				if (delta<0)
 					snprintf(buf2, 32, "%s-0x%x", ref->name, -delta);
@@ -526,10 +543,10 @@ int string_flag_offset(char *buf, u64 seek, int idx)
 	return 0;
 }
 
-void print_flag_offset(u64 seek)
+void print_flag_offset(const char *match, ut64 seek)
 {
-	char buf[256];
-	if ( string_flag_offset(buf, seek, -2) )
+	char buf[512];
+	if (string_flag_offset(match, buf, seek, -2))
 		cons_strcat(buf);
 }
 
@@ -537,16 +554,12 @@ void flag_do_empty(flag_t *flag)
 {
 	if (flag == NULL)
 		return;
-
 	flag->name[0]='\0';
 }
 
 int flag_is_empty(flag_t *flag)
 {
-	if (flag == NULL || flag->name[0]=='\0')
-		return 1;
-
-	return 0;
+	return (flag == NULL || flag->name[0]=='\0')?1:0;
 }
 
 void flag_list(char *arg)
@@ -560,19 +573,13 @@ void flag_list(char *arg)
 		/* filter per flag spaces */
 		if ((flag_space_idx != -1) && (flag->space != flag_space_idx))
 			continue;
-
 		/* TODO: use grep here */
 		//if (!arg || !*arg || strstr(flag->name, arg)
 		flag_show(flag, 0);
-#if 0
-		cons_printf("%03d 0x%08llx %4lld %s",
-			i++, flag->offset, flag->length, flag->name);
-#endif
-	// TODO: use flags[i]->format over flags[i]->data and flags[i]->length
 	}
 }
 
-void flag_clear_by_addr(u64 seek)
+void flag_clear_by_addr(ut64 seek)
 {
 	struct list_head *pos;
 
@@ -581,6 +588,9 @@ void flag_clear_by_addr(u64 seek)
 		flag_t *flag = (flag_t *)list_entry(pos, flag_t, list);
 		if (config.interrupted) break;
 		if (flag->offset == seek) {
+#if USE_BTREE
+#warning BTREE HERE
+#endif
 			list_del(&flag->list);
 			free(flag);
 			pos = flags.prev;
@@ -591,12 +601,10 @@ void flag_clear_by_addr(u64 seek)
 
 void flag_space_move(const char *name)
 {
-	flag_t *f;
-
 	if (name == NULL || name[0] == '\0' || name[0]=='?') {
 		eprintf("Usage: fm <flag_name>  - moves the selected flag to the current flagspace\n");
 	} else {
-		f = flag_get(name);
+		flag_t *f = flag_get(name);
 		f->space = flag_space_idx;
 	}
 }
@@ -604,6 +612,8 @@ void flag_space_move(const char *name)
 void flag_space_set(const char *name)
 {
 	int i;
+	if (name==NULL)
+		return;
 	for(i=0;i<FLAG_SPACES;i++) {
 		if (flag_spaces[i].name != NULL)
 		if (!strcmp(name, flag_spaces[i].name)) {
@@ -702,9 +712,25 @@ void flag_space(const char *name)
 	}
 }
 
-void flag_remove(const char *name)
+void flag_remove_at(ut64 addr)
 {
 	struct list_head *pos;
+	list_for_each(pos, &flags) {
+		flag_t *flag = (flag_t *)list_entry(pos, flag_t, list);
+		if (flag->offset == addr) {
+#if USE_BTREE
+			btree_del(f->tree, item, cmp, NULL);
+			btree_del(f->ntree, item, ncmp, NULL);
+#endif
+			list_del(&(flag->list));
+			break;
+		}
+	}
+}
+
+void flag_remove(const char *name)
+{
+	struct list_head *pos, *n;
 	char *str, *mask;
 	int l;
 	int found = 0;
@@ -718,10 +744,14 @@ void flag_remove(const char *name)
 	if (mask) {
 		mask[0]='\0';
 		l = strlen(str);
-		list_for_each(pos, &flags) {
+		list_for_each_safe(pos, n, &flags) {
 			flag_t *flag = (flag_t *)list_entry(pos, flag_t, list);
 			if (config.interrupted) break;
 			if (!memcmp(str, flag->name, l)) {
+#if USE_BTREE
+				btree_del(f->tree, item, cmp, NULL);
+				btree_del(f->ntree, item, ncmp, NULL);
+#endif
 				list_del(&(flag->list));
 				free(flag);
 				pos = flags.next;
@@ -729,15 +759,16 @@ void flag_remove(const char *name)
 			}
 		}
 	} else {
-		__restart2:
-		list_for_each(pos, &flags) {
+		list_for_each_safe(pos, n, &flags) {
 			flag_t *flag = (flag_t *)list_entry(pos, flag_t, list);
 			if (config.interrupted) break;
 			if (!strcmp(name, flag->name)) {
+#if USE_BTREE
+#warning btree here
+#endif
 				list_del(&(flag->list));
 				free(flag);
 				found = 1;
-				goto __restart2;
 			}
 		}
 	}
@@ -750,10 +781,8 @@ void flag_remove(const char *name)
 
 int flag_qsort_compare(const void *a, const void *b)
 {
-	if (a == NULL)
-		return -1;
-	if (b == NULL)
-		return 1;
+	if (a == NULL) return -1;
+	if (b == NULL) return 1;
 	return strcmp(b, a);
 }
 
@@ -790,24 +819,6 @@ int flag_is_valid_char(const char ch)
 }
 
 #define MAX_FLAG_LEN 20
-int flag_filter_name(char *name)
-{
-	int i;
-	char *oname;
-	for(;*name==' ';name=name+1);
-	oname=name;
-	for(i=0;*name!='\0'; name = name +1,i++) {
-		if (i>MAX_FLAG_LEN) {
-			name[0]='\0';
-			break;
-		}
-		if (!flag_is_valid_char(*name)) {
-			strbcpy(name, name+1);
-			name = name -1;
-		}
-	}
-	return flag_is_valid_name(oname);
-}
 
 int flag_is_valid_name(const char *name)
 {
@@ -820,7 +831,26 @@ int flag_is_valid_name(const char *name)
 	return 1;
 }
 
-int flag_set(const char *name, u64 addr, int dup)
+int flag_filter_name(char *name)
+{
+	int i;
+	char *oname;
+	for(;*name==' ';name=name+1);
+	oname=name;
+	for(i=0;*name!='\0'; name = name +1,i++) {
+		if (i>MAX_FLAG_LEN) {
+			name[0]='\0';
+			break;
+		}
+		if (!flag_is_valid_char(*name)) {
+			str_cpy(name, name+1);
+			name = name -1;
+		}
+	}
+	return flag_is_valid_name(oname);
+}
+
+int flag_set(const char *name, ut64 addr, int dup)
 {
 	const char *ptr;
 	flag_t *flag = NULL;
@@ -877,12 +907,17 @@ int flag_set(const char *name, u64 addr, int dup)
 	if (flag == NULL) {
 		flag = malloc(sizeof(flag_t));
 		memset(flag,'\0', sizeof(flag_t));
+#if USE_BTREE
+		btree_add(&flags_nam, flag, cmp);
+		btree_add(&flags_off, flag, ncmp);
+#endif
 		list_add_tail(&(flag->list), &flags);
 		if (flag==NULL)
 			return 1;
 	}
 
 	strncpy(flag->name, name, FLAG_BSIZE);
+	flag->namehash = strhash(flag->name);
 	flag->name[FLAG_BSIZE-1]='\0';
 	flag->offset = addr + flag_from_i;
 	flag->space = flag_space_idx;
@@ -902,7 +937,7 @@ int flag_set(const char *name, u64 addr, int dup)
 	return 0;
 }
 
-int flag_set_undef(const char *name, u64 addr, int dup)
+int flag_set_undef(const char *name, ut64 addr, int dup)
 {
 	flag_t *flag = flag_get_by_addr(addr);
 	if (flag == NULL || (strlen(flag->name)<4))
@@ -911,13 +946,13 @@ int flag_set_undef(const char *name, u64 addr, int dup)
 }
 
 /* used to get section name for disasembly */
-const char *flag_get_here_filter(u64 at, const char *str)
+const char *flag_get_here_filter(ut64 at, const char *str)
 {
-	static u64 sec_start=0, sec_end=0;
+	static ut64 sec_start=0, sec_end=0;
 	static char sec_str[64];
 	struct list_head *pos;
-	u64 ret = 0;
-	u64 nextflag = 0xFFFFFFFFFFFFFFFFFFLL;
+	ut64 ret = 0;
+	ut64 nextflag = U64_MAX;
 	flag_t *f = NULL;
 
 	if (at >=sec_start && at <sec_end) {
@@ -948,15 +983,15 @@ const char *flag_get_here_filter(u64 at, const char *str)
 }
 
 //XXX ugly hack
-const char *flag_get_here_filter2(u64 at, const char *str, const char *str2)
+const char *flag_get_here_filter2(ut64 at, const char *str, const char *str2)
 {
-	static u64 sec_start=0, sec_end=0;
+	static ut64 sec_start=0, sec_end=0;
 	static char sec_str[64];
 	struct list_head *pos;
-	u64 ret = 0;
-	u64 nextflag = 0xFFFFFFFFFFFFFFFFFFLL;
+	ut64 ret = 0;
+	ut64 nextflag = U64_MAX;
 	flag_t *f = NULL;
-	char *s1,*s2,*s=NULL;
+	const char *s1,*s2,*s=NULL;
 
 	if (at >=sec_start && at <sec_end) {
 		/* ADDED CACHE PAWAH */
@@ -993,7 +1028,6 @@ void flags_visual_menu()
 {
 	char cmd[1024];
 	struct list_head *pos;
-#define MAX_FORMAT 2
 	int format = 0;
 	const char *ptr;
 	const char *fs = NULL;
@@ -1006,13 +1040,13 @@ void flags_visual_menu()
 	int hit;
 
 	while(1) {
-		cons_gotoxy(0,0);
+		cons_gotoxy(0, 0);
 		cons_clear();
 		/* Execute visual prompt */
 		ptr = config_get("cmd.vprompt");
 		if (ptr&&ptr[0]) {
 			int tmp = last_print_format;
-			radare_cmd_raw(ptr, 0);
+			radare_cmd_raw (ptr, 0);
 			last_print_format = tmp;
 		}
 
@@ -1043,7 +1077,7 @@ void flags_visual_menu()
 			hit = 0;
 			i = j = 0;
 			list_for_each(pos, &flags) {
-				flag_t *flag = (flag_t *)list_entry(pos, flag_t, list);
+				flag_t *flag = (flag_t *)list_entry (pos, flag_t, list);
 				/* filter per flag spaces */
 				if ((flag_space_idx != -1) && (flag->space != flag_space_idx))
 					continue;
@@ -1091,9 +1125,13 @@ void flags_visual_menu()
 		switch(ch) {
 		case 'J':
 			option+=10;
+			if (option>=i)
+				option = i-1;
 			break;
 		case 'j':
 			option++;
+			if (option>=i)
+				option = i-1;
 			break;
 		case 'k':
 			if (--option<0)
@@ -1172,13 +1210,15 @@ void flags_visual_menu()
 		case ':':
 			cons_set_raw(0);
 #if HAVE_LIB_READLINE
-			char *ptr = (char *)readline(VISUAL_PROMPT);
+{
+			char *ptr = readline(VISUAL_PROMPT);
 			if (ptr) {
 				strncpy(cmd, ptr, sizeof(cmd));
 				radare_cmd(cmd, 1);
 				//commands_parse(line);
 				free(ptr);
 			}
+}
 #else
 			cmd[0]='\0';
 			dl_prompt = ":> ";
