@@ -20,16 +20,16 @@
 #include "main.h"
 
 struct ugraph_node_t {
-	u64 from;
-	u64 size;
+	ut64 from;
+	ut64 size;
 	char cmd[128];
 	void *ptr;
 	struct list_head list;
 };
 
 struct ugraph_edge_t {
-	u64 from;
-	u64 to;
+	ut64 from;
+	ut64 to;
 	struct list_head list;
 };
 
@@ -40,9 +40,9 @@ struct ugraph_t {
 
 static struct ugraph_t ug;
 
-char *ugraph_get_str(u64 addr, int size, int dot);
+char *ugraph_get_str(ut64 addr, int size, int dot);
 
-int graph_get_bb_size(struct program_t *prg, u64 addr)
+int graph_get_bb_size(struct program_t *prg, ut64 addr)
 {
 	struct block_t *b0;
 	struct list_head *head;
@@ -72,10 +72,10 @@ void graph_viz(struct program_t *prg, int body)
 		if (withweight) {
 			if (b0->tnext)
 				cons_printf("\t\"0x%08llx\" -> \"0x%08llx\" [color=\"green\" weight=\"%d\"];\n",
-						b0->addr, b0->tnext, (int)ABS(b0->tnext-b0->addr));
+						b0->addr, b0->tnext, (int)R_ABS(b0->tnext-b0->addr));
 			if (b0->fnext)
 				cons_printf("\t\"0x%08llx\" -> \"0x%08llx\" [color=\"red\" weight=\"%d\"];\n",
-						b0->addr, b0->fnext, (int)ABS(b0->fnext-b0->addr));
+						b0->addr, b0->fnext, (int)R_ABS(b0->fnext-b0->addr));
 		} else {
 			if (b0->tnext)
 				cons_printf("\t\"0x%08llx\" -> \"0x%08llx\" [color=\"green\"];\n", b0->addr, b0->tnext);
@@ -175,7 +175,7 @@ static struct program_t *prg = NULL; // last program code analysis
 void core_load_graph_at(void *obj, const char *str)
 {
 	//GravaWidget *widget = obj;
-	u64 off;
+	ut64 off;
 
 	monitors_run();
 	if (graphuser) {
@@ -200,7 +200,7 @@ void core_load_graph_at(void *obj, const char *str)
 void mygrava_bp_at(void *unk, const char *str)
 {
 	char buf[1024];
-	u64 off = get_offset(str);
+	ut64 off = get_offset(str);
 	sprintf(buf, "!bp %s", str);
 	radare_cmd(buf, 0);
 	eprintf("Breakpoint at (%08llx) (%s) added.\n", off, str);
@@ -210,7 +210,7 @@ void mygrava_bp_at(void *unk, const char *str)
 void mygrava_bp_rm_at(void *unk, const char *str)
 {
 	char buf[1024];
-	u64 off = get_offset(str);
+	ut64 off = get_offset(str);
 	sprintf(buf, "!bp -%s", str);
 	radare_cmd(buf, 0);
 	eprintf("Breakpoint at (%08llx) (%s) removed.\n", off, str);
@@ -282,7 +282,7 @@ static void core_load_graph_entry(void *widget, void *obj)
 	char *buf, *ptr;
 	struct program_t *prg;
 	struct mygrava_window *w = obj;
-	u64 off;
+	ut64 off;
 
 	if (w) {
 		str = gtk_entry_get_text(GTK_ENTRY(w->entry));
@@ -297,7 +297,8 @@ static void core_load_graph_entry(void *widget, void *obj)
 //	ptr = config_get("graph.layout");
 //	if (ptr && !strcmp(ptr, "graphviz"))
 //	else
-	grava_default_layout_reset_layout(last_window->grava->graph->layout);
+	grava_default_layout_reset_layout(GRAVA_DEFAULT_LAYOUT(
+		(last_window->grava->graph->layout)));
 
 	eprintf("Loading graph... (%s) 0x%llx\n", str, off);
 	if (off == 0 && str[0]!='0') {
@@ -310,11 +311,11 @@ static void core_load_graph_entry(void *widget, void *obj)
 		radare_cmd_raw(str, 0);
 		ptr = (char *)config_get("scr.seek");
 		if (ptr&&ptr[0]) {
-			u64 off = get_math(ptr);
+			ut64 off = get_math(ptr);
 			if (off != 0)
 				radare_seek(off, SEEK_SET);
 		} 
-		buf = cons_get_buffer();
+		buf = (char *)cons_get_buffer();
 		if (buf && buf[0]) {
 			printf("BUFFER(%s->%s)\n", str, buf);
 			gtk_text_buffer_set_text(gtk_text_view_get_buffer(GTK_TEXT_VIEW(w->text)), buf, -1);
@@ -480,7 +481,7 @@ struct mygrava_window *mygrava_get_widget(struct program_t *prg, int new)
 	#endif
 		g_signal_connect(G_OBJECT(win->w), "key_press_event", G_CALLBACK(grava_key_press_cb), win);
 		g_signal_connect (win->w, "destroy", G_CALLBACK (mygrava_close), win);
-		string_flag_offset(name, config.seek, -1);
+		string_flag_offset(NULL, name, config.seek, -1);
 		sprintf(title, "code graph: %s (0x%08x) %s", config.file, (unsigned int )config.seek, name);
 		gtk_window_set_title(GTK_WINDOW(win->w), title);
 	}
@@ -600,7 +601,7 @@ void do_grava_analysis_callgraph(struct program_t *prg, struct mygrava_window *w
 
 		/* label */
 		// TODO: support for real labelling stuff
-		string_flag_offset(cmd, b0->addr);
+		string_flag_offset(NULL, cmd, b0->addr);
 		cmd[127]='\0'; // XXX ugly string recycle hack
 		sprintf(cmd+128, "0x%08llx  %s", b0->addr, cmd);
 		grava_node_set(node, "color", "gray");
@@ -612,7 +613,7 @@ void do_grava_analysis_callgraph(struct program_t *prg, struct mygrava_window *w
 
 #if 0
 		// XXX this makes radare segfault with g_object_unref HUH!
-		if (flags_between((u64)b0->addr,(u64)(b0->addr + b0->n_bytes))>0)
+		if (flags_between((ut64)b0->addr,(ut64)(b0->addr + b0->n_bytes))>0)
 			grava_node_set(node, "bgcolor", "yellow");
 #endif
 
@@ -701,7 +702,7 @@ void ugraph_reset()
 	graphuserinit = 1;
 }
 
-void ugraph_node(u64 from, u64 size, const char *cmd)
+void ugraph_node(ut64 from, ut64 size, const char *cmd)
 {
 	struct list_head *pos;
 	struct ugraph_node_t *n;
@@ -723,7 +724,7 @@ void ugraph_node(u64 from, u64 size, const char *cmd)
 	list_add_tail(&(n->list), &ug.nodes);
 }
 
-void ugraph_edge(u64 from, u64 to)
+void ugraph_edge(ut64 from, ut64 to)
 {
 	struct ugraph_edge_t *e = MALLOC_STRUCT(struct ugraph_edge_t);
 	
@@ -734,7 +735,7 @@ void ugraph_edge(u64 from, u64 to)
 	list_add_tail(&(e->list), &ug.edges);
 }
 
-struct ugraph_node_t *ugraph_get(u64 addr)
+struct ugraph_node_t *ugraph_get(ut64 addr)
 {
 	struct list_head *head;
 	if (!graphuserinit)
@@ -747,13 +748,13 @@ struct ugraph_node_t *ugraph_get(u64 addr)
 	return NULL;
 }
 
-char *ugraph_get_str(u64 addr, int _size, int dot)
+char *ugraph_get_str(ut64 addr, int _size, int dot)
 {
 	int i,j;
 	char *cmdstr, *str = NULL;
 	char *cmd = "pd";
 	struct ugraph_node_t *n = ugraph_get(addr);
-	u64 from = addr;
+	ut64 from = addr;
 	int size = _size;
 	if (n) {
 		from = n->from;
@@ -762,9 +763,7 @@ char *ugraph_get_str(u64 addr, int _size, int dot)
 	}
 	radare_seek(from, SEEK_SET);
 	radare_set_block_size_i(size);
-//eprintf("PRE(%s)\n", cons_get_buffer());
 	cmdstr = radare_cmd_str(cmd);
-//eprintf("POST(%s)\n", cons_get_buffer());
 	if (!dot)
 		return cmdstr;
 	if (cmdstr != NULL)
@@ -819,11 +818,10 @@ void ugraph_print_dot(int body)
 				free(str);
 			}
 		}
-		if (withweight) {
+		if (withweight)
 			cons_printf(" \"0x%08llx\" -> \"0x%08llx\" ;\n", e->from, e->to);
-		} else {
-			cons_printf(" \"0x%08llx\" -> \"0x%08llx\" [weight=\"%d\"];\n", e->from, e->to, ABS(e->to-e->from));
-		}
+		else cons_printf(" \"0x%08llx\" -> \"0x%08llx\" [weight=\"%d\"];\n",
+				e->from, e->to, R_ABS(e->to-e->from));
 	}
 	cons_printf("}\n");
 	cons_flush();
@@ -833,7 +831,7 @@ void ugraph_print_dot(int body)
 
 void load_user_graph(struct program_t *prg, struct mygrava_window *win)
 {
-	u64 oseek = config.seek;
+	ut64 oseek = config.seek;
 	int i;
 	char *ptr;
 	char cmd[1024];
@@ -950,7 +948,7 @@ void do_grava_analysis(struct program_t *prg, struct mygrava_window *win)
 
 		/* label */
 		// TODO: support for real labelling stuff
-		string_flag_offset(cmd, b0->addr, -1);
+		string_flag_offset(NULL, cmd, b0->addr, -1);
 		cmd[127]='\0'; // XXX ugly string recycle hack
 		sprintf(cmd+128, "0x%08llx  %s", b0->addr, cmd);
 		if (!graph_flagblocks)
@@ -963,7 +961,7 @@ void do_grava_analysis(struct program_t *prg, struct mygrava_window *win)
 
 #if 0
 		// XXX this makes radare segfault with g_object_unref HUH!
-		if (flags_between((u64)b0->addr,(u64)(b0->addr + b0->n_bytes))>0)
+		if (flags_between((ut64)b0->addr,(ut64)(b0->addr + b0->n_bytes))>0)
 			grava_node_set(node, "bgcolor", "yellow");
 #endif
 
@@ -1053,7 +1051,7 @@ ptr= ugraph_get_str (b0->addr, b0->n_bytes, 0);
 #if HAVE_GUI
 void grava_program_graph(struct program_t *prg, struct mygrava_window *win)
 {
-	u64 here = config.seek;
+	ut64 here = config.seek;
 	//int graph_flagblocks = (int)config_get("graph.flagblocks");
 
 	/* create widget */
