@@ -22,6 +22,7 @@
 #define FALSE 0
 #define ASSERT //
 #define assert //
+#undef LOBYTE
 #define LOBYTE(_w)      ((_w) & 0xff)
 #define NEXT_CHAR(_x)   _x + 1
 #define PROFILER_IN(fd,fun,line) //
@@ -29,5 +30,6 @@
 #define PROFILER_ROUT(fd,fun,line,ret) return ret
 #define PROFILER_ERR(fd,fun,line,str,ret) { fprintf(stderr, str"\n"); return ret; }
 #define u_char unsigned char
+#define uint8_t unsigned char
 
 #endif
