@@ -74,6 +74,7 @@ format_info_t formats[] = {
 	{ '7', FMT_7BIT,       "7bit encoding (sms)",    NULL,   "entire block" },
 	{ '8', FMT_HEXQ,       "p8: 8bytes, 64 bit quad-word",  "8 bytes",     "(endian)"},
 	{ '9', FMT_EBASE64,    "p9: base64 decode (p6 to encode)",  "entire block",     "(endian)"},
+	{ 'w', FMT_DWORDDUMP,  "endian sensitive dword dump", "N byte",      "entire block" },
 	{ 'x', FMT_HEXB,       "hexadecimal dump", "N byte",      "entire block" },
 	{ 'X', FMT_HEXPAIRS,   "hexpairs", "N byte",      "entire block" },
 	{ 'z', FMT_ASC0,       "ascii null terminated",  NULL,          "until \\0" },
@@ -110,6 +111,16 @@ void getHTTPDate(char *DATE)
 #endif
 }
 
+const char* print_mem_get(const char *name)
+{
+	struct list_head *pos;
+	list_for_each_prev(pos, &print_mems) {
+		print_mem_t *im = list_entry(pos, print_mem_t, list);
+		if (!strcmp(name, im->name))
+			return im->fmt;
+	}
+	return NULL;
+}
 
 /* msdos date format */
 // From freebsd kernel msdosfs/direntry.h
@@ -125,7 +136,7 @@ void print_msdos_date(unsigned char _time[2], unsigned char _date[2])
         unsigned int seconds = (t&0x001f)<<1;
 
         /* la data de modificacio del fitxer, no de creacio del zip */
-        printf("%d-%02d-%02d %d:%d:%d",
+        cons_printf("%d-%02d-%02d %d:%d:%d\n",
                 year, month, day, hour, minutes, seconds);
 }
 
@@ -160,7 +171,7 @@ void format_show_help (print_mode_t mode)
 
 /* helpers */
 
-void print_addr(u64 off)
+void print_addr(ut64 off)
 {
 	int mod = config_get_i("cfg.addrmod");
 	char ch = (0==(off%(mod?mod:1)))?',':' ';
@@ -365,21 +376,7 @@ void print_mem_help()
 
 struct list_head print_mems;
 
-void print_mem_add(char *name, char *fmt)
-{
-	print_mem_t *pm = MALLOC_STRUCT(print_mem_t);
-	while(name[0]==' ')name=name+1;
-	while(fmt[0]==' ')fmt=fmt+1;
-	if (!*name || !*fmt) {
-		eprintf("Usage: am foo xxd\n");
-		return;
-	}
-	pm->name = strdup(name);
-	pm->fmt = strdup(fmt);
-	list_add_tail(&(pm->list), &(print_mems));
-}
-
-void print_mem_del(char *name)
+int print_mem_del(char *name)
 {
 	struct list_head *pos;
 	list_for_each_prev(pos, &print_mems) {
@@ -389,9 +386,26 @@ void print_mem_del(char *name)
 			free(im->fmt);
 			list_del(&(im->list));
 			free(im);
-			return;
+			return 1;
 		}
 	}
+	return 0;
+}
+
+void print_mem_add(char *name, char *fmt)
+{
+	print_mem_t *pm = MALLOC_STRUCT(print_mem_t);
+	while(name[0]==' ')name=name+1;
+	while(fmt[0]==' ')fmt=fmt+1;
+	if (print_mem_del(name)) /* overwrite */
+		eprintf("am format '%s' redefined.\n", name);
+	if (!*name || !*fmt) {
+		eprintf("Usage: am foo xxd\n");
+		return;
+	}
+	pm->name = strdup(name);
+	pm->fmt = strdup(fmt);
+	list_add_tail(&(pm->list), &(print_mems));
 }
 
 void print_mem_list(char *name, char *fmt)
@@ -403,28 +417,20 @@ void print_mem_list(char *name, char *fmt)
 	}
 }
 
-const char *print_mem_get(char *name)
-{
-	struct list_head *pos;
-	list_for_each_prev(pos, &print_mems) {
-		print_mem_t *im = list_entry(pos, print_mem_t, list);
-		if (!strcmp(name, im->name))
-			return im->fmt;
-	}
-	return NULL;
-}
-
 /* TODO: add support for 64bit pointers */
 /* following asm.bits ??? */
-void print_mem(u64 addr, const u8 *buf, u64 len, const char *fmt, int endian)
+void print_mem(ut64 addr, const u8 *buf, ut64 len, const char *fmt, int endian)
 {
 	unsigned char buffer[256];
-	int i,j,idx;
+	int i, j, idx;
+	char *rbuf = NULL;
 	int times, otimes;
 	char tmp, last;
 	char *args, *bracket;
 	int nargs;
-	u64 paddr = 0LL;
+	ut64 paddr = 0LL;
+	ut64 oaddr = 0LL;
+	const char *cfmt = fmt;
 	const char *arg = fmt;
 	char hexfmt[32];
 	int ptrsize = (int)(config_get_i("asm.bits")/8);
@@ -473,36 +479,35 @@ void print_mem(u64 addr, const u8 *buf, u64 len, const char *fmt, int endian)
 			addr = 0LL;
 			switch (ptrsize) {
 			case 1:
-				addr =  (u64)(*(buf+i));
+				addr =  (ut64)(*(buf+i));
 				strcpy(hexfmt, "0x%02llx");
 				break;
 			case 2:
 				if (endian) 
-					addr =  (u64)(*(buf+i))<<8 | (u64)(*(buf+i+1));
-				else
-					addr =  (u64)(*(buf+i+1))<<8 | (u64)(*(buf+i));
+					addr = (ut64)(*(buf+i))<<8 | (ut64)(*(buf+i+1));
+				else addr = (ut64)(*(buf+i+1))<<8 | (ut64)(*(buf+i));
 				strcpy(hexfmt, "0x%04llx");
 				break;
 			case 4:
 				if (endian) 
-					addr =  (u64)(*(buf+i))<<24 | (u64)(*(buf+i+1))<<16 | 
-							(u64)(*(buf+i+2))<<8 | (u64)(*(buf+i+3));
+					addr =  (ut64)(*(buf+i))<<24 | (ut64)(*(buf+i+1))<<16 | 
+						(ut64)(*(buf+i+2))<<8 | (ut64)(*(buf+i+3));
 				else 
-					addr =  (u64)(*(buf+i+3))<<24 | (u64)(*(buf+i+2))<<16 | 
-							(u64)(*(buf+i+1))<<8 | (u64)(*(buf+i));
+					addr =  (ut64)(*(buf+i+3))<<24 | (ut64)(*(buf+i+2))<<16 | 
+						(ut64)(*(buf+i+1))<<8 | (ut64)(*(buf+i));
 				strcpy(hexfmt, "0x%08llx");
 				break;
 			case 8:
 				if (endian) 
-					addr =  (u64)(*(buf+i))<<56 | (u64)(*(buf+i+1))<<48 | 
-							(u64)(*(buf+i+2))<<40 | (u64)(*(buf+i+3))<<32 | 
-							(u64)(*(buf+i+4))<<24 | (u64)(*(buf+i+5))<<16 | 
-							(u64)(*(buf+i+6))<<8 | (u64)(*(buf+i+7));
+					addr =  (ut64)(*(buf+i))<<56 | (ut64)(*(buf+i+1))<<48 | 
+						(ut64)(*(buf+i+2))<<40 | (ut64)(*(buf+i+3))<<32 | 
+						(ut64)(*(buf+i+4))<<24 | (ut64)(*(buf+i+5))<<16 | 
+						(ut64)(*(buf+i+6))<<8 | (ut64)(*(buf+i+7));
 				else 
-					addr =  (u64)(*(buf+i+7))<<56 | (u64)(*(buf+i+6))<<48 | 
-							(u64)(*(buf+i+5))<<40 | (u64)(*(buf+i+4))<<32 | 
-							(u64)(*(buf+i+3))<<24 | (u64)(*(buf+i+2))<<16 | 
-							(u64)(*(buf+i+1))<<8 | (u64)(*(buf+i));
+					addr =  (ut64)(*(buf+i+7))<<56 | (ut64)(*(buf+i+6))<<48 | 
+						(ut64)(*(buf+i+5))<<40 | (ut64)(*(buf+i+4))<<32 | 
+						(ut64)(*(buf+i+3))<<24 | (ut64)(*(buf+i+2))<<16 | 
+						(ut64)(*(buf+i+1))<<8 | (ut64)(*(buf+i));
 				strcpy(hexfmt, "0x%016llx");
 			}
 			tmp = *arg;
@@ -557,7 +562,7 @@ void print_mem(u64 addr, const u8 *buf, u64 len, const char *fmt, int endian)
 			case '?': // help
 				print_mem_help();
 				idx--;
-				i=len; // exit
+				i = len; // force while break
 				continue;
 			case '[':
 				{
@@ -567,16 +572,54 @@ void print_mem(u64 addr, const u8 *buf, u64 len, const char *fmt, int endian)
 						return;
 					}
 					*end='\0';
-					fmt = print_mem_get(arg+1);
-					if (fmt) {
-#if 0 
+					oaddr = addr;
+					if (arg[1]=='*') {
+						/* follow pointer */
+						arg = arg+1;
+						/* SPAGUETTI FTW */
+						switch (ptrsize) {
+						case 1:
+							addr =  (ut64)(*(buf+i));
+							break;
+						case 2:
+							if (endian) addr = (ut64)(*(buf+i))<<8 | (ut64)(*(buf+i+1));
+							else addr = (ut64)(*(buf+i+1))<<8 | (ut64)(*(buf+i));
+							break;
+						case 4:
+							if (endian) addr = (ut64)(*(buf+i))<<24 | (ut64)(*(buf+i+1))<<16 | 
+								(ut64)(*(buf+i+2))<<8 | (ut64)(*(buf+i+3));
+							else addr = (ut64)(*(buf+i+3))<<24 | (ut64)(*(buf+i+2))<<16 | 
+								(ut64)(*(buf+i+1))<<8 | (ut64)(*(buf+i));
+							break;
+						case 8:
+							if (endian) addr = (ut64)(*(buf+i))<<56 | (ut64)(*(buf+i+1))<<48 | 
+								(ut64)(*(buf+i+2))<<40 | (ut64)(*(buf+i+3))<<32 | 
+									(ut64)(*(buf+i+4))<<24 | (ut64)(*(buf+i+5))<<16 | 
+									(ut64)(*(buf+i+6))<<8 | (ut64)(*(buf+i+7));
+							else addr = (ut64)(*(buf+i+7))<<56 | (ut64)(*(buf+i+6))<<48 | 
+								(ut64)(*(buf+i+5))<<40 | (ut64)(*(buf+i+4))<<32 | 
+									(ut64)(*(buf+i+3))<<24 | (ut64)(*(buf+i+2))<<16 | 
+									(ut64)(*(buf+i+1))<<8 | (ut64)(*(buf+i));
+						}
+
+
+						//if (oaddr != addr) {
+addr-=i;
+							rbuf = malloc(config.block_size);
+							radare_read_at((ut64)addr, rbuf, config.block_size+i);
+							radare_seek(addr, SEEK_SET);
+						//}
+					}
+					cfmt = print_mem_get(arg+1);
+					if (cfmt) {
+#if 0
 						D cons_printf("0x%08llx = ", config.seek+i);
 						paddr = buf[0]<<24 | buf[1]<<16 | buf[2]<<8 | buf[3];
 						cons_printf("0x%08llx -> pm %s {\n", addr, fmt);
 #endif 
-						cons_printf("pm %s {\n", fmt);
+						cons_printf("pm %s {\n", cfmt);
 						/* XXX: this is 32bit pointer only FUCK! */
-						print_mem(addr, buf+i, config.block_size, fmt, endian); //config.endian);
+						print_mem(addr, buf+i, config.block_size, cfmt, endian); //config.endian);
 						cons_printf("}\n");
 					} else {
 						eprintf("Unknown named print format '%s' (Use 'am name format')\n", arg+1);
@@ -584,6 +627,11 @@ void print_mem(u64 addr, const u8 *buf, u64 len, const char *fmt, int endian)
 					arg = end;
 					idx--;
 					i+=ptrsize;
+					}
+				if (oaddr != addr) {
+					free((void *)rbuf);
+					addr = oaddr;
+					radare_seek(oaddr, SEEK_SET);
 				}
 				continue;
 			}
@@ -597,11 +645,11 @@ void print_mem(u64 addr, const u8 *buf, u64 len, const char *fmt, int endian)
 				{
 				/* dirty hack */
 				int oldfmt = last_print_format;
-				u64 old = config.seek;
+				ut64 old = config.seek;
 				radare_seek(config.seek+i, SEEK_SET);
 				radare_read(0);
-				print_data(config.seek+i, "8", buf+i, 4, FMT_TIME_UNIX);
-				last_print_format=oldfmt;
+				print_data(config.seek+i, "8", (ut8*)buf+i, 4, FMT_TIME_UNIX);
+				last_print_format = oldfmt;
 				radare_seek(old, SEEK_SET);
 				}
 				break;
@@ -620,7 +668,7 @@ void print_mem(u64 addr, const u8 *buf, u64 len, const char *fmt, int endian)
 				break;
 			case 'B':
 				memset(buffer, '\0', 255);
-				radare_read_at((u64)addr, buffer, 248);
+				radare_read_at((ut64)addr, buffer, 248);
 				D cons_printf("0x%08llx = ", config.seek+i);
 				for(j=0;j<10;j++) cons_printf("%02x ", buf[j]);
 				cons_strcat(" ... (");
@@ -647,7 +695,7 @@ void print_mem(u64 addr, const u8 *buf, u64 len, const char *fmt, int endian)
 				char buf[128];
 				D cons_printf("0x%08llx = ", config.seek+i);
 				cons_printf(hexfmt, addr);
-				if (string_flag_offset(buf, addr, -1))
+				if (string_flag_offset(NULL, buf, addr, -1))
 					cons_printf(" ; %s", buf);
 				i+=ptrsize;
 				ptrsize = optrsize;
@@ -673,8 +721,8 @@ void print_mem(u64 addr, const u8 *buf, u64 len, const char *fmt, int endian)
 			case 's':
 				D cons_printf("0x%08llx = ", config.seek+i);
 				memset(buffer, '\0', 255);
-				radare_read_at((u64)addr, buffer, 248);
-				cons_printf("0x%08llx -> ", (u64)config.seek+i);
+				radare_read_at((ut64)addr, buffer, 248);
+				cons_printf("0x%08llx -> ", (ut64)config.seek+i);
 				cons_printf(hexfmt, addr);
 				cons_printf(" %s ", buffer);
 				i+=ptrsize;
@@ -698,9 +746,10 @@ void print_mem(u64 addr, const u8 *buf, u64 len, const char *fmt, int endian)
 
 static int zoom = 0;
 
-void print_zoom(u64 from, u64 to, char *byte, int enable)
+void print_zoom(ut64 from, ut64 to, const char *byte, int enable)
 {
 	zoom = enable;
+	config.zoom.enabled = enable;
 	config_set_i("zoom.from", from);
 	config_set_i("zoom.to", to);
 	config_set("zoom.byte", byte);
@@ -716,19 +765,25 @@ void print_zoom(u64 from, u64 to, char *byte, int enable)
  * mode: print mode
  *
  */
-void print_data(u64 seek, char *arg, u8 *buf, int olen, print_fmt_t fmt)
+void print_data(ut64 seek, char *arg, u8 *buf, int olen, print_fmt_t fmt)
 {
 	int tmp, i, j, k;
 	int lines   = 0;
-	int endian  = (int)config_get("cfg.bigendian");
-	int inverse = (int)config_get("cfg.inverse");
+	int endian  = PTRCAST (config_get("cfg.bigendian"));
+	int inverse = PTRCAST (config_get("cfg.inverse"));
+	const char *cstr;
 	char *str;
 	// code anal
 	struct program_t *prg;
 	unsigned char buffer[256];
-	unsigned char *bufi = NULL; // inverted buffer
+	unsigned char *bufi = NULL, *bufz = NULL; // inverted buffer
 	unsigned long addr = seek;
-	u64 len = olen;
+	ut64 len = olen;
+
+	if (olen<0) {
+		eprintf("Too many length\n");
+		return;
+	}
 
 	last_print_format = fmt;
 	if (buf == NULL)
@@ -742,65 +797,72 @@ void print_data(u64 seek, char *arg, u8 *buf, int olen, print_fmt_t fmt)
 	}
 
 	if (zoom) {
-		u64 sz = 4;
+		ut64 sz;
+		u8 *bufz2;
 		const char *mode = config_get("zoom.byte");
-		u64 ptr = config_get_i("zoom.from");
-		switch(mode[0]) {
-		case 'e':
-			buf = (u8 *)malloc(config.zoom.piece);
-			sz = (u64)config.zoom.piece;
-			break;
-		default:
-			buf = (u8 *)malloc(len<<1);
-			break;
+		ut64 ptr = config_get_i("zoom.from");
+		config.zoom.piece  = (config.zoom.to - config.zoom.from)/config.block_size;
+		if (config.zoom.piece < 1)
+			config.zoom.piece = 1;
+
+		if (!mode) {
+			eprintf("Error: Invalid zoom mode");
+			return;
 		}
+
+		bufz = (u8 *)malloc(len);
+		bufz2 = (u8 *)malloc(config.zoom.piece);
+		sz = (ut64)config.zoom.piece;
+
 		for(i=0;!config.interrupted && i<len;i++) {
 			io_lseek(config.fd, ptr, SEEK_SET);
-			buf[0]='\xff';
-			io_read(config.fd, buf, sz);
+			bufz2[0]='\xff';
+			io_read(config.fd, bufz2, sz);
 
 			switch(mode[0]) {
 			case 'F': // 0xFF
-				config.block[i] = 0;
+				bufz[i] = 0;
 				for(j=0;j<sz;j++)
-					if (buf[j]==0xff)
-						config.block[i]++;
+					if (bufz2[j]==0xff)
+						bufz[i]++;
 				break;
 			case 'c': // code
 				{
 				struct data_t *data = data_get_between(ptr, ptr+config.zoom.piece);
-				config.block[i] = (unsigned char) 0;
+				bufz[i] = (unsigned char) 0;
 				if (data->type == DATA_CODE)
-					config.block[i] = (unsigned char) data->times;
+					bufz[i] = (unsigned char) data->times;
 				}
 				break;
 			case 's': // strings
 				{
 				struct data_t *data = data_get_between(ptr, ptr+config.zoom.piece);
-				config.block[i] = (unsigned char) 0;
+				bufz[i] = (unsigned char) 0;
 				if (data->type == DATA_STR)
-					config.block[i] = (unsigned char) data->times;
+					bufz[i] = (unsigned char) data->times;
 				}
 				break;
 			case 't': // traces
-				config.block[i] = (unsigned char)trace_get_between(ptr, ptr+config.zoom.piece);
+				bufz[i] = (unsigned char)trace_get_between(ptr, ptr+config.zoom.piece);
 				break;
 			case 'f': // flags
-				config.block[i] = (unsigned char)flags_between(ptr, ptr+config.zoom.piece);
+				bufz[i] = (unsigned char)flags_between(ptr, ptr+config.zoom.piece);
 				break;
 			case 'p': // % printable chars
-				config.block[i] = (unsigned char)2.55*hash_pcprint(buf, sz);
+				bufz[i] = (unsigned char)2.55*hash_pcprint(bufz2, sz);
 				break;
 			case 'e': // entropy
-				config.block[i] = (unsigned char)hash_entropy(buf, sz);
+				bufz[i] = (unsigned char)hash_entropy(bufz2, sz);
 				break;
 			//case 'h':
 			default:
-				config.block[i] = buf[0];
+				bufz[i] = bufz2[0];
 				break;
 			}
 			ptr += config.zoom.piece;
 		}
+		buf = bufz;
+		free(bufz2);
 	}
 
 	if (len <= 0) len = config.block_size;
@@ -837,7 +899,7 @@ void print_data(u64 seek, char *arg, u8 *buf, int olen, print_fmt_t fmt)
 		{
 			int i,j,pc,pce;
 			for(i=0;i<len;i++) {
-				if (zoom) print_addr(seek+(config.zoom.piece*i));
+				if (zoom) print_addr(config.zoom.from+(config.zoom.piece*i));
 				else print_addr(seek);
 				cons_printf(" %02x |", buf[i]);
 				pc = (buf[i]*100)/255;
@@ -853,14 +915,14 @@ void print_data(u64 seek, char *arg, u8 *buf, int olen, print_fmt_t fmt)
 		break;
 	case FMT_PERCENT: {
 			int w = config.width-4;
-			u64 s = config.size;
-			u64 piece = 0;
+			ut64 s = config.size;
+			ut64 piece = 0;
 			if (s==-1)
 				s = 0x100000000LL; // XXX WTF
 			piece = s/w;
 			cons_printf("[");
 			for(i=0;i<w;i++) {
-				u64 from = (piece*i);
+				ut64 from = (piece*i);
 				C { struct data_t *d = data_get_between(from, from+piece);
 //printf("%lld, %lld (piece =%lld\n", from , from+piece, piece);
 				if (d != NULL)
@@ -884,18 +946,17 @@ void print_data(u64 seek, char *arg, u8 *buf, int olen, print_fmt_t fmt)
 		break;
 	case FMT_PRINT:
 		i = last_print_format;
-		radare_cmd( config_get("cmd.print"), 0);
+		radare_cmd_raw (config_get ("cmd.print"), 0);
 		last_print_format = i;
 		break;
 	case FMT_REF:
 		{
 		char buf[128];
-		char *str = NULL;
 		buf[0]='\0';
-		sprintf(buf, "!!rsc dwarf-addrs '$FILE' 0x%llx", config.seek);
+		sprintf(buf, "!!rsc dwarf-addrs \"$FILE\" 0x%llx", config.seek);
 		str = pipe_command_to_string(buf);
 		if (str) {
-			cons_printf(str);
+			cons_strcat(str);
 			free(str);
 		}
 		} break;
@@ -911,11 +972,11 @@ void print_data(u64 seek, char *arg, u8 *buf, int olen, print_fmt_t fmt)
 #endif
 	case FMT_VISUAL:
 		i = last_print_format;
-		radare_cmd( config_get("cmd.visual"),0);
+		radare_cmd (config_get("cmd.visual"), 0);
 		last_print_format = i;
 		break;
 	case FMT_MEMORY:
-		print_mem((u64)addr, (const u8*)buf, (u64)len, (char *)arg, (int)endian);
+		print_mem((ut64)addr, (const u8*)buf, (ut64)len, (char *)arg, (int)endian);
 		break;
 	case FMT_DISAS:
 		radis(config.block_size, len);
@@ -953,6 +1014,21 @@ void print_data(u64 seek, char *arg, u8 *buf, int olen, print_fmt_t fmt)
 			endian_memcpy((u8*)&f, buf+i, sizeof(float));
 			cons_printf("%f\n", f);
 		} } break;
+	case FMT_DWORDDUMP:
+		{
+		/* TODO: control width of screen like everybody does */
+		for(i=j=0;!config.interrupted && i<len;i+=4,j++) {
+			ut32 n;
+			/* TODO: endian here */
+			memcpy(&n, buf+i, sizeof(ut32));
+			switch(j%6) {
+			case 0: cons_printf("0x%08llx:  0x%08x  ", config.seek+i, n); break;
+			case 5: cons_printf("\n"); break;
+			default: cons_printf("0x%08x  ", n);
+			}
+		}
+		cons_newline();
+		} break;
 	case FMT_INT:
 		{
 			int *iv;
@@ -965,7 +1041,6 @@ void print_data(u64 seek, char *arg, u8 *buf, int olen, print_fmt_t fmt)
 		} break;
 	case FMT_LONG:
 		{
-			int i;
 			long *l;
 			for(i=0;!config.interrupted && i<len;i+=sizeof(long)) {
 				endian_memcpy(buffer, config.block+i, sizeof(long));
@@ -1008,7 +1083,7 @@ void print_data(u64 seek, char *arg, u8 *buf, int olen, print_fmt_t fmt)
 	case FMT_USER: {
 		const char *ptr = config_get("cmd.user");
 		if (ptr && ptr[0])
-			radare_cmd(ptr, 0);
+			radare_cmd_raw (ptr, 0);
 		} break;
 	/*   DATES   */
 	case FMT_TIME_DOS: {
@@ -1020,48 +1095,49 @@ void print_data(u64 seek, char *arg, u8 *buf, int olen, print_fmt_t fmt)
 			endian_memcpy(_date, config.block+i+2, 2);
 			print_msdos_date(_time, _date);
 			// TODO: add tzdelta support
-			cons_newline();
 		} } break;
 	case FMT_TIME_UNIX: {
-		int delta = config_get_i("cfg.tzdelta");
-		time_t t;
-		char datestr[256];
-		const char *datefmt;
-		for(i=0;!config.interrupted && i<len;i+=sizeof(time_t)) {
-			endian_memcpy((unsigned char*)&t, config.block+i, sizeof(time_t));
-			//printf("%s", (char *)ctime((const time_t*)&t));
-			t += (delta*3600); // time zone delta
-			datefmt = config_get("cfg.datefmt");
-			if (datefmt&&datefmt[0])
-				tmp = strftime(datestr,256,datefmt,
-					(const struct tm*)gmtime((const time_t*)&t));
-			else 	tmp = strftime(datestr,256,"%d:%m:%Y %H:%M:%S %z",
-					(const struct tm*)gmtime((const time_t*)&t));
-			// TODO colorize depending on the distance between dates
-			if (tmp) cons_printf("%s",datestr); else printf("*failed*");
-			cons_newline();
-		} } break;
+	    struct tm * tmptr;
+	    int delta = config_get_i("cfg.tzdelta");
+	    time_t t;
+	    char datestr[256];
+	    const char *datefmt = config_get("cfg.datefmt");
+	    for(i=0;!config.interrupted && i<len;i+=sizeof(time_t)) {
+		    endian_memcpy((unsigned char*)&t, config.block+i, sizeof(time_t));
+		    //printf("%s", (char *)ctime((const time_t*)&t));
+		    t += (delta*3600); // time zone delta
+		    tmptr = gmtime((const time_t*)&t);
+		    if (tmptr == NULL)
+			    tmp = 0;
+		    else
+			    if (datefmt&&datefmt[0])
+				    tmp = strftime(datestr, 256, datefmt, tmptr);
+			    else    tmp = strftime(datestr, 256, "%d:%m:%Y %H:%M:%S %z", tmptr);
+		    // TODO colorize depending on the distance between dates
+		    if (tmp) cons_printf("%s\n", datestr); else cons_printf("(fail: 0x%16llx)\n", (ut64)t);
+	    } } break;
 	case FMT_TIME_FTIME: {
-		int delta = config_get_i("cfg.tzdelta");
-		unsigned long long l, L = 0x2b6109100LL;
-		time_t t;
-		char datestr[256];
-		const char *datefmt;
-		for(i=0;!config.interrupted && i<len;i+=8) {
-			endian_memcpy((unsigned char*)&l, config.block+i, sizeof(unsigned long long));
-			l /= 10000000; // 100ns to s
-			l = (l > L ? l-L : 0); // isValidUnixTime?
-			t = (time_t) l; // TODO limit above!
-			t += (delta*3600); // time zone delta
-			datefmt = config_get("cfg.datefmt");
-			if (datefmt&&datefmt[0])
-				tmp = strftime(datestr, 256, datefmt,
-					(const struct tm*)gmtime((const time_t*)&t));
-			else 	tmp = strftime(datestr, 256, "%d:%m:%Y %H:%M:%S %z",
-					(const struct tm*)gmtime((const time_t*)&t));
-			if (tmp) cons_printf("%s", datestr); else cons_printf("*failed*");
-			cons_newline();
-		} } break;
+	     struct tm * tmptr;
+	     int delta = config_get_i("cfg.tzdelta");
+	     unsigned long long l, L = 0x2b6109100LL;
+	     time_t t;
+	     char datestr[256];
+	     const char *datefmt = config_get("cfg.datefmt");
+	     for(i=0;!config.interrupted && i<len;i+=8) {
+		     endian_memcpy((unsigned char*)&l, config.block+i, sizeof(unsigned long long));
+		     l /= 10000000; // 100ns to s
+		     l = (l > L ? l-L : 0); // isValidUnixTime?
+		     t = (time_t) l; // TODO limit above!
+		     t += (delta*3600); // time zone delta
+		     tmptr = gmtime((const time_t*)&t);
+		     if (tmptr == NULL)
+			     tmp = 0;
+		     else
+			     if (datefmt&&datefmt[0])
+				     tmp = strftime(datestr, 256, datefmt, tmptr);
+			     else    tmp = strftime(datestr, 256, "%d:%m:%Y %H:%M:%S %z", tmptr);
+		     if (tmp) cons_printf("%s\n", datestr); else cons_printf("(fail: 0x%16llx)\n", (ut64)t);
+	     } } break;
 	case FMT_RAW:
 		// XXX TODO: measure the string length and make it fit properly
 		V i = config.width*config.height; else i=len;
@@ -1076,9 +1152,9 @@ void print_data(u64 seek, char *arg, u8 *buf, int olen, print_fmt_t fmt)
 		cons_newline();
 		break;
 	case FMT_ASHC:
-		str = flag_name_by_offset(seek);
-		if (!*str) str = "shellcode";
-                cons_printf("%s:", str);
+		cstr = flag_name_by_offset(seek);
+		if (!*cstr) cstr = "shellcode";
+                cons_printf("%s:", cstr);
 		inc = config.width/7;
 		if (inc<1)inc = 1;
 		lines = 0;
@@ -1094,17 +1170,17 @@ void print_data(u64 seek, char *arg, u8 *buf, int olen, print_fmt_t fmt)
                         if (((i+1)%inc) && i+1<len) cons_printf(", ");
                 }
 		cons_newline();
-                cons_printf(".equ %s_len, %d", str, len); cons_newline();
+                cons_printf(".equ %s_len, %d", cstr, len); cons_newline();
                 break;
 	case FMT_CSTR:
 		inc = config_get_i("scr.bytewidth");
 		if (!inc) inc = config.width/6;
 		if (inc<1)inc = 1;
-		cons_printf("#define _BUFFER_SIZE %d", len); cons_newline();
-		cons_printf("unsigned char buffer[_BUFFER_SIZE] = {"); cons_newline();
+		cons_printf("#define _BUFFER_SIZE %d\n", len);
+		cons_strcat("unsigned char buffer[_BUFFER_SIZE] = {\n");
 		for(j = i = 0; !config.interrupted && i < len;) {
 			print_color_byte_i(i, "0x%02x", config.block[i]);
-			if (++i<len)  cons_printf(", ");
+			if (++i<len) cons_strcat(", ");
 			if (!(i%inc)) {
 				cons_newline(); 
 				V if (++j+5>config.height)
@@ -1112,7 +1188,7 @@ void print_data(u64 seek, char *arg, u8 *buf, int olen, print_fmt_t fmt)
 						break;
 			}
 		}
-		cons_printf(" };\n");
+		cons_strcat(" };\n");
 		break;
 	case FMT_BIN:
 		inc = config_get_i("scr.bytewidth");
@@ -1142,8 +1218,10 @@ void print_data(u64 seek, char *arg, u8 *buf, int olen, print_fmt_t fmt)
 		break;
 	case FMT_OCT:
 		inc = config_get_i("scr.bytewidth");
-		if (!inc) inc = (int)((config.width)/6);
-		if (inc<1)inc = 1;
+		if (!inc) inc = (int)((config.width-10)/7);
+		if (inc<1) inc = 1;
+
+		inc++;
 		D {
 			C cons_strcat(cons_palette[PAL_HEADER]);
 			cons_printf("   offset   ");
@@ -1170,6 +1248,7 @@ void print_data(u64 seek, char *arg, u8 *buf, int olen, print_fmt_t fmt)
 						print_color_byte_i(i, "%c", buf[i]);
 					else	print_color_byte_i(i, ".", buf[i]);
 				}
+				i--;
 			}
 			cons_newline();
 		}
@@ -1186,7 +1265,9 @@ void print_data(u64 seek, char *arg, u8 *buf, int olen, print_fmt_t fmt)
 		cons_newline();
 		break;
 	case FMT_ASC0:
-		cons_printf("%s\n", buf);
+		for(i=0;buf[i]&&i<len;i++)
+			cons_printf("%c", buf[i]);
+		cons_newline();
 		break;
 	case FMT_ASC:
 		for(i=0;!config.interrupted && i<len;i++)
@@ -1223,89 +1304,11 @@ void print_data(u64 seek, char *arg, u8 *buf, int olen, print_fmt_t fmt)
 		}  D{}else cons_newline();
 		} break;
 	case FMT_ZOOM: {
-		u8 *buf = NULL;
-		u64 sz = 4;
-		const char *mode = config_get("zoom.byte");
-		u64 ptr = config_get_i("zoom.from");
-		//u64 to = config_get_i("zoom.to");
-#if 0
-		config.size = to-ptr;
-		if (config.size<0)
-			config.size = -config.size;
-#endif
-	
-		if (!mode)
-			break;
-		zoom = 1;
-		// XXX config.seek = ptr;
-
-		config.zoom.piece = config.size / config.block_size ;
-		fmt = FMT_HEXB;
-		//buf = (char *)malloc(config.zoom.piece+10);
-
-		switch(mode[0]) {
-		case 'e':
-			buf = (u8 *)malloc(config.zoom.piece);
-			sz = (u64)config.zoom.piece;
-			break;
-		default:
-			buf = (u8 *)malloc(len<<1);
-			break;
-		}
-		for(i=0;!config.interrupted && i<len;i++) {
-			io_lseek(config.fd, ptr, SEEK_SET);
-			buf[0]='\xff';
-			io_read(config.fd, buf, sz);
-
-			switch(mode[0]) {
-			case 'F': // 0xFF
-				config.block[i] = 0;
-				for(j=0;j<sz;j++)
-					if (buf[j]==0xff)
-						config.block[i]++;
-				break;
-			case 'c': // code
-				{
-				struct data_t *data = data_get_between(ptr, ptr+config.zoom.piece);
-				config.block[i] = (unsigned char) 0;
-				if (data->type == DATA_CODE)
-					config.block[i] = (unsigned char) data->times;
-				}
-				break;
-			case 's': // strings
-				{
-				struct data_t *data = data_get_between(ptr, ptr+config.zoom.piece);
-				config.block[i] = (unsigned char) 0;
-				if (data->type == DATA_STR)
-					config.block[i] = (unsigned char) data->times;
-				}
-				break;
-			case 't': // traces
-				config.block[i] = (unsigned char)trace_get_between(ptr, ptr+config.zoom.piece);
-				break;
-			case 'f': // flags
-				config.block[i] = (unsigned char)flags_between(ptr, ptr+config.zoom.piece);
-				break;
-			case 'p': // % printable chars
-				config.block[i] = (unsigned char)2.55*hash_pcprint(buf, sz);
-				break;
-			case 'e': // entropy
-				config.block[i] = (unsigned char)hash_entropy(buf, sz);
-				break;
-			//case 'h':
-			default:
-				config.block[i] = buf[0];
-				break;
-			}
-			ptr += config.zoom.piece;
-		}
-		free(buf);
-		}
-	case FMT_HEXPAIRS:
-		for(i=0;i<len;i++)
-			cons_printf("%02x", buf[i]);
-		cons_newline();
+		if (!zoom)
+			config_set("zoom.enable", "true");
+		print_data(seek, arg, buf,len, FMT_HEXB);
 		break;
+		}
 	case FMT_HEXBS:
 	case FMT_HEXB:
 		inc = config_get_i("scr.bytewidth");
@@ -1331,7 +1334,7 @@ void print_data(u64 seek, char *arg, u8 *buf, int olen, print_fmt_t fmt)
 		for(i=0; !config.interrupted && i<len; i+=inc) {
 			V if (inc==0 && (i/inc)+4>config.height) break;
 			D { if ( fmt == FMT_HEXB ) {
-				if (zoom) print_addr(seek+(config.zoom.piece*i));
+				if (zoom) print_addr(config.zoom.from+(config.zoom.piece*i));
 				else print_addr(seek+i+config.vaddr);
 			} }
 
@@ -1372,6 +1375,11 @@ void print_data(u64 seek, char *arg, u8 *buf, int olen, print_fmt_t fmt)
 		if (fmt == FMT_HEXBS)
 			cons_newline();
 		break;
+	case FMT_HEXPAIRS:
+		for(i=0;i<len;i++)
+			cons_printf("%02x", buf[i]);
+		cons_newline();
+		break;
 	case FMT_BASE64:
 		{
 		/* decode base64 */
@@ -1407,7 +1415,7 @@ void print_data(u64 seek, char *arg, u8 *buf, int olen, print_fmt_t fmt)
 	if (inverse)
 		free(bufi);
 	if (zoom)
-		free(buf);
+		free(bufz);
 	radare_controlc_end();
 }
 
