@@ -59,13 +59,13 @@ static void ELF_(aux_swap_endian)(u8 *value, int size)
 	}
 }
 
-static int ELF_(aux_stripstr_from_file)(const char *filename, int min, int encoding, u64 seek, u64 limit, const char *filter, int str_limit, dietelf_string *strings)
+static int ELF_(aux_stripstr_from_file)(const char *filename, int min, int encoding, ut64 seek, ut64 limit, const char *filter, const char *section, int str_limit, dietelf_string *strings)
 {
 	int fd = open(filename, O_RDONLY);
 	dietelf_string *stringsp;
 	unsigned char *buf;
-	u64 i = seek;
-	u64 len, string_len;
+	ut64 i = seek;
+	ut64 len, string_len;
 	int unicode = 0, matches = 0;
 	static int ctr = 0;
 	char str[ELF_STRING_LENGTH];
@@ -77,14 +77,14 @@ static int ELF_(aux_stripstr_from_file)(const char *filename, int min, int encod
 
 	len = lseek(fd, 0, SEEK_END);
 
-	// TODO: use read here ?!?
-	/* TODO: do not use mmap */
-#if __UNIX__
-	buf = mmap(NULL, len, PROT_READ, MAP_SHARED, fd, 0);
-	if (((int)buf) == -1 ) {
-		perror("mmap");
+	buf = malloc(len);
+	if (buf == NULL) {
+		perror("malloc");
 		return 1;
 	}
+	lseek(fd, 0, SEEK_SET);
+	read(fd, buf, len);
+
 	if (min <1)
 		min = 5;
 
@@ -108,6 +108,7 @@ static int ELF_(aux_stripstr_from_file)(const char *filename, int min, int encod
 				string_len = strlen(str);
 				if (string_len>2) {
 					if (!filter || strstr(str, filter)) {
+						strncpy(stringsp->section, section, ELF_STRING_LENGTH);
 						stringsp->offset = i-matches;
 						stringsp->type = (unicode?'U':'A');
 						stringsp->size = string_len;
@@ -122,10 +123,8 @@ static int ELF_(aux_stripstr_from_file)(const char *filename, int min, int encod
 		}
 	}
 
-	munmap(buf, len); 
-#elif __WINDOWS__
-	fprintf(stderr, "Not yet implemented\n");
-#endif
+	free(buf);
+
 	return ctr;
 }
 
@@ -154,7 +153,7 @@ static int ELF_(do_elf_checks)(ELF_(dietelf_bin_t) *bin)
 
 char* ELF_(dietelf_get_arch)(ELF_(dietelf_bin_t) *bin)
 {
-	u16 machine = bin->ehdr.e_machine;
+	ut16 machine = bin->ehdr.e_machine;
 
 
 	switch (machine) {
@@ -180,12 +179,12 @@ char* ELF_(dietelf_get_arch)(ELF_(dietelf_bin_t) *bin)
 	}
 }
 
-u64 ELF_(dietelf_get_base_addr)(ELF_(dietelf_bin_t) *bin)
+ut64 ELF_(dietelf_get_base_addr)(ELF_(dietelf_bin_t) *bin)
 {
 	return bin->phdr->p_vaddr & ELF_ADDR_MASK;
 }
 
-u64 ELF_(dietelf_get_entry_offset)(ELF_(dietelf_bin_t) *bin)
+ut64 ELF_(dietelf_get_entry_offset)(ELF_(dietelf_bin_t) *bin)
 {
 	return bin->ehdr.e_entry - bin->base_addr; 
 }
@@ -351,8 +350,8 @@ char* ELF_(dietelf_get_elf_class)(ELF_(dietelf_bin_t) *bin)
 		case ELFCLASS32:   return "ELF32";
 		case ELFCLASS64:   return "ELF64";
 		default:
-						   snprintf (buff, sizeof (buff), "<unknown: %x>", elf_class);
-						   return buff;
+			snprintf (buff, sizeof (buff), "<unknown: %x>", elf_class);
+			return buff;
 	}
 }
 
@@ -370,7 +369,7 @@ char* ELF_(dietelf_get_osabi_name)(ELF_(dietelf_bin_t) *bin)
 		case ELFOSABI_AIX:			return "aix";
 		case ELFOSABI_IRIX:			return "irix";
 		case ELFOSABI_FREEBSD:		return "freebsd";
-		case ELFOSABI_TRU64:		return "tru64";
+		case ELFOSABI_TRU64:		return "trut64";
 		case ELFOSABI_MODESTO:		return "modesto";
 		case ELFOSABI_OPENBSD:		return "openbsd";
 		case ELFOSABI_STANDALONE:	return "standalone";
@@ -381,7 +380,7 @@ char* ELF_(dietelf_get_osabi_name)(ELF_(dietelf_bin_t) *bin)
 	}
 }
 
-u64
+ut64
 ELF_(dietelf_get_section_index)(ELF_(dietelf_bin_t) *bin, int fd, const char *section_name)
 {
 	ELF_(Ehdr) *ehdr = &bin->ehdr;
@@ -398,7 +397,7 @@ ELF_(dietelf_get_section_index)(ELF_(dietelf_bin_t) *bin, int fd, const char *se
 	return -1;
 }
 
-u64 ELF_(dietelf_get_section_offset)(ELF_(dietelf_bin_t) *bin, int fd, const char *section_name)
+ut64 ELF_(dietelf_get_section_offset)(ELF_(dietelf_bin_t) *bin, int fd, const char *section_name)
 {
 	ELF_(Ehdr) *ehdr = &bin->ehdr;
 	ELF_(Shdr) *shdr = bin->shdr, *shdrp;
@@ -437,18 +436,18 @@ int ELF_(dietelf_is_big_endian)(ELF_(dietelf_bin_t) *bin)
 	return (ehdr->e_ident[EI_DATA] == ELFDATA2MSB);
 }
 
-static u64 ELF_(get_import_addr)(ELF_(dietelf_bin_t) *bin, int fd, int sym)
+static ut64 ELF_(get_import_addr)(ELF_(dietelf_bin_t) *bin, int fd, int sym)
 {
 	ELF_(Ehdr) *ehdr = &bin->ehdr;
 	ELF_(Shdr) *shdr = bin->shdr, *shdrp;
 	ELF_(Addr) plt_sym_addr, got_addr = 0;
 	const char *string = bin->string;
 	int i, j;
-	u64 got_offset;
+	ut64 got_offset;
 	
 	shdrp = shdr;
 	for (i = 0; i < ehdr->e_shnum; i++, shdrp++) {
-		if (!strcmp(&string[shdrp->sh_name], ".got.plt"))
+		if (!strcmp(&string[shdrp->sh_name], ".got"))
 			got_addr = shdrp->sh_offset;
 	}
 	if (got_addr == 0) {
@@ -729,7 +728,7 @@ int ELF_(dietelf_get_symbols)(ELF_(dietelf_bin_t) *bin, int fd, dietelf_symbol *
 	ELF_(Sym) *sym, *symp;
 	ELF_(Shdr) *strtabhdr;
 	dietelf_symbol *symbolp;
-	u64 sym_offset;
+	ut64 sym_offset;
 	char *string;
 	int i, j, k;
 
@@ -790,10 +789,10 @@ int ELF_(dietelf_get_symbols)(ELF_(dietelf_bin_t) *bin, int fd, dietelf_symbol *
 				if (k == 0)
 					continue;
 				if (symp->st_shndx != STN_UNDEF && ELF_ST_TYPE(symp->st_info) != STT_SECTION && ELF_ST_TYPE(symp->st_info) != STT_FILE) {
-					symbolp->size = (u64)symp->st_size; 
+					symbolp->size = (ut64)symp->st_size; 
 					memcpy(symbolp->name, &string[symp->st_name], ELF_NAME_LENGTH); 
 					symbolp->name[ELF_NAME_LENGTH-1] = '\0';
-					symbolp->offset = (u64)symp->st_value + sym_offset;
+					symbolp->offset = (ut64)symp->st_value + sym_offset;
 					if (symbolp->offset >= bin->base_addr)
 						symbolp->offset -= bin->base_addr;
 					switch (ELF_ST_BIND(symp->st_info)) {
@@ -839,16 +838,25 @@ int ELF_(dietelf_get_fields)(ELF_(dietelf_bin_t) *bin, dietelf_field *field)
 	int i = 0, j = 0;
 
 	strncpy(field[i].name, "ehdr", ELF_NAME_LENGTH); 
-	field[i++].offset = 0;
+	field[i].offset = 0;
+	field[i++].vaddr = 0;
 	strncpy(field[i].name, "shoff", ELF_NAME_LENGTH); 
-	field[i++].offset = ehdr->e_shoff;
+	field[i].offset = ehdr->e_shoff;
+	field[i++].vaddr = ehdr->e_shoff + bin->base_addr;
 	strncpy(field[i].name, "phoff", ELF_NAME_LENGTH); 
-	field[i++].offset = ehdr->e_phoff;
+	field[i].offset = ehdr->e_phoff;
+	field[i++].vaddr = ehdr->e_phoff + bin->base_addr;
 
 	for (j = 0; j < ehdr->e_phnum; i++, j++) {
 		snprintf(string, ELF_NAME_LENGTH, "phdr_%i", j);
 		strncpy(field[i].name, string, ELF_NAME_LENGTH); 
 		field[i].offset = phdr[i].p_offset;
+		field[i].vaddr = phdr[i].p_vaddr;
+		field[i].type = phdr[i].p_type;
+		field[i].flags = phdr[i].p_flags;
+		field[i].end = phdr[i].p_memsz+(0x1000-(phdr[i].p_memsz%0x1000)); /* filesize or memsize here ?? */
+		field[i].size = phdr[i].p_memsz; /* filesize or memsize here ?? */
+//printf("TYPE=%d\n", phdr[i].p_flags);
 	}
 
 	return 0;
@@ -915,10 +923,11 @@ int ELF_(dietelf_get_strings)(ELF_(dietelf_bin_t) *bin, int fd, int verbose, int
 
 	shdrp = shdr;
 	for (i = 0; i < ehdr->e_shnum; i++, shdrp++) {
-		if (verbose < 2 && i != 0 && !strcmp(&string[shdrp->sh_name], ".rodata"))
-			ctr = ELF_(aux_stripstr_from_file)(bin->file, 3, ENCODING_ASCII, shdrp->sh_offset, shdrp->sh_offset+shdrp->sh_size, NULL, str_limit-ctr, strings+ctr);
+		if (verbose < 2 && i != 0 && !strcmp(&string[shdrp->sh_name], ".rodata")) {
+			ctr = ELF_(aux_stripstr_from_file)(bin->file, 3, ENCODING_ASCII, shdrp->sh_offset, shdrp->sh_offset+shdrp->sh_size, NULL, &string[shdrp->sh_name], str_limit-ctr, strings+ctr);
+		}
 		if (verbose == 2 && i != 0 && !(shdrp->sh_flags & SHF_EXECINSTR)) {
-			ctr = ELF_(aux_stripstr_from_file)(bin->file, 3, ENCODING_ASCII, shdrp->sh_offset, shdrp->sh_offset+shdrp->sh_size, NULL, str_limit-ctr, strings+ctr);
+			ctr = ELF_(aux_stripstr_from_file)(bin->file, 3, ENCODING_ASCII, shdrp->sh_offset, shdrp->sh_offset+shdrp->sh_size, NULL, &string[shdrp->sh_name], str_limit-ctr, strings+ctr);
 		}
 	}
 
@@ -936,7 +945,7 @@ int ELF_(dietelf_get_libs)(ELF_(dietelf_bin_t) *bin, int fd, int str_limit, diet
 	for (i = 0; i < ehdr->e_shnum; i++, shdrp++) {
 		if (!strcmp(&string[shdrp->sh_name], ".dynstr")) {
 			ctr = ELF_(aux_stripstr_from_file)(bin->file, 3, ENCODING_ASCII, shdrp->sh_offset,
-				shdrp->sh_offset+shdrp->sh_size, ".so.", str_limit, strings+ctr);
+				shdrp->sh_offset+shdrp->sh_size, ".so.", &string[shdrp->sh_name], str_limit, strings+ctr);
 		}
 	}
 
@@ -989,15 +998,13 @@ static int ELF_(dietelf_init)(ELF_(dietelf_bin_t) *bin, int fd)
 
 	if (lseek(fd, ehdr->e_phoff, SEEK_SET) < 0) {
 		perror("lseek");
-		fprintf(stderr, "Warning: Cannot read program headers (0x%08x->0x%08x)\n",
-			ehdr->e_phoff, (long)&ehdr->e_phoff-(long)&ehdr->e_ident);
+		fprintf(stderr, "Warning: Cannot read program headers");
 		//return -1;
 	}
 
 	if (read(fd, bin->phdr, bin->plen) != bin->plen) {
 		perror("read");
-		fprintf(stderr, "Warning: Cannot read program headers (0x%08x->0x%08x)\n",
-			ehdr->e_phoff, (long)&ehdr->e_phoff-(long)&ehdr->e_ident);
+		fprintf(stderr, "Warning: Cannot read program headers");
 		//return -1;
 	}
 
@@ -1028,14 +1035,14 @@ static int ELF_(dietelf_init)(ELF_(dietelf_bin_t) *bin, int fd)
 		perror("lseek");
 		ehdr->e_shnum=0;
 		fprintf(stderr, "Warning: Cannot read %d section headers (0x%08x->0x%08x)\n",
-			ehdr->e_shnum, ehdr->e_shoff, (long)&ehdr->e_shoff-(long)&ehdr->e_ident);
+			ehdr->e_shnum, ehdr->e_shoff, (unsigned int)(size_t)&ehdr->e_shoff-(unsigned int)(size_t)&ehdr->e_ident);
 	//	return -1;
 	}
 
 	if (read(fd, bin->shdr, slen) != slen) {
 		perror("read");
 		fprintf(stderr, "Warning: Cannot read %d section headers (0x%08x->0x%08x)\n",
-			ehdr->e_shnum, ehdr->e_shoff, (long)&ehdr->e_shoff-(long)&ehdr->e_ident);
+			ehdr->e_shnum, ehdr->e_shoff, (unsigned int)(size_t)&ehdr->e_shoff-(unsigned int)(size_t)&ehdr->e_ident);
 		ehdr->e_shnum=0;
 		//return -1;
 	}
