@@ -9,9 +9,9 @@
 #include "elf.h"
 
 
-static int   ELF_(aux_stripstr_from_file)(const char*, int, int, u64, u64, const char*, int, dietelf_string*);
+static int ELF_(aux_stripstr_from_file)(const char *filename, int min, int encoding, ut64 seek, ut64 limit, const char *filter, const char *section, int str_limit, dietelf_string *strings);
 static void  ELF_(aux_swap_endian)(u8*, int);
 static int   ELF_(dietelf_init)(ELF_(dietelf_bin_t)*, int);
 static int   ELF_(do_elf_checks)(ELF_(dietelf_bin_t)*);
-static u64   ELF_(get_import_addr)(ELF_(dietelf_bin_t)*, int, int);
+static ut64   ELF_(get_import_addr)(ELF_(dietelf_bin_t)*, int, int);
 static int   ELF_(load_section)(char**, int, ELF_(Shdr)*);
