@@ -52,6 +52,7 @@ char *filename, output[MAX_F_LENGTH], *field, *out;
 
 char *fileaddr, *startaddr;
 
+// XXX This must be ut64 !!!!
 static void *offset, *toff, *init_offset;
 static unsigned int ncmds;
 
@@ -232,6 +233,8 @@ void dm_read_thread (int i)
 		}
 		printf("f entrypoint @ 0x%08x\n", n0(state[15]));
 		printf("e asm.arch=ppc\n");
+		printf("e cfg.bigendian=true\n");
+		printf("e io.paddr=0x1000\n");
 		}
 		break;
 	case 12: // ARM
@@ -934,8 +937,8 @@ p = offset;
 			break;
 		}
 		{
-			//u32 csz = n0(seg_command->cmdsize);
-			u32 csz = n0(seg_command->cmdsize);
+			//ut32 csz = n0(seg_command->cmdsize);
+			ut32 csz = n0(seg_command->cmdsize);
 			if (csz == 0) /* avoid infinite loops or so? XXX can be tricked */
 				break;
 			offset += csz; //seg_command->cmdsize;
