@@ -23,13 +23,13 @@
 #include "dietpe_static.h"
 #include "dietpe_types.h"
 
-static int PE_(dietpe_aux_stripstr_from_file)(PE_(dietpe_obj) *bin, int min, int encoding, u64 seek, u64 limit, const char *filter, int str_limit, dietpe_string *strings)
+static int PE_(dietpe_aux_stripstr_from_file)(PE_(dietpe_obj) *bin, int min, int encoding, ut64 seek, ut64 limit, const char *filter, const char *section, int str_limit, dietpe_string *strings)
 {
 	int fd = open(bin->file, O_RDONLY);
 	dietpe_string *stringsp;
 	unsigned char *buf;
-	u64 i = seek;
-	u64 len, string_len;
+	ut64 i = seek;
+	ut64 len, string_len;
 	int unicode = 0, matches = 0;
 	static int ctr = 0;
 	char str[PE_STRING_LENGTH];
@@ -41,14 +41,14 @@ static int PE_(dietpe_aux_stripstr_from_file)(PE_(dietpe_obj) *bin, int min, int
 
 	len = lseek(fd, 0, SEEK_END);
 
-	// TODO: use read here ?!?
-	/* TODO: do not use mmap */
-#if __UNIX__
-	buf = mmap(NULL, len, PROT_READ, MAP_SHARED, fd, 0);
-	if (((int)buf) == -1 ) {
-		perror("mmap");
+	buf = malloc(len);
+	if (buf == NULL) {
+		perror("malloc");
 		return 1;
 	}
+	lseek(fd, 0, SEEK_SET);
+	read(fd, buf, len);
+
 	if (min <1)
 		min = 5;
 
@@ -72,6 +72,7 @@ static int PE_(dietpe_aux_stripstr_from_file)(PE_(dietpe_obj) *bin, int min, int
 				string_len = strlen(str);
 				if (string_len>2) {
 					if (!filter || strstr(str, filter)) {
+						strncpy(stringsp->section, section, PE_STRING_LENGTH);
 						stringsp->offset = i-matches;
 						stringsp->rva = PE_(dietpe_aux_offset_to_rva)(bin, i-matches);
 						stringsp->type = (unicode?'U':'A');
@@ -86,11 +87,8 @@ static int PE_(dietpe_aux_stripstr_from_file)(PE_(dietpe_obj) *bin, int min, int
 			unicode = 0;
 		}
 	}
+	free(buf);
 
-	munmap(buf, len); 
-#elif __WINDOWS__
-	fprintf(stderr, "Not yet implemented\n");
-#endif
 	return ctr;
 }
 
@@ -128,8 +126,25 @@ static PE_DWord PE_(dietpe_aux_offset_to_rva)(PE_(dietpe_obj) *bin, PE_DWord off
 	return 0;
 }
 
-static int PE_(dietpe_do_checks)(PE_(dietpe_obj) *bin)
+static int PE_(dietpe_init)(PE_(dietpe_obj) *bin)
 {
+	int sections_size, len;
+
+	len = lseek(bin->fd, 0, SEEK_END);
+
+	lseek(bin->fd, 0, SEEK_SET);
+	bin->dos_header = malloc(sizeof(PE_(image_dos_header)));
+	read(bin->fd, bin->dos_header, sizeof(PE_(image_dos_header)));
+
+	if (bin->dos_header->e_lfanew > len) {
+		fprintf(stderr, "Invalid e_lfanew field\n");
+		return -1;
+	}
+
+	lseek(bin->fd, bin->dos_header->e_lfanew, SEEK_SET);
+	bin->nt_headers = malloc(sizeof(PE_(image_nt_headers)));
+	read(bin->fd, bin->nt_headers, sizeof(PE_(image_nt_headers)));
+
 	if (strncmp((char*)&bin->dos_header->e_magic, "MZ", 2)) {
 		fprintf(stderr, "File not PE\n");
 		return -1;
@@ -140,25 +155,13 @@ static int PE_(dietpe_do_checks)(PE_(dietpe_obj) *bin)
 		return -1;
 	}
 
-	return 0;
-}
-
-static int PE_(dietpe_init)(PE_(dietpe_obj) *bin)
-{
-	int sections_size;
-
-	lseek(bin->fd, 0, SEEK_SET);
-	bin->dos_header = malloc(sizeof(PE_(image_dos_header)));
-	read(bin->fd, bin->dos_header, sizeof(PE_(image_dos_header)));
-
-	lseek(bin->fd, bin->dos_header->e_lfanew, SEEK_SET);
-	bin->nt_headers = malloc(sizeof(PE_(image_nt_headers)));
-	read(bin->fd, bin->nt_headers, sizeof(PE_(image_nt_headers)));
-
-	if (PE_(dietpe_do_checks)(bin) == -1)
-		return -1;
-
 	sections_size = sizeof(PE_(image_section_header)) * bin->nt_headers->file_header.NumberOfSections;
+
+	if (sections_size > len) {
+		fprintf(stderr, "Invalid NumberOfSections value\n");
+		return -1;
+	}
+
 	lseek(bin->fd, bin->dos_header->e_lfanew + sizeof(PE_(image_nt_headers)), SEEK_SET);
 	bin->section_header = malloc(sections_size);
 	read(bin->fd, bin->section_header, sections_size);
@@ -230,21 +233,9 @@ static int PE_(dietpe_parse_imports)(PE_(dietpe_obj) *bin, dietpe_import **impor
 	PE_DWord import_table = 0, off = 0;
 	int i = 0;
 
-	lseek(bin->fd, PE_(dietpe_aux_rva_to_offset)(bin, OriginalFirstThunk), SEEK_SET);
-	read(bin->fd, &import_table, sizeof(PE_DWord));
-	lseek(bin->fd, PE_(dietpe_aux_rva_to_offset)(bin, import_table) + sizeof(PE_Word), SEEK_SET);
-	read(bin->fd, name, PE_NAME_LENGTH);
-	if (aux_is_printable(name[0]))
-		off = PE_(dietpe_aux_rva_to_offset)(bin, OriginalFirstThunk);
-	else {
-		lseek(bin->fd, PE_(dietpe_aux_rva_to_offset)(bin, FirstThunk), SEEK_SET);
-		read(bin->fd, &import_table, sizeof(PE_DWord));
-		lseek(bin->fd, PE_(dietpe_aux_rva_to_offset)(bin, import_table) + sizeof(PE_Word), SEEK_SET);
-		read(bin->fd, name, PE_NAME_LENGTH);
-		if (aux_is_printable(name[0]))
-			off = PE_(dietpe_aux_rva_to_offset)(bin, FirstThunk);
-		else return 0;
-	}
+	if ((off = PE_(dietpe_aux_rva_to_offset)(bin, OriginalFirstThunk)) == 0 &&
+		(off = PE_(dietpe_aux_rva_to_offset)(bin, FirstThunk)) == 0)
+		return 0;
 
 	do {
 		lseek(bin->fd, off + i * sizeof(PE_DWord), SEEK_SET);
@@ -273,7 +264,7 @@ static int PE_(dietpe_parse_imports)(PE_(dietpe_obj) *bin, dietpe_import **impor
 		}
 	} while (import_table);
 
-	return 0;
+	return i;
 }
 
 int PE_(dietpe_close)(PE_(dietpe_obj) *bin)
@@ -410,9 +401,9 @@ int PE_(dietpe_get_file_alignment)(PE_(dietpe_obj) *bin)
 	return bin->nt_headers->optional_header.FileAlignment;
 }
 
-u64 PE_(dietpe_get_image_base)(PE_(dietpe_obj) *bin)
+ut64 PE_(dietpe_get_image_base)(PE_(dietpe_obj) *bin)
 {
-	return(u64)bin->nt_headers->optional_header.ImageBase;
+	return(ut64)bin->nt_headers->optional_header.ImageBase;
 }
 
 int PE_(dietpe_get_imports)(PE_(dietpe_obj) *bin, dietpe_import *import)
@@ -434,16 +425,18 @@ int PE_(dietpe_get_imports)(PE_(dietpe_obj) *bin, dietpe_import *import)
 	for (i = 0; i < import_dirs_count; i++, import_dirp++) {
 		lseek(bin->fd, PE_(dietpe_aux_rva_to_offset)(bin, import_dirp->Name), SEEK_SET);
 		read(bin->fd, dll_name, PE_NAME_LENGTH);
-		PE_(dietpe_parse_imports)(bin, &importp, dll_name,
-			import_dirp->Characteristics, import_dirp->FirstThunk);
+		if (!PE_(dietpe_parse_imports)(bin, &importp, dll_name,
+			import_dirp->Characteristics, import_dirp->FirstThunk))
+			break;
 	}
 
 	delay_import_dirp = bin->delay_import_directory;
 	for (i = 0; i < delay_import_dirs_count; i++, delay_import_dirp++) {
 		lseek(bin->fd, PE_(dietpe_aux_rva_to_offset)(bin, delay_import_dirp->Name), SEEK_SET);
 		read(bin->fd, dll_name, PE_NAME_LENGTH);
-		PE_(dietpe_parse_imports)(bin, &importp, dll_name, delay_import_dirp->DelayImportNameTable,
-			delay_import_dirp->DelayImportAddressTable);
+		if (!PE_(dietpe_parse_imports)(bin, &importp, dll_name, delay_import_dirp->DelayImportNameTable,
+			delay_import_dirp->DelayImportAddressTable))
+			break;
 	}
 
 	return 0;
@@ -454,7 +447,6 @@ int PE_(dietpe_get_imports_count)(PE_(dietpe_obj) *bin)
 	PE_(image_import_directory) *import_dirp;
 	PE_(image_delay_import_directory) *delay_import_dirp;
 	PE_DWord import_table, off = 0;
-	char name[PE_NAME_LENGTH];
 	int import_dirs_count = PE_(dietpe_get_import_dirs_count)(bin);
 	int delay_import_dirs_count = PE_(dietpe_get_delay_import_dirs_count)(bin);
 	int imports_count = 0, i, j;
@@ -465,26 +457,13 @@ int PE_(dietpe_get_imports_count)(PE_(dietpe_obj) *bin)
 	import_dirp = bin->import_directory;
 	import_table = 0;
 	for (i = 0; i < import_dirs_count; i++, import_dirp++) {
-		lseek(bin->fd, PE_(dietpe_aux_rva_to_offset)(bin, import_dirp->Characteristics), SEEK_SET);
-		read(bin->fd, &import_table, sizeof(PE_DWord));
-		lseek(bin->fd, PE_(dietpe_aux_rva_to_offset)(bin, import_table) + sizeof(PE_Word), SEEK_SET);
-		read(bin->fd, name, PE_NAME_LENGTH);
-		if (aux_is_printable(name[0]))
-			off = PE_(dietpe_aux_rva_to_offset)(bin, import_dirp->Characteristics);
-		else {
-			lseek(bin->fd, PE_(dietpe_aux_rva_to_offset)(bin, import_dirp->FirstThunk), SEEK_SET);
-			read(bin->fd, &import_table, sizeof(PE_DWord));
-			lseek(bin->fd, PE_(dietpe_aux_rva_to_offset)(bin, import_table)+ sizeof(PE_Word), SEEK_SET);
-			read(bin->fd, name, PE_NAME_LENGTH);
-			if (aux_is_printable(name[0]))
-				off = PE_(dietpe_aux_rva_to_offset)(bin, import_dirp->FirstThunk);
-			else return 0;
-		}
-
+		if ((off = PE_(dietpe_aux_rva_to_offset)(bin, import_dirp->Characteristics)) == 0 &&
+			(off = PE_(dietpe_aux_rva_to_offset)(bin, import_dirp->FirstThunk)) == 0)
+			break;
 		j = 0;
 		do {
 			lseek(bin->fd, off + j * sizeof(PE_DWord), SEEK_SET);
-    			read(bin->fd, &import_table, sizeof(PE_DWord));
+    		read(bin->fd, &import_table, sizeof(PE_DWord));
 			
 			if (import_table) {
 				imports_count++;
@@ -496,10 +475,13 @@ int PE_(dietpe_get_imports_count)(PE_(dietpe_obj) *bin)
 	delay_import_dirp = bin->delay_import_directory;
 	import_table = 0;
 	for (i = 0; i < delay_import_dirs_count; i++, delay_import_dirp++) {
+		if ((off = PE_(dietpe_aux_rva_to_offset)(bin, delay_import_dirp->DelayImportNameTable)) == 0 &&
+			(off = PE_(dietpe_aux_rva_to_offset)(bin, delay_import_dirp->DelayImportAddressTable)) == 0)
+			break;
 		j = 0;
 		do {
-			lseek(bin->fd, PE_(dietpe_aux_rva_to_offset)(bin, delay_import_dirp->DelayImportNameTable) + j * sizeof(PE_DWord), SEEK_SET);
-    			read(bin->fd, &import_table, sizeof(PE_DWord));
+			lseek(bin->fd, off + j * sizeof(PE_DWord), SEEK_SET);
+			read(bin->fd, &import_table, sizeof(PE_DWord));
 			
 			if (import_table) {
 				imports_count++;
@@ -736,7 +718,7 @@ int PE_(dietpe_get_strings)(PE_(dietpe_obj) *bin, int verbose, int str_limit, di
 	for (i = 0; i < sections_count; i++, shdrp++) {
 		ctr = PE_(dietpe_aux_stripstr_from_file)(
 				bin, 5, ENCODING_ASCII, shdrp->PointerToRawData,
-				shdrp->PointerToRawData+shdrp->SizeOfRawData, NULL,
+				shdrp->PointerToRawData+shdrp->SizeOfRawData, NULL, shdrp->Name,
 				str_limit, strings+ctr);
 	}
 
