@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2007, 2008
+ * Copyright (C) 2007, 2008, 2009
  *       pancake <youterm.com>
  *
  * radare is free software; you can redistribute it and/or modify
@@ -31,10 +31,10 @@
 #include <stdarg.h>
 
 #if __WINDOWS__
-u16 NTOHS(u16 x) {
-	u16 y;
-	u8 *buffer = x;
-	u8 *dest = y;
+ut16 NTOHS(ut16 x) {
+	ut16 y;
+	u8 *buffer = (u8*)&x;
+	u8 *dest = (u8*)&y;
 	dest[0] = buffer[1];
 	dest[1] = buffer[0];
 	return y;
@@ -52,7 +52,7 @@ int radare_output = 0;
 #define R if (radare_output)
 #define NR if (!radare_output)
 
-#define USHORT(x,y) (unsigned short)(x[y+1]|(x[y]<<8))
+#define USHORT(x,y) (unsigned short)((x[y+1]|(x[y]<<8)) & 0xffff)
 #define UINT(x,y) (unsigned int) ((x[y]<<24)|(x[y+1]<<16)|(x[y+2]<<8)|x[y+3])
 
 struct classfile {
@@ -92,7 +92,7 @@ struct cp_item {
 	char name[255];
 	char *value;
 	unsigned char bytes[5];
-	u64 off;
+	ut64 off;
 };
 
 struct cp_item *cp_items;
@@ -338,12 +338,11 @@ static int java_resolve(int idx, char *str)
 	} else
 	if (!strcmp(cp_items[idx].name, "Utf8")) {
 		sprintf(str, "\"%s\"", get_cp(idx)->value);
-	} else
-		sprintf(str, "0x%04x", USHORT(get_cp(idx)->bytes,0));
+	} else sprintf(str, "0x%04x", USHORT(get_cp(idx)->bytes,0));
 	return 0;
 }
 
-int java_print_opcode(int idx, unsigned char *bytes, char *output)
+int java_print_opcode(int idx, const unsigned char *bytes, char *output)
 {
 	char arg[1024];
 
@@ -464,9 +463,9 @@ static int attributes_walk(FILE *fd, int sz2, int fields)
 					printf("      Max Stack: %d\n", USHORT(buf, 0));
 					printf("      Max Locals: %d\n", USHORT(buf, 2));
 					printf("      Code Length: %d\n", UINT(buf, 4));
-					printf("      Code At Offset: 0x%08llx\n", (u64)ftell(fd));
+					printf("      Code At Offset: 0x%08llx\n", (ut64)ftell(fd));
 				} else {
-					printf("@ 0x%08llx\n", (u64)ftell(fd));
+					printf("@ 0x%08llx\n", (ut64)ftell(fd));
 				}
 				fread(buf, UINT(buf, 4), 1, fd); // READ CODE
 				sz4 = read_short(fd);
@@ -525,7 +524,7 @@ int java_classdump(const char *file)
 	struct classfile2 cf2;
 	unsigned short sz, sz2;
 	int this_class;
-	char buf[0x9999];
+	unsigned char buf[0xffff+1];
 	int i,j;
 	FILE *fd = fopen(file, "rb");
 
@@ -553,11 +552,16 @@ int java_classdump(const char *file)
 	
 	cf.cp_count--;
 	NR printf("ConstantPoolCount %d\n", cf.cp_count);
+	else printf("fs header\n");
 	cp_items = malloc(sizeof(struct cp_item)*(cf.cp_count+1));
 	for(i=0;i<cf.cp_count;i++) {
 		struct constant_t *c;
 
-		fread(buf, 1, 1, fd);
+		//printf("off 0x%08llx : ", (ut64)ftell(fd));
+		if (fread(buf, 1, 1, fd) != 1) {
+			fprintf (stderr, "Unexpected eof\n");
+			break;
+		}
 
 		c = NULL;
 		for(j=0;constants[j].name;j++) {
@@ -567,7 +571,8 @@ int java_classdump(const char *file)
 			}
 		}
 		if (c == NULL) {
-			fprintf(stderr, "Invalid tag '%d'\n", buf[0]);
+			fprintf(stderr, "Invalid tag '%d' at offset 0x%08llx\n",
+				buf[0], (ut64)(ftell (fd)-1));
 			return 0;
 		}
 		NR printf(" %3d %s: ", i+1, c->name);
@@ -580,11 +585,11 @@ int java_classdump(const char *file)
 
 		/* read bytes */
 		switch(c->tag) {
-		case 1: // utf 8 string
+		case 1: // Utf8 string
 			fread(buf, 2, 1, fd);
-			sz = USHORT(buf,0); //(buf[0]<<8)|buf[1];
-			//cp_items[i].len = sz;
-			fread(buf, sz, 1, fd);
+			sz = USHORT(buf, 0);
+			if (sz>0)
+				fread(buf, sz, 1, fd);
 			buf[sz] = '\0';
 			break;
 		default:
@@ -593,14 +598,19 @@ int java_classdump(const char *file)
 
 		memcpy(cp_items[i].bytes, buf, 5);
 
-		R printf("b %d\n", c->len);
-		R printf("f cp.%d @ 0x%04llx\n", i+1, cp_items[i].off);
+		R printf("f cp.%d %d @ 0x%04llx\n", i+1, c->len, cp_items[i].off);
 		/* parse value */
 		switch(c->tag) {
 		case 1:
-			NR printf("%s\n", buf);
+			NR printf("%s (length=%d)\n", buf, sz);
+			//else printf("fs strings && f str.%d @ 0x%08llx\n",
+			//	i+1, (ut64)(cp_items[i].off+3));
 			cp_items[i].value = strdup(buf);
 			break;
+		case 5: // Long
+		case 6: // Double
+			i+=2;
+			break;
 		case 7:
 			NR printf("%d\n", USHORT(buf,0));
 			break;
@@ -643,6 +653,7 @@ int java_classdump(const char *file)
 
 	sz = read_short(fd);
 	NR printf("Fields count: %d\n", sz);
+	else printf("fs header\n");
 	if (sz>0) {
 		for (i=0;i<sz;i++) {
 			fread(buf, 8, 1, fd);
@@ -663,6 +674,7 @@ int java_classdump(const char *file)
 
 	sz = read_short(fd);
 	NR printf("Methods count: %d\n", sz);
+	else printf("fs symbols\n");
 	if (sz>0) {
 		for (i=0;i<sz;i++) {
 			fread(buf, 8, 1, fd);
@@ -737,8 +749,8 @@ int hexpair2bin(const char *arg) // (0A) => 10 || -1 (on error)
 int hexstr2binstr(const char *in, unsigned char *out) // 0A 3B 4E A0
 {
 	const char *ptr;
-	unsigned char  c = '\0';
-	unsigned char  d = '\0';
+	unsigned char c = '\0';
+	unsigned char d = '\0';
 	unsigned int len = 0, j = 0;
 
 	for (ptr = in; ;ptr = ptr + 1) {
@@ -781,9 +793,8 @@ int main(int argc, char **argv)
 	unsigned char buf[1024];
 	char output[128];
 	
-	while ((c = getopt(argc, argv, "ra:d:c:hV")) != -1)
-	{
-		switch( c ) {
+	while ((c = getopt(argc, argv, "ra:d:c:hV")) != -1) {
+		switch (c) {
 		case 'r':
 			radare_output = 1;
 			break;
