@@ -53,7 +53,7 @@
 
 #include "dietline.h"
 
-u64 tmpoff = -1;
+ut64 tmpoff = -1;
 int tmpbsz = -1;
 int std = 0;
 
@@ -199,9 +199,10 @@ int radare_close()
 
 void radare_sync()
 {
-	u64 base, phys;
+	ut64 base;
 	int limit = DEFAULT_BLOCK_SIZE;
 
+#if 0
 	if (config_get_i("cfg.sections")) {
 		base = section_get_vaddr(config.seek);
 		if (base != -1)
@@ -210,8 +211,9 @@ void radare_sync()
 		if (phys != -1)
 			config_set_i("io.paddr", phys);
 	}
+#endif
 
-	if (config.debug||config.unksize)
+	if (config.debug||config.unksize||config.limit==-1LL)
 		return;
 
 	/* enlarge your integer overflows */
@@ -235,14 +237,14 @@ void radare_sync()
 	}
 }
 
-int stripstr_iterate(const unsigned char *buf, int i, int min, int max, int enc, u64 offset, const char *match);
+int stripstr_iterate(const unsigned char *buf, int i, int min, int max, int enc, ut64 offset, const char *match);
 int radare_strsearch(const char *str)
 {
-	u64 i, minlen = 0;
+	ut64 i, minlen = 0;
 	int j, ret;
 	int range_n=0;
-	u64 oseek, seek;
-	u64 size = config.size;
+	ut64 oseek, seek;
+	ut64 size = config.size;
 	int useranges = 0;
 	int min = 3;
 	int max = 0;
@@ -317,16 +319,16 @@ void radare_cmd_foreach(const char *cmd, const char *each)
 	int i=0,j;
 	char ch;
 	char *word = NULL;
-	char *str, *p;
+	char *str, *ostr, *p;
 	struct list_head *pos;
-	u64 oseek, addr;
-	u32 tmpbsz = config.block_size;
+	ut64 oseek, addr;
+	ut32 tmpbsz = config.block_size;
 
 	for(;*each==' ';each=each+1);
 	for(;*cmd==' ';cmd=cmd+1);
 
 	oseek = config.seek;
-	str = strdup(each);
+	ostr = str = strdup(each);
 	radare_controlc();
 
 	/* strip ansi codes */
@@ -378,12 +380,12 @@ void radare_cmd_foreach(const char *cmd, const char *each)
 				addr = get_math(each);
 				str[0]=' ';
 			}
-			if (addr == 0 && each[0]!='0')
-				break;
+			//if (addr == 0 && each[0]!='0')
+			//	break;
 			each = str+1;
 			radare_seek(addr, SEEK_SET);
 			eprintf("\n"); //eprintf("===(%s)at(0x%08llx)\n", cmd, addr);
-			radare_cmd(cmd, 0);
+			radare_cmd_raw(cmd, 0);
 		} while(str != NULL && !config.interrupted);
 		break;
 	case '.':
@@ -418,7 +420,7 @@ void radare_cmd_foreach(const char *cmd, const char *each)
 					addr = get_math(buf);
 					eprintf("0x%08llx\n", addr, cmd);
 					sprintf(cmd2, "%s @ 0x%08llx", cmd, addr);
-					radare_seek(buf, SEEK_SET);
+					radare_seek(addr, SEEK_SET);
 					radare_cmd(cmd2, 0);
 					macro_counter++;
 				}
@@ -453,7 +455,7 @@ void radare_cmd_foreach(const char *cmd, const char *each)
 					config.seek = flag->offset;
 					cons_printf("; @@ 0x%08llx (%s)\n", config.seek, flag->name);
 					//radare_read(0);
-					radare_cmd(cmd,0);
+					radare_cmd_raw(cmd, 0);
 				}
 			} else {
 				/* for all flags in current flagspace */
@@ -473,7 +475,7 @@ void radare_cmd_foreach(const char *cmd, const char *each)
 					//radare_sync(); // XXX MUST BE CALLED 
 						radare_read(0);
 						cons_printf("; @@ 0x%08llx (%s)\n", config.seek, flag->name);
-						radare_cmd(cmd,0);
+						radare_cmd_raw(cmd,0);
 					}
 				}
 	#if 0
@@ -500,7 +502,7 @@ void radare_cmd_foreach(const char *cmd, const char *each)
 
 	free(word);
 	word = NULL;
-	free(str);
+	free(ostr);
 }
 
 void radare_fortunes()
@@ -548,7 +550,7 @@ int radare_cmd_raw(const char *tmp, int log)
 	char *grep = NULL;
 	char *next = NULL;
 	int ret = 0;
-	u32 tmpbsz = -1;
+	ut32 tmpbsz = -1;
 
 	if (strnull(tmp))
 		return 0;
@@ -1072,14 +1074,21 @@ char *radare_cmd_str(const char *cmd)
 	int newlen, fus = cons_flushable;
 	char *grep;
 
-	if (strchr(cmd, '>') ||strchr(cmd, '`')) {
-		eprintf("Warning: Cannot use '>' or '`' in radare_cmd_str\n");
+	if (strchr(cmd, '>')) {
+		int fd = dup(1); // UGLY FIX
+		radare_cmd_raw(cmd, 0);
+		dup2(fd, 1); // UGLY FIX
+		close(fd);
+		return strdup("");
+	}
+	if (strchr(cmd, '`')) {
+		eprintf("Warning: Cannot use '`' in radare_cmd_str\n");
 		return strdup("");
 	}
 	/* backup cons buffer for nested fun */
 	// TODO: rename to cons_push();
 #if MYFIX
-	cbuf = cons_get_buffer();
+	cbuf = (char *)cons_get_buffer();
 //eprintf("%d\n", strlen(cbuf));
 	if (cbuf) cbuf = strdup(cbuf);
 #else
@@ -1097,9 +1106,7 @@ char *radare_cmd_str(const char *cmd)
 		cons_grep(grep+1);
 	}
 	cons_noflush = 1;
-#if __WINDOWS__
 cons_flushable = 0;
-#endif
 	radare_cmd_raw(dcmd, 0);
 #if 0
 #if __WINDOWS__
@@ -1110,8 +1117,8 @@ cons_render();
 	cons_noflush = 0;
 	free (dcmd);
 #if 1
-	buf = cons_get_buffer();
-//eprintf("BUF(%s)\n", buf);
+	buf = (char *)cons_get_buffer();
+//eprintf("BUF(%s)(%s)\n", dcmd, buf);
 	if (buf) {
 		buf = strdup(buf);
 		newlen = cons_grepbuf(buf, strlen(buf));
@@ -1213,7 +1220,7 @@ void radare_nullcmd()
 #endif
 	if (config_get("dbg.dwarf")) {
 		radare_cmd("pR @ eip",0);
-		u64 pc = flag_get_addr("eip");
+		ut64 pc = flag_get_addr("eip");
 		if (pc<config.seek || pc > config.seek+config.block_size)
 			radare_seek(pc, SEEK_SET);
 	}
@@ -1227,7 +1234,7 @@ void radare_nullcmd()
 }
 
 
-int radare_cmd(char *input, int log)
+int radare_cmd(const char *input, int log)
 {
 	int repeat;
 	int i;
@@ -1369,9 +1376,9 @@ int radare_interpret(const char *file)
 
 int radare_move(char *arg)
 {
-	u64 src = config.seek;
-	u64 len =  0;
-	u64 pos = -1;
+	ut64 src = config.seek;
+	ut64 len =  0;
+	ut64 pos = -1;
 	char *str;
 	u8 *buf;
 
@@ -1410,8 +1417,8 @@ void radare_prompt_command()
 
 	if (config_get("cfg.vbsze_enabled")) {
 		struct list_head *pos;
-		u64 uh = config.seek + config_get_i("cfg.vbsize");
-		u64 old = uh;
+		ut64 uh = config.seek + config_get_i("cfg.vbsize");
+		ut64 old = uh;
 		list_for_each(pos, &flags) {
 			flag_t *flag = (flag_t *)list_entry(pos, flag_t, list);
 			if (flag->offset > config.seek && (flag->offset < old))
@@ -1442,19 +1449,25 @@ void radare_prompt_command()
 	monitors_run();
 }
 
+static int check_session (const char *file, const char *session) {
+	char *ptr = strchr (file, '.');
+	if (ptr)
+		return !memcmp(ptr+1, session, strlen(session));
+	return 0;
+}
+
 void monitors_run()
 {
 	char file[1024];
 	char path[1024];
-	int i;
 	FILE *fd;
 	const char *ptr;
 	int tmp;
 	struct dirent *de;
 	DIR *dir;
-	char *session = config_get("cfg.session");
-	u64 oseek = config.seek;
-	u64 obsize= config.block_size;
+	const char *session = config_get("cfg.session");
+	ut64 oseek = config.seek;
+	ut64 obsize= config.block_size;
 	int flush = cons_flushable;
 	cons_flushable = 0;
 
@@ -1476,10 +1489,8 @@ void monitors_run()
 				if (de->d_name[0] != '.' && !strstr(de->d_name, ".txt")) {
 					cons_flush();
 					sprintf(file, "%s/%s", path, de->d_name);
-					if (session) {
-						strcat(file, ".");
-						strcat(file, session);
-					}
+					if (check_session (file, session))
+						continue;
 					fd = fopen(file, "r");
 					if (fd) {
 						strcat(file, ".txt");
@@ -1506,6 +1517,8 @@ void monitors_run()
 						if (_print_fd != 1) // XXX stdout
 							close(_print_fd);
 						fclose(fd);
+					} else {
+						fprintf(stderr, "rsc monitor (cannot open '%s')\n", file);
 					}
 					_print_fd = 1;
 				}
@@ -1639,7 +1652,7 @@ int radare_prompt()
 	return 1;
 }
 
-void radare_set_block_size_i(int sz)
+int radare_set_block_size_i(int sz)
 {
 	if (sz<1) sz = 1;
 
@@ -1655,7 +1668,7 @@ void radare_set_block_size_i(int sz)
 			radare_set_block_size_i(DEFAULT_BLOCK_SIZE);
 		}
 	}
-	radare_read(0);
+	return radare_read(0);
 }
 
 void radare_set_block_size(char *arg)
@@ -1682,7 +1695,7 @@ void radare_set_block_size(char *arg)
 void radare_resize(const char *arg)
 {
 	int fd_mode = O_RDONLY;
-	u64 size  = get_math(arg);
+	ut64 size  = get_math(arg);
 
 	// XXX move this check into a only one function for all write-mode functions
 	// or just define them as write-only. and activate/deactivate them from
@@ -1708,7 +1721,7 @@ void radare_resize(const char *arg)
 	}
 
 	if (arg[0]=='-') {
-		u64 rest;
+		ut64 rest;
 		size = -size; // be positive
 		D eprintf("stripping %lld bytes\n", size);
 		rest = config.size - (config.seek -size);
@@ -1732,7 +1745,7 @@ void radare_resize(const char *arg)
 		D printf("Truncating...\n");
 		ftruncate(config.fd, (off_t)size);
 		close(config.fd);
-		if (config.limit > size)
+		if (config.limit && config.limit > size)
 			config.limit = size;
 		config.size = size;
 	}
@@ -1759,9 +1772,9 @@ int radare_open(int rst)
 	const char *cptr;
 	char buf[4096];
 	struct config_t ocfg;
-	int wm = (int)config_get("file.write");
+	int wm = (int)(size_t)config_get("file.write");
 	int fd_mode = wm?O_RDWR:O_RDONLY;
-	u64 seek_orig = config.seek;
+	ut64 seek_orig = config.seek;
 
 	if (config.file == NULL)
 		return 0;
@@ -1813,6 +1826,9 @@ int radare_open(int rst)
 		}
 	}
 
+	if ((ptr = strstr(config.file, "://")))
+		config.file = ptr + 3;
+
 	/* handles all registered debugger prefixes */
 	// ugly hack? :) a plugin should have a field specifying
 	// if it's for debug or not
@@ -1832,20 +1848,18 @@ int radare_open(int rst)
 	if (config.debug) {
 		config_set("file.write", "true");
 		rdb_init();
-		config.file = strstr(config.file, "://") + 3;
 	}
 
-	config.size = io_lseek(config.fd, (u64)0, SEEK_END);
-	io_lseek(config.fd, (u64)seek_orig, SEEK_SET);
+	config.size = io_lseek(config.fd, (ut64)0, SEEK_END);
+	io_lseek(config.fd, (ut64)seek_orig, SEEK_SET);
 
 	if (config.size == -1 || config.unksize) {
-		config.size  = -1;
+	//	config.size  = -1;
 		config.limit =  0;
 	} else {
 		if (config.size == -1) {
 			eprintf("warning: unknown file size."
 				" Use 'l'imit to define boundaries.\n");
-			config.size = -1;
 			config.limit = 0;
 		} else
 			config.limit = config.size;
@@ -1868,20 +1882,21 @@ int radare_open(int rst)
 	if (ocfg.fd != -1)
 		io_close(ocfg.fd); // close old filedescriptor
 
-	config.zoom.size   = config.size;
+	config_set_i("zoom.to", config.size);
+	config.zoom.to     = config.size;
 	config.zoom.from   = 0;
 	config.zoom.piece  = config.size/config.block_size;
 
 	{
-	char *cmd = config_get("cmd.open");
+	const char *cmd = config_get("cmd.open");
 	if (cmd)
-		radare_cmd(cmd, 0);
+		radare_cmd_raw (cmd, 0);
 	}
 
 	return 0;
 }
 
-int radare_compare_code(u64 off, const u8 *a, int len)
+int radare_compare_code(ut64 off, const u8 *a, int len)
 {
 	u8 *b = alloca(len);
 	radare_read_at(off, b, len);
@@ -1909,12 +1924,11 @@ int radare_compare(unsigned char *f, unsigned char *d, int len)
 	return len-eq;
 }
 
-int radare_compare_hex(u64 addr, unsigned char *f, unsigned char *d, int len)
+int radare_compare_hex(ut64 addr, unsigned char *f, unsigned char *d, int len)
 {
 	int i, eq = 0;
 	char str0[256], str1[256], tmp[32];
 	int rowlen = 16;
-	int row = 0;
 
 	str0[0]='\0';
 	str1[0]='\0';
@@ -1925,12 +1939,6 @@ int radare_compare_hex(u64 addr, unsigned char *f, unsigned char *d, int len)
 			str1[0]='\0';
 		}
 		if (f[i]!=d[i]) {
-#if 0
-			D cons_printf("0x%08llx (byte=%.2d)   %02x '%c'  ->  %02x '%c'\n",
-				config.seek+i, i+1,
-				f[i], (is_printable(f[i]))?f[i]:' ',
-				d[i], (is_printable(d[i]))?d[i]:' ');
-#endif
 			sprintf(tmp, "\x1b[7m%02x\x1b[0m", f[i]);
 			strcat(str0, tmp);
 			sprintf(tmp, "\x1b[7m%02x\x1b[0m", d[i]);
@@ -1950,8 +1958,8 @@ int radare_compare_hex(u64 addr, unsigned char *f, unsigned char *d, int len)
 
 int radare_go()
 {
-	u64 bsize;
-	int i, t = (int)config_get("cfg.verbose");
+	ut64 bsize;
+	int i, t = (int)(size_t)config_get("cfg.verbose");
 
 	radare_controlc_end();
 
@@ -1966,10 +1974,22 @@ int radare_go()
 		}
 	}
 
+	if (config.file == NULL) {
+		help_message(0);
+		return 1;
+	}
+	if (!strcmp(config.file,"-")) {
+		config.file = strdup("malloc://4096");
+		config_set("file.write", "true");
+		config.unksize = 1;
+		config.limit = -1;
+	}
+
 	/* open file */
 	bsize = config.block_size;
 	if (radare_open(0))
 		return 1;
+	config.realfile = r_file_exist(config.file);
 	if (bsize)
 		radare_set_block_size_i(bsize);
 
@@ -1979,6 +1999,7 @@ int radare_go()
 		return 0;
 	}
 
+	env_prepare(NULL);
 	radare_controlc();
 
 	if (!config.noscript) {
@@ -2005,16 +2026,17 @@ int radare_go()
 		radare_fortunes();
 
 	/* load rabin stuff here */
-	//rabin_load();
-	if (config_get("file.id"))
+	config_set("file.type", "unk");
+	if (config.realfile && config_get("file.id"))
 		rabin_id();
 
 	/* flag all syms and strings */
-	//radare_cmd(".!rsc flag $FILE", 0);
+	if (strcmp(config_get("file.type"), "unk"))
 	if (strnull(config_get("file.project"))) {
-		if (config_get("file.flag"))
+		if (config.realfile && config_get("file.flag"))
 			rabin_flag();
 
+		/* if not unknown file type */
 		if (config_get("file.analyze")) {
 			eprintf("> Analyzing code...\n");
 			radare_controlc();
@@ -2028,13 +2050,20 @@ int radare_go()
 				radare_cmd("e asm.profile=default", 0);
 			}
 			eprintf("\n");
-			if (!config.interrupted && config_get("file.analdata")) {
+			if (!config.interrupted && config_get("anal.data")) {
 				eprintf("> Analyzing data...");
 				radare_cmd("b section._data_end-section._data", 0);
 				radare_cmd(".ad* @ section._data", 0);
 				radare_cmd("b 128", 0);
 				eprintf(" done\n");
 			}
+			if (!config.interrupted && config_get("anal.funhdr")) {
+				eprintf("> Analyzing function preludes...\n");
+				radare_cmd("s entrypoint", 0);
+				if (config.debug)
+					radare_cmd("ap 0xffff", 0);
+				else radare_cmd("ap", 0);
+			}
 			radare_cmd_raw("Ci", 0);
 			radare_controlc_end();
 		}
@@ -2056,14 +2085,16 @@ int radare_go()
 			radare_cmd(".!!rsc maps ${DPID}", 0);
 		}
 
-		if (config_get("dbg.sections") && !config_get("file.flag"))
-			//radare_cmd(":.!rsc flag-sections $FILE", 0);
-			radare_cmd(".!!rabin -rS $FILE",0);
+		if (config.realfile) {
+			if (config_get("dbg.sections") && !config_get("file.flag"))
+				//radare_cmd(":.!rsc flag-sections $FILE", 0);
+				radare_cmd(".!!rabin -rS $FILE",0);
 
-		if (config_get("dbg.strings") && !config_get("file.flag")) {
-			//eprintf("Loading strings...press ^C when tired\n");
-			//radare_cmd(".!rsc strings-flag $FILE", 0);
-			radare_cmd(".!!rabin -rz $FILE",0);
+			if (config_get("dbg.strings") && !config_get("file.flag")) {
+				//eprintf("Loading strings...press ^C when tired\n");
+				//radare_cmd(".!rsc strings-flag $FILE", 0);
+				radare_cmd(".!!rabin -rz $FILE",0);
+			}
 		}
 
 		radare_set_block_size_i(100); // 48 bytes only by default in debugger
@@ -2088,7 +2119,7 @@ int radare_go()
 #endif
 		radare_cmd("s eip", 0);
 	} else {
-		//eprintf("> Rebasing strings...\n");
+		/* rebase strings */
 		radare_cmd("fb $${io.vaddr} @@ str. > /dev/null", 0);
 	}
 
@@ -2125,7 +2156,9 @@ int pipe_stdout_to_tmp_file(char *tmpfile, const char *cmd)
 	/* WORKS BUT IT IS UGLY */
 	int fd = make_tmp_file(tmpfile);
 	int std;
+int flood = cons_floodprot;
 int fus = cons_flushable;
+cons_floodprot = 0;
 
 	cons_reset();
 	//cons_flush();
@@ -2158,6 +2191,7 @@ cons_render();
 	}
 
 cons_flushable = fus;
+cons_floodprot = flood;
 	return 1;
 }
 
@@ -2238,10 +2272,10 @@ int radare_seek_search_backward(const char *str)
 	unsigned char kw[1024];
 	int kw_idx = 0;
 	int i, j, kw_len = 0;
-	u64 oseek = config.seek;
-	u64 eseek = config_get_i("search.from");
-	u64 maxbytes = config_get_i("search.limit");
-	u64 bytes = 0;
+	ut64 oseek = config.seek;
+	ut64 eseek = config_get_i("search.from");
+	ut64 maxbytes = config_get_i("search.limit");
+	ut64 bytes = 0;
 
 	switch(str[0]) {
 	case ' ': // string search
@@ -2309,10 +2343,10 @@ int radare_seek_search(const char *str)
 	unsigned char kw[1024];
 	int kw_idx = 0;
 	int i, kw_len = 0;
-	u64 oseek = config.seek;
-	u64 eseek = config.size;
-	u64 maxbytes = config_get_i("search.limit");
-	u64 bytes = 0;
+	ut64 oseek = config.seek;
+	ut64 eseek = config.size;
+	ut64 maxbytes = config_get_i("search.limit");
+	ut64 bytes = 0;
 
 	if (config_get_i("search.to")) {
 		eseek = config_get_i("search.to");
