@@ -98,9 +98,9 @@ int _strnstr(char *from, char *to, int size)
 	return (size!=i);
 }
 
-u64 file_size(const char *str)
+ut64 file_size(const char *str)
 {
-	u64 sz;
+	ut64 sz;
 	FILE *fd = fopen(str, "r");
 	if (fd == NULL)
 		return 0;
@@ -123,7 +123,7 @@ char *slurp(const char *str, int *len)
 	fseek(fd, 0,SEEK_SET);
 	ret = (char *)malloc(sz+1);
 	if (ret == NULL) {
-		*len = 0;
+		if (len) *len = 0;
 		return NULL;
 	}
 	fread(ret, sz, 1, fd);
@@ -175,7 +175,7 @@ int file_dump(const char *file, const u8 *buf, int len)
 	return 0;
 }
 
-void endian_memcpy_e(u8 *dest, u8 *orig, int size, int endian)
+void endian_memcpy_e(u8 *dest, const u8 *orig, int size, int endian)
 {
 	if (endian) {
 		memcpy(dest, orig, size);
@@ -222,10 +222,10 @@ void drop_endian(u8 *dest, u8 *orig, unsigned int size)
 }
 
 
-void endian_memcpy(u8 *dest, u8 *orig, unsigned int size)
+void endian_memcpy(u8 *dest, const u8 *orig, unsigned int size)
 {
 #if RADARE_CORE
-	endian_memcpy_e(dest, orig, size, (int)config_get("cfg.bigendian"));
+	endian_memcpy_e(dest, orig, size, PTRCAST (config_get("cfg.bigendian")));
 #else
 	endian_memcpy_e(dest, orig, size, 0); // lilendian by default
 #endif
@@ -237,7 +237,7 @@ const char *get_tmp_dir()
 	// TODO: Do not enter this function twice!
 #if __WINDOWS__
 	// http://msdn.microsoft.com/en-us/library/aa364992(VS.85).aspx
-	GetTempPath(1023, &tmpdir);
+	GetTempPath(1023, tmpdir);
 #else
 	const char *tmp;
 #if RADARE_CORE
@@ -245,15 +245,27 @@ const char *get_tmp_dir()
 #else
 	tmp = getenv("TMP");
 #endif
-	if (tmp)
-		strcpy(tmpdir, tmp);
-	else
-		strcpy(tmpdir, "/tmp/");
+	if (tmp) strcpy(tmpdir, tmp);
+	else strcpy(tmpdir, "/tmp/");
 #endif
 	return (const char *)&tmpdir;
 }
 
-int strbcpy(char *dst, char *org)
+int r_file_exist(const char *str)
+{
+	struct stat buf;
+	return (stat(str, &buf) == 0);
+
+#if __WRONG_WAY__
+	int fd = open(str, O_RDONLY);
+	if (fd == -1)
+		return 0;
+	close(fd);
+	return 1;
+#endif
+}
+
+int str_cpy(char *dst, const char *org)
 {
 	int i = 0;
 	if (org&&*org)
@@ -276,33 +288,35 @@ int make_tmp_file(char *str)
 	return fd;
 }
 
-void progressbar(int pc)
+void progressbar(int nhit, ut64 addr, ut64 size)
 {
 #if RADARE_CORE
-	int tmp, cols = config_get_i("scr.width");
+	int tmp, cols = config_get_i("scr.width")-20;
 #else
-	int tmp, cols = 78;
+	int tmp, cols = 58;
 #endif
+	int pc = (int)((100*addr)/size);
 
 	(pc<0)?pc=0:(pc>100)?pc=100:0;
-	fprintf(stderr, "\x1b[K  %3d%% [", pc);
+	fprintf(stderr, "\x1b[K 0x%08llx %2d%% [", addr, pc);
 	cols-=15;
 	for(tmp=cols*pc/100;tmp;tmp--) fprintf(stderr,"#");
 	for(tmp=cols-(cols*pc/100);tmp;tmp--) fprintf(stderr,"-");
-	fprintf(stderr, "]\r");
+	fprintf(stderr, "] %d\r", nhit);
 	fflush(stderr);
 }
 
 #if RADARE_CORE
-u64 get_pointer(u64 addr, int size)
+ut64 get_pointer(ut64 addr, int size)
 {
 	int endian = !config_get_i("cfg.bigendian"); // XXX -?
-	u64 ret = 0;
+	ut64 ret = 0;
 	u8 newa8 = 0;
-	u16 newa16 = 0;
-	u32 newa32 = 0;
-	u64 newa64 = 0;
-	u64 sk = config.seek;
+	ut16 newa16 = 0;
+	ut32 newa32 = 0;
+	ut64 newa64 = 0;
+	ut64 sk = config.seek;
+
 	radare_seek(addr, SEEK_SET);
 	switch(size) {
 	case 1:
@@ -329,11 +343,11 @@ u64 get_pointer(u64 addr, int size)
 }
 #endif
 
-/* Converts a string to u64 type. u64 jmp = get_offset("0x123456"); */
-u64 get_offset(const char *orig)
+/* Converts a string to ut64 type. ut64 jmp = get_offset("0x123456"); */
+ut64 get_offset(const char *orig)
 {
 	char *arg;
-	u64 ret = 0;
+	ut64 ret = 0;
 	int i, j;
 #if RADARE_CORE
 	flag_t *flag;
@@ -344,7 +358,7 @@ u64 get_offset(const char *orig)
 
 	while(*orig==' '&&*orig) orig++;
 	arg = alloca(strlen(orig)+32);
-	strbcpy(arg, orig);
+	str_cpy(arg, orig);
 
 	/* single char 'A' */
 	if (arg[0]=='\'' && arg[0+2]=='\'')
@@ -354,14 +368,16 @@ u64 get_offset(const char *orig)
 	for(i=0;arg[i]==' ';i++);
 	for(;arg[i]=='\\';i++); i++;
 
-
 #if RADARE_CORE
+#if 0
+/* DEPRECATED WITH $$ special variable */
 	if (!strcmp(orig, "here"))
 		return config.seek;
+#endif
 
 	if (arg[0]=='.') {
-		u64 real = 0;
-		u64 mask = 0;
+		ut64 real = 0;
+		ut64 mask = 0;
 		for(ptr = arg; ptr[0];ptr = ptr +1) {
 			if (ptr[0]!='.') {
 				mask<<= 4;
@@ -375,8 +391,7 @@ u64 get_offset(const char *orig)
 	}
 
 	ptr = strchr(arg, '[');
-	if (ptr)
-		return get_pointer(get_offset(ptr+1),4);
+	if (ptr) return get_pointer(get_offset(ptr+1),4);
 
 	ret = config_get_i(arg);
 	if (((int)ret) != 0)
@@ -389,7 +404,7 @@ u64 get_offset(const char *orig)
 			ret = io_write_last;
 			break;
 		case '$':
-			arch_aop(config.seek, config.block,&aop);
+			arch_aop (config.seek, config.block, &aop);
 			ret = aop.length;
 			break;
 		case '?':
@@ -439,6 +454,8 @@ u64 get_offset(const char *orig)
 		}
 		return ret;
 	} else {
+		char *ch = strchr (arg, ' ');
+		if (ch) *ch='\0';
 		flag = flag_get(arg);
 		if (flag)
 			return flag->offset; // - config.vaddr;
@@ -450,6 +467,13 @@ u64 get_offset(const char *orig)
 		sscanf(arg, OFF_FMTd, &ret);
 
 		switch(arg[strlen(arg)-1]) {
+		case 'f': // float
+			{
+			float f;
+			sscanf(arg, "%f", &f);
+			memcpy(&ret, &f, sizeof(f));
+			}
+			break;
 		case 'o': // octal
 			sscanf(arg, "%llo", &ret);
 			break;
@@ -475,9 +499,9 @@ u64 get_offset(const char *orig)
 	return ret;
 }
 
-u32 get_offset32(const char *foo)
+ut32 get_offset32(const char *foo)
 {
-	return (u32) get_offset (foo);
+	return (ut32) get_offset (foo);
 }
 
 int set0word(char *str)
@@ -487,7 +511,7 @@ int set0word(char *str)
 	if (str[0]=='\0')
 		return 0;
 	/* chop trailing spaces */
-	while(*str==' ') strbcpy(str, str+1);
+	while(*str==' ') str_cpy(str, str+1);
 	for(i=strlen(str)-1;str[i]==' ';i--) str[i]='\0';
 	/* count words */
 	for(i=1,q=p=str;*p;p+=1) {
@@ -534,11 +558,11 @@ char *mytok(char *ptr, char *delim, char *backup)
 }
 
 static int level = 0; // recursivity counter
-u64 get_math(const char* text)
+ut64 get_math(const char* text)
 {
-	u64 t;
-	u64 new_off = 0;
-	u64 cmp_off = 0;
+	ut64 t;
+	ut64 new_off = 0;
+	ut64 cmp_off = 0;
 	int is_cmp  = 0;
 	int  sign   = 1;
 	char op     = 0;
@@ -561,14 +585,14 @@ u64 get_math(const char* text)
 	for(tmp = txt = strdup(text); txt && txt[0] && txt[1]; txt = txt+1) {
 #if 1
 		if ((txt[0]=='!' && txt[1]=='='))
-			strbcpy(txt+1, txt+2);
+			str_cpy(txt+1, txt+2);
 		else
 #endif
 		if ((txt[0]=='=' && txt[1]=='='))
-			txt[0]==' ';
+			txt[0]=' ';
 		else
 		if ((txt[0]==' '))
-			strbcpy(txt, txt+1);
+			str_cpy(txt, txt+1);
 	}
 	txt = tmp;
 
@@ -671,14 +695,13 @@ u64 get_math(const char* text)
 	else
 	if (is_cmp<0)
 		return ((new_off - cmp_off)==0)?1:0;
-
 	return new_off;
 }
 
 /* int byte = hexpair2bin("A0"); */
 int get_cmp(const char *str0, const char *str1)
 {
-	u64 a,b;
+	ut64 a,b;
 	a = get_math(str0);
 	b = get_math(str1);
 	return (int)(a-b);
@@ -708,6 +731,12 @@ int hexpair2bin(const char *arg) // (0A) => 10 || -1 (on error)
 	unsigned char c = '\0';
 	unsigned char d = '\0';
 	unsigned int  j = 0;
+	int len = 0;
+
+	if (arg && (!arg[0] || !arg[1])) {
+		eprintf("Too short\n");
+		return -1;
+	}
 
 	for (ptr = (unsigned char *)arg; ;ptr = ptr + 1) {
 		if (ptr[0]=='\0'||ptr[0]==' ' || j==2)
@@ -718,7 +747,9 @@ int hexpair2bin(const char *arg) // (0A) => 10 || -1 (on error)
 			return -1;
 		}
 		c |= d;
-		if (j++ == 0) c <<= 4;
+		if (j++ == 0)
+			c <<= 4;
+		len++;
 	}
 
 	return (int)c;
@@ -753,8 +784,8 @@ int hexstr2binstr(const char *in, unsigned char *out) // 0A 3B 4E A0
 
 		d = c;
 		if (ptr[0]=='0' && ptr[1]=='x' ){ //&& c==0) {
-			u64 addr   = get_math(ptr);
-			unsigned int addr32 = (u32) addr;
+			ut64 addr   = get_math(ptr);
+			unsigned int addr32 = (ut32) addr;
 			if (addr & ~0xFFFFFFFF) {
 				// 64 bit fun
 			} else {
@@ -965,7 +996,7 @@ int strccmp(char *dst, char *orig, int ch)
 	return 0;
 }
 
-int strccpy(char *dst, char *orig, int ch)
+int str_ccpy(char *dst, char *orig, int ch)
 {
 	int i;
 	for(i=0;orig[i] && orig[i] != ch; i++)
@@ -984,8 +1015,8 @@ int strccpy(char *dst, char *orig, int ch)
 	(((x) & 0x000000000000ff00LL) << 40)  | \
 	(((x) & 0x00000000000000ffLL) << 56))
 
-u64 htonq(u64 value) {
-	u64 ret  = value;
+ut64 htonq(ut64 value) {
+	ut64 ret  = value;
 #if LIL_ENDIAN
 	endian_memcpy_e((u8*)&ret, (u8*)&value, 8, 0);
 #endif
@@ -1034,7 +1065,6 @@ int str_grep(const char *str, const char *needle)
 }
 
 /* Profiling utilities */
-#define R_ABS(x) ((x<0)?-x:x)
 static int timeval_subtract(struct timeval *result, struct timeval *end, struct timeval *begin)
 {
 	// Perform the carry for the later subtraction by updating Y
@@ -1081,32 +1111,68 @@ const char *resolve_path(const char *str)
 {
 	char buf[1024];
 	char *path = getenv("PATH");
+	char *pwd = getenv("PWD");
 	char *foo;
 	char *bar;
 #if __WINDOWS__
 	return strdup(str);
 #else
-	if (path != NULL) {
-		bar = alloca(strlen(str)+1);
-		strcpy(bar, str);
-		path = strdup(path);
-		foo = path;
-		while(bar) {
-			bar = strchr(foo, ':');
-			if (bar != NULL)
-				bar[0]='\0';
-			snprintf(buf, 1022, "%s/%s", foo, str);
-			//if (strstr(buf, ".exe") || strstr(buf, ".com")) {
-			if (!access(buf, X_OK)) {
-				char *ret = strdup(buf);
-				free (path);
-				return ret;
+	if (path != NULL && pwd != NULL) {
+		if (*str == '.') {
+			snprintf(buf, 1022, "%s/%s", pwd, str);
+			char *ret = strdup(buf);
+			return ret;
+		} else if (*str != '/') {
+			bar = alloca(strlen(str)+1);
+			strcpy(bar, str);
+			path = strdup(path);
+			foo = path;
+			while(bar) {
+				bar = strchr(foo, ':');
+				if (bar != NULL)
+					bar[0]='\0';
+				snprintf(buf, 1022, "%s/%s", foo, str);
+				//if (strstr(buf, ".exe") || strstr(buf, ".com")) {
+				if (!access(buf, X_OK)) {
+					char *ret = strdup(buf);
+					free (path);
+					return ret;
+				}
+				foo = bar + 1;
 			}
-			foo = bar + 1;
+			free (path);
 		}
-		free (path);
 	}
 	return strdup(str);
 #endif
 }
 
+char *r_sys_cmd_str(const char *cmd, const char *input, int *len)
+{
+#if __UNIX__
+	char *b = malloc(1024);
+	FILE *fd = popen(cmd, "r");
+	*len = 0;
+	if (fd == NULL)
+		return NULL;
+	*len = fread(b, 1, 1024, fd);
+	pclose(fd);
+	return b;
+#else
+#warning NO r_sys_cmd_str support for this platform
+	return NULL;
+#endif
+}
+
+static const char *home = NULL;
+const char *get_home_directory()
+{
+	if (home == NULL) {
+#if __WIN32__
+		home = getenv("HOMEPATH");
+#else
+		home = getenv("HOME");
+#endif
+	}
+	return strdup(strget(home));
+}
