@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2006, 2007, 2008
+ * Copyright (C) 2006, 2007, 2008, 2009
  *       pancake <youterm.com>
  *
  * radare is free software; you can redistribute it and/or modify
@@ -49,7 +49,7 @@ struct binding {
 	char *cmd;
 };
 
-static u64 visual_seeks[255];
+static ut64 visual_seeks[255];
 static int visual_seeks_init = 0;
 static int cursorseek=1; /* MUST BE IN SCR.CURSORSEEK */
 static unsigned char *yank_buffer = NULL;
@@ -62,7 +62,7 @@ static int accel=1;
 static int nbds = 0;
 static struct binding *bds = NULL;
 static int inv = 0;
-static u64 mark = 0;
+static ut64 mark = 0;
 #define VMODES 12
 static char modes[VMODES] =
 { FMT_HEXB, FMT_VISUAL, FMT_UDIS, FMT_OCT, FMT_CSTR,
@@ -97,7 +97,6 @@ CMD_DECL(zoom);
 //CMD_DECL(trace);
 CMD_DECL(zoom_reset);
 CMD_DECL(edit_screen_filter);
-CMD_DECL(zoom_reset);
 CMD_DECL(setblocksize);
 CMD_DECL(visual_bind_run);
 
@@ -130,7 +129,6 @@ command_t keystrokes[] = {
 	COMMAND('\0',0, NULL, seek_to_flag)
 };
 
-
 #define TITLE if (config.color) cons_printf("\x1b[36m");
 #define TITLE_END if (config.color) cons_printf("\x1b[0m");
 
@@ -145,7 +143,7 @@ void visual_show_help()
 	cons_printf(
 	" :<cmd>     radare command (vi like)\n"
 	" ;          edit or add comment\n"
-	" \"         change metadata of current cursor seek\n"
+	" \"          change metadata of current cursor seek\n"
 	" ,.         ',' marks an offset, '.' seeks to mark or eip if no mark\n"
 	" m'         m1 -> mark current position as char '1'. '1 will jump to mark 1\n"
 	" g,G        seek to beggining or end of file\n"
@@ -157,7 +155,7 @@ void visual_show_help()
 	" f,F        seek between flag list (f = forward, F = backward)\n"
 	" n,N        seek between hits of the hit0 search\n"
 	" t          visual track/browse flagspaces and flags\n"
-	" e          visual eval configuration variables\n"
+	" e,E        visual eval configuration variables, Edit notes file\n"
 	" C          toggle scr.color\n"
 	" d          convert cursor selected bytes to ascii, code or hex\n"
 //	" b[k][cmd]  binds key 'k' to the specified command\n"
@@ -170,8 +168,10 @@ void visual_show_help()
 	" H,L        scroll left, right by 2 bytes (16 bits).\n"
 	" p,P        switch between hex, bin and string formats\n"
 	" x          show xrefs of the current offset\n"
+	" v          visual code analysis mode (edit/view functions, variables, ...)\n"
 	" w          write hexpair bytes in cursor\n"
-	" q          exits visual mode\n");
+	" q          exits visual mode\n"
+	" z, Z       zoom full file, reset zoom preferences\n");
 	TITLE
 	cons_printf("\nCursor mode:\n");
 	TITLE_END
@@ -205,7 +205,7 @@ CMD_DECL(visual_bind_run)
 {
 	const char *cmd = config_get("cmd.visualbind");
 	if (cmd && cmd[0])
-		radare_cmd_raw(cmd, 0);
+		radare_cmd (cmd, 0);
 }
 
 CMD_DECL(edit_screen_filter)
@@ -214,7 +214,7 @@ CMD_DECL(edit_screen_filter)
 	const char *ed = config_get("cfg.editor");
 	if (!strnull(ed)) {
 		/* TODO: Handle errors, put a different file name for */
-		snprintf(buf, 1023, "%s/.radare/screen-filter.txt", getenv("HOME"));
+		snprintf(buf, 1023, "%s/.radare/screen-filter.txt", get_home_directory());
 		config_set("file.scrfilter", buf);
 		eprintf(":!!%s '%s'\n", ed, buf);
 		{
@@ -266,24 +266,38 @@ CMD_DECL(insert)
 
 CMD_DECL(invert)
 {
-	int inv = config_get_i("cfg.inverse")^1;
-	config_set("cfg.inverse", inv?"true":"false");
+	int inv = (int) config_get_i ("cfg.inverse")^1;
+	config_set ("cfg.inverse", inv?"true":"false");
 	return 0;
 }
 
 CMD_DECL(zoom_reset)
 {
-	config.zoom.from = 0;
-	config.zoom.size = config.size;
-	config.zoom.piece = config.zoom.size/config.block_size;
+	config_set_i ("zoom.from", 0);
+	config_set_i ("zoom.to", config.size);
 	return 0;
 }
 
+static void visual_edit ()
+{
+	int ret;
+	char file[1024];
+	char cmd[1024];
+
+	snprintf (file, sizeof (file), "%s.txt", config_get("file.project"));
+	// TODO: append screen dump to file
+	snprintf (cmd, sizeof (cmd), "%s '%s'", config_get("cfg.editor"), file);
+	if (system (cmd)) {
+		eprintf ("command error (%s)\n", cmd);
+		sleep (2);
+	}
+}
+
 static void visual_convert_bytes(int fmt, int defkey)
 {
 	char argstr[128];
 	int c;
-	u64 off, len;
+	ut64 off, len;
 	if (fmt == -1) {
 		if (defkey) {
 			c = defkey;
@@ -348,36 +362,35 @@ static void visual_convert_bytes(int fmt, int defkey)
 
 CMD_DECL(zoom)
 {
-	if (config.zoom.size == 0)
-		config.zoom.size = config.size;
-
 	CLRSCR();
-	config.zoom.enabled ^= 1;
-	if (config.zoom.enabled) {
-		config.cursor = config.block_size * config.seek/config.size;
-	//	config.cursor_mode = 1;
-		config.zoom.size = config.block_size;
+	if (config.cursor_mode && config.ocursor == -1) {
+		config_set_i("zoom.from", 
+				config.zoom.from + config.cursor * config.zoom.piece);
+		config_set_i("zoom.to", config.zoom.from + config.zoom.piece); 
+		config.seek = config.zoom.from;
+		radare_cmd("pO", 0);
+	} else if (config.cursor_mode) {
+		int cfrom, cto;
+		if (config.ocursor > config.cursor) {
+			cfrom = config.cursor;
+			cto = config.ocursor;
+		} else {
+			cfrom = config.ocursor;
+			cto = config.cursor;
+		}
+		config_set_i("zoom.from", 
+				config.zoom.from + cfrom * config.zoom.piece);
+		config_set_i("zoom.to",
+				config.zoom.from + (cto - cfrom) * config.zoom.piece);
+		config.seek = config.zoom.from;
 		radare_cmd("pO", 0);
 	} else {
-		if (config.cursor_mode && ( config.ocursor != -1)) {
-			config.zoom.from = config.cursor * config.zoom.piece;
-			config.zoom.size = config.cursor-config.ocursor;
-			if (config.zoom.size<0) config.zoom.size *= -1;
-			config.zoom.size *=config.zoom.piece;
-			config.zoom.piece = (config.zoom.size-config.zoom.from)/config.block_size;
-			config.zoom.enabled = 1;
-			config.seek = config.zoom.from;
-			radare_cmd("pO", 0);
-		} else {
-			if (config.cursor_mode)
-				config.seek = config.zoom.from + config.cursor * config.zoom.piece;
-			radare_cmd("x", 0);
-		}
-		/* reset cursor */
-		config.cursor_mode = 0;
-		config.cursor = 0;
-		config.ocursor = -1;
+		config_set_i("zoom.from", 0);
+		config_set_i("zoom.to", config.size);
+		radare_cmd("pO", 0);
 	}
+	config.cursor = 0;
+	config.ocursor = -1;
 	CLRSCR();
 	return 0;
 }
@@ -516,9 +529,9 @@ CMD_DECL(yank_paste)
 	} else {
 		if (config_get("file.write")) {
 			char *ptr = strchr(input, ' ');
-			u64 sz;
+			ut64 sz;
 			int off = 0;
-			u64 old = config.seek;
+			ut64 old = config.seek;
 			sz = get_math(ptr?ptr:input);
 			if (sz == 0)
 				sz = yank_buffer_size;
@@ -543,7 +556,7 @@ CMD_DECL(yank_paste)
 /* deprecated!! */
 CMD_DECL(xrefs_here)
 {
-	u64 addr;
+	ut64 addr;
 	int foo;
 	cons_printf("Select XREF from list:\n");
 	data_xrefs_here(config.seek);
@@ -578,11 +591,15 @@ CMD_DECL(stepu_in_dbg)
 CMD_DECL(step_in_dbg)
 {
 	if (!config.debug) {
-		eprintf("not in debugger\n");
-		cons_any_key();
-	} else
-		radare_cmd("!step", 0);
-	radare_sync();
+		char *str = strdup (config_get("asm.profile"));
+#warning non-debugger code emulation is not integrated correctly
+		/* this is not working properly yet */
+		radare_cmd ("avx 1", 0); // USE THE VM HERE
+		radare_cmd (".av*", 0);
+		radare_cmd("f vm.eip@vm.eip+$$$", 0);
+		config_set ("asm.profile", str);
+	} else radare_cmd ("!step", 0);
+	radare_sync ();
 	//trace_add(get_offset("eip"));
 	return 0;
 }
@@ -592,8 +609,7 @@ CMD_DECL(stepo_in_dbg)
 	if (!config.debug) {
 		eprintf("not in debugger\n");
 		cons_any_key();
-	} else
-		radare_cmd("!stepo", 0);
+	} else radare_cmd("!stepo", 0);
 	radare_sync();
 	return 0;
 }
@@ -601,11 +617,10 @@ CMD_DECL(stepo_in_dbg)
 CMD_DECL(seek_to_flag)
 {
 	flag_t *flg;
-	unsigned char key;
-
-	flag_list("");
-	key = cons_readchar();
+	ut8 key;
 
+	flag_list ("");
+	key = cons_readchar ();
 	if (key>='0' && key<='9') {
 		flg = flag_get_i(key-'0');
 		if (flg) {
@@ -620,7 +635,7 @@ CMD_DECL(seek_to_end)
 {
 	if (config.cursor_mode)
 		config.cursor = config.block_size - 1;
-	else	config.seek = config.size - config.block_size;
+	else config.seek = config.size - config.block_size;
 	return 0;
 }
 
@@ -631,7 +646,6 @@ CMD_DECL(insert_assembly_rsc)
 		cons_any_key();
 		return 1;
 	}
-
 	printf("write assembly (end with ^d):\n");
 	fflush(stdout);
 	cons_set_raw(0);
@@ -680,22 +694,22 @@ CMD_DECL(insert_assembly_hack)
 #if DEBUGGER
 	char buf[16];
 	if (config.debug) {
-		radare_cmd("!hack", 0);
+		radare_cmd ("!hack", 0);
 	} else {
 		/* for non debugger -- all of them must be available !! */
-		printf(" 0 - nop one opcode\n");
-		printf(" 1 - negate jump (jz->jnz , ja->jbe, ..)\n");
-		printf(" 2 - force jmp (only for { 0f, 0x80-0x8f })\n");
-		printf(" 5 - add ret\n");
-		printf(" 6 - add ret with eax=0\n");
-		printf(" 7 - negate zero flag (TODO)\n");
+		printf (" 0 - nop one opcode\n");
+		printf (" 1 - negate jump (jz->jnz , ja->jbe, ..)\n");
+		printf (" 2 - force jmp (only for { 0f, 0x80-0x8f })\n");
+		printf (" 5 - add ret\n");
+		printf (" 6 - add ret with eax=0\n");
+		printf (" 7 - negate zero flag (TODO)\n");
 	}
 	
-	buf[0]='\0';
-	buf[0]=cons_readchar();
-	buf[1]='\0';
-	arch_hack(buf);
-	CLRSCR();
+	buf[0] = '\0';
+	buf[0] = cons_readchar();
+	buf[1] = '\0';
+	arch_hack (buf);
+	CLRSCR ();
 #endif
 	return 0;
 }
@@ -730,7 +744,7 @@ CMD_DECL(insert_hexa_string) // TODO: control file has growed here too!! maybe i
 {
 	char buf[1025];
 	const char *dl_prompt_old = dl_prompt;
-	u64 oseek = config.seek;
+	ut64 oseek = config.seek;
 
 	if (!config_get("file.write")) {
 		eprintf("Not in write mode.\n");
@@ -759,9 +773,7 @@ CMD_DECL(insert_hexa_string) // TODO: control file has growed here too!! maybe i
 			io_write(config.fd, tmp, size);
 			free(tmp);
 			radare_seek(oseek, SEEK_SET);
-		} else {
-			radare_cmd(buf, 0);
-		}
+		} else radare_cmd(buf, 0);
 	}
 	
 	cons_set_raw(1);
@@ -787,7 +799,6 @@ char *get_print_format_name(int j)
 	int i;
 	if (last_print_format == FMT_ZOOM)
 		return zoom_string;
-
 	for(i=0;i<VMODES;i++)
 		if (modes[i] == j)
 			return modestr[i];
@@ -808,15 +819,13 @@ CMD_DECL(rotate_print_format_prev)
 static int keystroke_run(unsigned char key) {
 	command_t cmd;
 	int i = 0;
-
 	for (cmd = keystrokes[0]; cmd.sname ; cmd = keystrokes[i++])
 		if (cmd.sname == key) {
-			(void)cmd.hook(""); // no args
+			cmd.hook (""); // no args
 			if (cmd.options)
-				cons_any_key();
+				cons_any_key ();
 			return 1;
 		}
-
 	return 0;
 }
 
@@ -826,8 +835,7 @@ static void visual_bind_key()
 #if HAVE_LIB_READLINE
 	char *ptr;
 #endif
-	char key;
-	char buf[1024];
+	char key, buf[1024];
 	int i, n;
 
 	printf("Press a key to bind (? for listing): ");
@@ -848,16 +856,12 @@ static void visual_bind_key()
 		}
 	}
 	if (key=='?') {
-		if (nbds == 0) {
-			cons_printf("No user keybindings defined.\n");
-		} else {
+		if (nbds != 0) {
 			cons_printf("Keybindings:");
 			cons_newline();
-			for(i=0;i<nbds;i++) {
-				cons_printf(" key '%c' = \"%s\"", bds[i].key, bds[i].cmd);
-				cons_newline();
-			}
-		}
+			for(i=0;i<nbds;i++)
+				cons_printf(" key '%c' = \"%s\"\n", bds[i].key, bds[i].cmd);
+		} else cons_printf("No user keybindings defined.\n");
 		cons_any_key();
 		return;
 	}
@@ -934,15 +938,15 @@ void visual_draw_screen()
 		strcpy(buf, "<insert assembly> ('tab')");
 		break;
 	default:
-		string_flag_offset(buf, config.seek, -2);
+		string_flag_offset(NULL, buf, config.seek, -2);
 		if (config.cursor!=-1)
-			string_flag_offset(buf2, config.seek+config.cursor, -1);
+			string_flag_offset(NULL, buf2, config.seek+config.cursor, -1);
 	}
 
 #if 0
 	ptr = config_get("scr.seek");
 	if (ptr&&ptr[0]&&last_print_format==FMT_REF) {
-		u64 off = get_math(ptr);
+		ut64 off = get_math(ptr);
 		if (off != 0)
 		radare_seek(off, SEEK_SET);
 	}
@@ -950,9 +954,7 @@ void visual_draw_screen()
 
 	monitors_run();
 
-	/* HUH */
 	cons_clear00();
-	//V cons_printf("\x1b[0m");
 
 	if (inc >config.block_size)
 		inc = config.block_size;
@@ -970,8 +972,9 @@ void visual_draw_screen()
 	cons_printf("[ 0x%llx (bs=%d mark=0x%llx) %s %s] %s            \n",
 		(config.seek+config.vaddr),
 		(unsigned int)config.block_size,
-		mark, get_print_format_name(last_print_format),
-		(inv)?"inv ":"", buf);
+		mark, strget(get_print_format_name(last_print_format)),
+		strget((char *)(size_t)inv),
+		strget(buf));
 	/* Spaguetti monster lives here ! */
 	ptr = config_get("cmd.vprompt");
 	if (ptr&&ptr[0]) {
@@ -1049,7 +1052,7 @@ static void check_accel(int foo)
 
 static char *visual_prompt_entry(const char *title, char *buf, int len)
 {
-	char *o =dl_prompt;
+	const char *o = dl_prompt;
 	dl_prompt = title;
 	cons_set_raw(1);
 	cons_fgets(buf, len, 0, NULL);
@@ -1058,23 +1061,23 @@ static char *visual_prompt_entry(const char *title, char *buf, int len)
 	return buf;
 }
 
-void visual_change_metadata(u64 seek)
+void visual_change_metadata(ut64 seek)
 {
-	u64 off;
+	ut64 off;
 	char buf[128];
 	char buf2[128];
 	int key;
 	eprintf("\nCurrent seek: 0x%08llx\n\n", seek);
-	eprintf("q quit this menu\n");
-	eprintf("f Analyze code here to define a function\n");
-	eprintf("F Remove function definition\n");
-	eprintf("; Add comment\n");
-	eprintf("- Remove comment\n");
-	eprintf("c Convert to code (remove previous type conversions)\n");
-	eprintf("d Convert type of full block or selected bytes to hex data\n");
-	eprintf("x Add xref\n");
-	eprintf("X Remove xref\n");
-	eprintf("\n=> Press key: \n");
+	eprintf("q quit this menu\n"
+		"f Analyze code here to define a function\n"
+		"F Remove function definition\n"
+		"; Add comment\n"
+		"- Remove comment\n"
+		"c Convert to code (remove previous type conversions)\n"
+		"d Convert type of full block or selected bytes to hex data\n"
+		"x Add xref\n"
+		"X Remove xref\n"
+		"\n=> Press key: \n");
 	key = cons_readchar();
 	switch(key) {
 	case 'q':
@@ -1101,16 +1104,10 @@ void visual_change_metadata(u64 seek)
 		radare_cmd(buf, 0);
 		break;
 	case 'x':
-		{
-		int type = 0;
 		visual_prompt_entry("type (code/data): ", buf2, sizeof(buf2));
-		switch(buf2[0]) {
-		case 'c': type = 0; break;
-		case 'd': type = 1; break;
-		}
+		if (buf2[0] == 'd') key = 1; else key = 0;
 		off = get_math(visual_prompt_entry("xref addr: ", buf, sizeof(buf)));
-		data_xrefs_add(seek, off, type);
-		}
+		data_xrefs_add(seek, off, key);
 		break;
 	case 'X':
 		off = get_math(visual_prompt_entry("xref addr: ", buf, sizeof(buf)));
@@ -1121,75 +1118,75 @@ void visual_change_metadata(u64 seek)
 
 void visual_f(int f)
 {
-	u64 addr;
+	ut64 addr;
 	struct bp_t *bp;
 	char line[128];
 
-	switch(f) {
-		case 1:
-			cons_clear();
-			radare_cmd("!help", 0);
-			cons_any_key();
-			cons_clear();
-			break;
+	switch (f) {
+	case 1:
+		cons_clear();
+		radare_cmd("!help", 0);
+		cons_any_key();
+		cons_clear();
+		break;
 #if DEBUGGER
-		case 2:
-			addr = config.seek + (config.cursor_mode?config.acursor:0);
-			bp = debug_bp_get(addr);
-			if (bp) {
-				sprintf(line, "!bp -0x%08x", (unsigned int)addr);
-				radare_cmd(line,0);
-				//debug_rm_bp(addr, 0);
-			} else {
-				sprintf(line, "!bp 0x%08x", (unsigned int)addr);
-				radare_cmd(line,0);
-			}
-			if (!debug_bp_get(addr))
-				flag_clear_by_addr(addr);
-			cons_clear();
-			break;
-		case 3:
-			cons_set_raw(0);
-			printf("Watchpoint at: ");
-			strcpy(line, "!drw ");
-			fgets(line+5, sizeof(line), stdin);
-			line[strlen(line)-1]='\0';
+	case 2:
+		addr = config.seek + (config.cursor_mode?config.acursor:0);
+		bp = debug_bp_get(addr);
+		if (bp) {
+			sprintf(line, "!bp -0x%08x", (unsigned int)addr);
 			radare_cmd(line,0);
-			cons_set_raw(1);
-			cons_any_key();
+			//debug_rm_bp(addr, 0);
+		} else {
+			sprintf(line, "!bp 0x%08x", (unsigned int)addr);
+			radare_cmd(line,0);
+		}
+		if (!debug_bp_get(addr))
+			flag_clear_by_addr(addr);
+		cons_clear();
+		break;
+	case 3:
+		cons_set_raw(0);
+		printf("Watchpoint at: ");
+		strcpy(line, "!drw ");
+		fgets(line+5, sizeof(line), stdin);
+		line[strlen(line)-1]='\0';
+		radare_cmd(line,0);
+		cons_set_raw(1);
+		cons_any_key();
+		cons_clear();
+		break;
+	case 4:
+		if (config.cursor_mode) {
+			sprintf(line, "!bp 0x%08llx", config.seek+config.cursor);
+			radare_cmd_raw(line, 0);
+			radare_cmd_raw("!cont", 0);
+			sprintf(line, "!bp -0x%08llx", config.seek+config.cursor);
+			radare_cmd_raw(line, 0);
+		} else {
+			radare_cmd("!contuh", 0);
 			cons_clear();
-			break;
-		case 4:
-			if (config.cursor_mode) {
-				sprintf(line, "!bp 0x%08llx", config.seek+config.cursor);
-				radare_cmd_raw(line, 0);
-				radare_cmd_raw("!cont", 0);
-				sprintf(line, "!bp -0x%08llx", config.seek+config.cursor);
-				radare_cmd_raw(line, 0);
-			} else {
-				radare_cmd("!contuh", 0);
-				cons_clear();
-			}
-			break;
-		case 5:
-			arch_jmp(config.seek + (config.cursor_mode?config.acursor:0));
-			break;
+		}
+		break;
+	case 5:
+		arch_jmp(config.seek + (config.cursor_mode?config.acursor:0));
+		break;
 #endif
-		case 6:
-			radare_cmd("!contsc", 0);
-			break;
-		case 7:
-			if (config_get("trace.libs")) {
-				//CMD_NAME(step)(NULL);
-				debug_step(1);
-			} else {
-				CMD_NAME(stepu_in_dbg)(NULL);
-			}
-			break;
-		case 8:
-			if (config.debug)
-				CMD_NAME(stepo_in_dbg)(NULL);
-			break;
+	case 6:
+		radare_cmd("!contsc", 0);
+		break;
+	case 7:
+		if (config_get("trace.libs")) {
+			//CMD_NAME(step)(NULL);
+			debug_step(1);
+		} else {
+			CMD_NAME(stepu_in_dbg)(NULL);
+		}
+		break;
+	case 8:
+		if (config.debug)
+			CMD_NAME(stepo_in_dbg)(NULL);
+		break;
 	}
 }
 
@@ -1198,12 +1195,12 @@ CMD_DECL(visual)
 	unsigned char key;
 	int i, lpf, ret;
 	int nibble;
+	u8 byte;
 	char line[1024];
 	char tmpbuf[512];
 #if HAVE_LIB_READLINE
 	char *ptr, *optr;
 #endif
-
 	switch(input[0]) {
 	case 'g':
 	case 'G':
@@ -1215,17 +1212,9 @@ CMD_DECL(visual)
 	cons_get_real_columns();
 	config_set_i("scr.width", config.width);
 	config_set_i("scr.height", config.height);
-
 	nibble = 1; // high first
-
 	undo_push();
 
-#if 0
-	config.height = config_get_i("scr.height");
-	if (config.height<1)
-		config_set_i("scr.height", 24);
-#endif
-
 	if (bds == NULL)
 		bds = (struct binding *)malloc(sizeof(struct binding));
 
@@ -1239,7 +1228,7 @@ CMD_DECL(visual)
 	cons_set_raw(1);
 	cons_flushable = 1;
 	cons_skipxy(0,0);
-	while(1) {
+	while (1) {
 		const char *scrseek = config_get("scr.seek");
 		if (inc<1) inc = 1;
 		dec = inc;
@@ -1253,9 +1242,9 @@ CMD_DECL(visual)
 		visual_draw_screen();
 		cons_flushit();
 
-			// XXX 
-		if (config.debug&&last_print_format == FMT_VISUAL&&scrseek&&scrseek[0]) {
-			u64 off = get_math(scrseek);
+		// XXX 
+		if (config.debug&&last_print_format==FMT_VISUAL&&scrseek&&scrseek[0]) {
+			ut64 off = get_math(scrseek);
 			if ((off < config.seek) || ((config.seek+config.block_size) < off)) {
 				radare_seek(off, SEEK_SET);
 				continue;
@@ -1264,33 +1253,15 @@ CMD_DECL(visual)
 
 	__go_read_a_key:
 		/* user input */
-		key = cons_readchar();
+		if (input[0]) {
+			key = input[0];
+			strcpy(input, input+1);
+		} else key = cons_readchar();
 
 		/* insert mode . 'i' key */
-		switch(config.insert_mode) {
+		switch (config.insert_mode) {
 		case 1:
 			key = cons_get_arrow(key); // get ESC+char, return 'hjkl' char
-#if 0
-			if (key==0x1b) {
-				key = cons_readchar();
-				if (key==0x5b) {
-					// TODO: must also work in interactive visual write ascii mode
-					key = cons_readchar();
-					switch(key) {
-					case 0x35: key='K'; break; // re.pag
-					case 0x36: key='J'; break; // av.pag
-					case 0x41: key='k'; break; // up
-					case 0x42: key='j'; break; // down
-					case 0x43: key='l'; break; // right
-					case 0x44: key='h'; break; // left
-					case 0x3b:
-						break;
-					default:
-						key = 0;
-					}
-				}
-			}
-#endif
 			switch(key) {
 			case 9: // TAB
 				if (last_print_format == FMT_DISAS
@@ -1590,8 +1561,8 @@ CMD_DECL(visual)
 			}
 		}
 
-		if (inc<1)inc=1; // XXX dup check?
-		if (repeat==1)repeat=0;
+		if (inc<1) inc=1; // XXX dupped check?
+		if (repeat==1) repeat=0;
 		//else repeat-=1;
 		repeat = 1;
 		for (i=0;i<repeat;i++)
@@ -1608,7 +1579,7 @@ CMD_DECL(visual)
 			cons_set_raw(0);
 			lpf = last_print_format;
 			config.visual=0;
-		cons_flushable = 0;
+			cons_flushable = 0;
 #if HAVE_LIB_READLINE
 			ptr = readline(VISUAL_PROMPT);
 			line[0]='\0';
@@ -1617,7 +1588,6 @@ CMD_DECL(visual)
 			//strcpy(line, ptr);
 			//ptr = line;
 			optr = ptr;
-			
 			tmpbuf[0]='\0';
 			do {
 				if (ptr[strlen(ptr)-1]=='\\') {
@@ -1638,23 +1608,24 @@ CMD_DECL(visual)
 				ret = cons_fgets(ptr, 128, 0, NULL);
 			} while(ret);
 			free(optr);
-			radare_cmd(line, 1);
 #else
 			line[0]='\0';
 			dl_prompt = ":> ";
 			if (cons_fgets(line, 1000, 0, NULL) <0)
 				line[0]='\0';
 			//line[strlen(line)-1]='\0';
-			radare_cmd(line, 1);
 #endif
-		cons_flushable = 1;
-		cons_flush();
+			if (strchr(line, '>') || strchr(line, '|') || strchr(line,'`'))
+				eprintf("Cannot execute pipes in visual mode\n");
+			else radare_cmd(line, 1);
+			cons_flushable = 1;
+			cons_flush();
 			config.visual=1;
 			last_print_format = lpf;
 			cons_set_raw(1);
 			if (line[0])
 				cons_any_key();
-			cons_gotoxy(0,0);
+			cons_gotoxy(0, 0);
 			cons_clear();
 			continue;
 		case '"':
@@ -1677,7 +1648,7 @@ CMD_DECL(visual)
 				config.cursor = 0;
 			} else {
 				if (mark==0) {
-					u64 u = get_offset("eip");	
+					ut64 u = get_offset("eip");	
 					if (u!=0) {
 						undo_push();
 						radare_seek(u, SEEK_SET);
@@ -1694,31 +1665,22 @@ CMD_DECL(visual)
 			break;
 #endif
 		case '\'':
-			{
-			u8 buf = (u8)cons_readchar();
-			if (visual_seeks_init && visual_seeks[buf] != 0xFFFFFFFF)
+			byte = (u8)cons_readchar();
+			if (visual_seeks_init && visual_seeks[byte] != U64_MAX) {
 				undo_push();
-				radare_seek(visual_seeks[buf], SEEK_SET);
+				radare_seek(visual_seeks[byte], SEEK_SET);
 			}
 			break;
 		case 'm':
-			{
-			u8 buf = (u8)cons_readchar();
+			byte = (u8)cons_readchar();
 			if (visual_seeks_init == 0) {
 				int i;
 				for (i=0;i<255;i++)
-					visual_seeks[i] = 0xFFFFFFFF;
+					visual_seeks[i] = U64_MAX;
 				visual_seeks_init = 1;
 			}
-			visual_seeks[buf] = config.seek;
-			}
+			visual_seeks[byte] = config.seek;
 			break;
-#if 0
-			printf("\nrfile magic:\n\n");
-			radare_dump_and_process( DUMP_MAGIC, config.block_size);
-			cons_any_key();
-			break;
-#endif
 		case 'd':
 			visual_convert_bytes(-1, 0);
 			break;
@@ -1772,8 +1734,7 @@ CMD_DECL(visual)
 				}
 				if (config.cursor >= config.block_size)
 					config.cursor = config.block_size - 1;
-			} else
-				config.seek += 2;
+			} else config.seek += 2;
 			break;
 		case 0xd:
 			if (config.cursor_mode) {
@@ -1898,14 +1859,20 @@ CMD_DECL(visual)
 					cons_clear(); }
 			}
 			break;
+		case 'E':
+			visual_edit ();
+			break;
 		case 'e':
 			config_visual_menu();
 			break;
 		case 't':
 			config.scrdelta += 20;
-			flags_visual_menu();
+			flags_visual_menu ();
 			config.scrdelta -= 20;
 			break;
+		case 'v':
+			var_visual_menu ();
+			break;
 		case '<':
 			// fold
 			if (config.cursor_mode) {
@@ -1915,12 +1882,10 @@ CMD_DECL(visual)
 					// unexpand function or close folder
 					int type = data_type_range(config.seek+config.cursor);
 					if (type == -1 || type == DATA_FOLD_O) {
-						data_set((u64)(config.seek+config.cursor), DATA_FOLD_C);
+						data_set((ut64)(config.seek+config.cursor), DATA_FOLD_C);
 						cons_clear();
 					}
-				} else {
-					visual_convert_bytes(DATA_FOLD_C, 0);
-				}
+				} else visual_convert_bytes(DATA_FOLD_C, 0);
 			} else {
 				config.seek -= config.block_size;
 				if (config.seek % config.block_size)
@@ -1930,17 +1895,13 @@ CMD_DECL(visual)
 		case '>':
 			if (config.cursor_mode) {
 				int type = data_type_range(config.seek+config.cursor);
-				if (type == DATA_FOLD_C) {
+				if (type == DATA_FOLD_C)
 					data_set(config.seek+config.cursor, DATA_FOLD_O);
-				}
 				cons_clear();
-				
 				// unfold or expand
 				// check if current cursor position is jump -> open it
 				// check if current cursor position is a folder -> open it
-			} else {
-				config.seek += config.block_size - (config.seek % config.block_size);
-			}
+			} else config.seek += config.block_size - (config.seek % config.block_size);
 			break;
 		case 'H':
 			if (config.cursor_mode) {
@@ -1954,16 +1915,14 @@ CMD_DECL(visual)
 					config.ocursor = config.cursor+1;
 				if (IS_LTZ(config.cursor))
 					config.cursor =0;
-			} else
-				config.seek -= 2;
+			} else config.seek -= 2;
 			break;
 		case 'l':
 			if (config.cursor_mode) {
 				if (++config.cursor>=config.block_size)
 					config.cursor = config.block_size - 1;
 				config.ocursor = -1;
-			} else
-				config.seek++;
+			} else config.seek++;
 			break;
 		case '(': cons_skipxy(0,-1); break;
 		case ')': cons_skipxy(0,1); break;
@@ -2004,12 +1963,12 @@ CMD_DECL(visual)
 				if (config.cursor>=0 && config.cursor< config.block_size) {
 					c = config.cursor;
 					ch = config.block[config.cursor];
-					sprintf(buf, "wx %02x @ 0x%llx", (unsigned char)(--ch), config.seek); //+config.cursor);
+					sprintf(buf, "wx %02x @ 0x%llx", (ut8)(--ch), config.seek);
 					radare_cmd(buf, 0);
 					config.cursor = c;
 					config.ocursor = -1;
 				}
-			} else radare_set_block_size_i(config.block_size-1);
+			} else radare_set_block_size_i (config.block_size-1);
 			cons_clear();
 			break;
 		case '!':
@@ -2032,14 +1991,12 @@ CMD_DECL(visual)
 			goto __visual_exit;
 		default:
 			if (!keystroke_run(key)) {
-				int i;
 				for(i=0;i<nbds;i++)
 					if (bds[i].key == key)
 						radare_cmd(bds[i].cmd, 0);
 				goto __go_read_a_key;
 			}
 		}
-
 		if (config.seek<0)
 			config.seek = 0;
 		do_repeat = 0;
